@@ -1,12 +1,14 @@
 ﻿/// <reference path="jquery-1.7.1.js" />
 
 var userid;
+var username;
 var cid;
 var sessionID;
 var auth;
 var itemid;
 var templid;
 var logindata;
+var settings;
 var attachlist = [];
 var uploadlist = [];
 var savedata;
@@ -36,6 +38,15 @@ $().ready(function () {// Once ready, load templates.
             $('head').append('<link rel="stylesheet" type="text/css" href="./templates/' + templid + '-style.css">');
         }
     }
+    itemid = getURLParameter("email");
+    i18n.init(option, function(t) {
+        $('.view').i18n();
+        if (root !== undefined) view('login');
+        if (itemid === null) {
+            $('#loginHeader').text(i18n.t('Please provide your credentials'));
+        }
+    });
+
     $.get('./l10n/languages', function (languages) {
         var selectedLanguage = getURLParameter('lang');
         if (selectedLanguage === null) {
@@ -53,6 +64,8 @@ $().ready(function () {// Once ready, load templates.
         var user = getURLParameter('user');
         if (user !== null) {
             $('#Username').val(user);
+            $('#Username').hide();
+            $('#loginHeader').text(i18n.t('Please provide your encryption password'));
             $('#Password').focus();
         }
     }, 'json');
@@ -63,10 +76,10 @@ $().ready(function () {// Once ready, load templates.
 
 function loadLanguage (ev) {
     var url = document.URL;
-    if (url.indexOf("&lang") > 0) {
-        url = url.substring (0, url.indexOf("&lang"));
+    if (url.indexOf('&lang=') > 0) {
+        url = url.substring (0, url.indexOf('&lang='));
     }
-    url = url + "&lang=" + $(ev.target).attr("language");
+    url = url + (url.indexOf('?') > 0 ? '': '?') + '&lang=' + $(ev.target).attr('language');
     window.location.replace(url);
 }
 
@@ -85,7 +98,7 @@ function getTemplate (templid, template, locationID) {
 
 // Provides login to oxguard backend.  Authentication key is returned and stored
 function login() {
-    var username = $('#Username').val();
+    username = $('#Username').val();
     var password = $('#Password').val();
     if(typeof String.prototype.trim === 'function') {// Trim not supported in IE
         password = $('#Password').val().trim();
@@ -95,13 +108,22 @@ function login() {
             username : username,
             password : password
     }
-    $.post(root + "oxguard/login?action=guest", JSON.stringify(logindata), function (data) {
-        log("loging data: ", data);
+    var url = 'oxguard/login?action=guest';
+    // If this is a getkey login, then we need a different auth
+    var action = getURLParameter('action');
+    if (action === 'getkey') {
+        logindata.salt = getURLParameter('token');
+        logindata.userid = getURLParameter('userid');
+        logindata.cid = getURLParameter('cid');
+        url = 'oxguard/retrieve?action=login';
+    }
+    $.post(root + url, JSON.stringify(logindata), function (data) {
+        log('loging data: ', data);
         if (data === null) {
             alert('error');
             return;
         }
-        if (data.auth === "Password Needed") {
+        if (data.auth === 'Password Needed') {
             $('#login').hide();
             $('#NewPassword').show();
             userid = data.userid;
@@ -110,18 +132,17 @@ function login() {
             auth = data.auth;
             return;
         }
-        if (data.auth === "Bad Password" || data.auth === "No Key") {
+        if (data.auth === 'Bad Password' || data.auth === 'No Key') {
             //#. Bad username or Password at login
-            $('#ErrorBox').html(i18n.t("Bad username / Password"));
+            $('#ErrorBox').html(i18n.t('Bad username / Password'));
             $('#ErrorBox').show();
-            $('#Username').focus();
-            $('#Username').val('');
-            $('#Password').val('');
+            $('#Username').show();
+            $('#Password').val('').focus();
             $('#authorizing').hide();
             return;
         }
-        if (data.auth === "lockout") {
-            $('#ErrorBox').html(i18n.t("Account locked out try later"));
+        if (data.auth === 'lockout') {
+            $('#ErrorBox').html(i18n.t('Account locked out try later'));
             $('#ErrorBox').show();
             $('#Username').hide();
             $('#Password').hide();
@@ -129,11 +150,17 @@ function login() {
             $('#authorizing').hide();
             return;
         }
-        if (data.auth.length > 20) {
-            okLogin(data);
+        if (data.auth === 'not found') {
+            $('#ErrorBox').html(i18n.t('Account not found'));
+            $('#ErrorBox').show();
+            $('#authorizing').hide();
             return;
         }
-        if (data.auth === "PIN") {
+        if (data.auth.length > 20) {
+            okLogin(data, action);
+            return;
+        }
+        if (data.auth === 'PIN') {
             $('#authorizing').hide();
             $('#pin').show();
             $('#pinbox').focus();
@@ -147,17 +174,28 @@ function login() {
     });
 }
 
-function okLogin(data) {
+function okLogin(data, action) {
     userid = data.userid;
     cid = data.cid;
     sessionID = data.sessionID;
     auth = data.auth;
-    var extrapass = getURLParameter("extrapass");
-    if (extrapass !== null) extrapass = extrapass.trim();
-    if (extrapass === 'true') {
-        getExtraPass(data);
+    if (action === 'getkey') {
+        rKeys(data);
+        return;
+    }
+    settings = data.settings;
+    itemid = getURLParameter("email");
+    if (itemid === null) {
+        $('.back').remove(); // remove back button if PGPManage only option
+        view('pgpManage');
     } else {
-        openreader(data, "");
+        var extrapass = getURLParameter("extrapass");
+        if (extrapass !== null) extrapass = extrapass.trim();
+        if (extrapass === 'true') {
+            getExtraPass(data);
+        } else {
+            openreader(data, "");
+        }
     }
     logindata = null;
 }
@@ -334,6 +372,9 @@ function openreader(ldata, extra) {
     var url = root + 'oxguard/guest?action=getmail&session=' + sessionID + '&auth=' + encodeURIComponent(auth);
     $.post(url, JSON.stringify(json), function (data) {
         $('#Reader').show();
+        if (typeof data === "string") {
+            data = JSON.parse(data);
+        }
         log("reader data: ", data);
         if (data === null) {
             fail();
@@ -384,7 +425,6 @@ function openreader(ldata, extra) {
                 if((navigator.userAgent.match(/iPhone/i)) || (navigator.userAgent.match(/iPod/i))) {
                     $('.passchange').css('height', '100%').css('width', '100%').css('margin-top', '0%')
                     .bind('touchstart', function (e) {
-                        console.log(e.target.type);
                         if (e.target.type !== 'password' && e.target.type !== 'text') {
                             document.activeElement.blur();
                         }
@@ -513,7 +553,13 @@ function getCC(data) {
 }
 
 function getFrom(data) {
-    return (data.from[0][0] + surround(data.from[0][1]));
+    try {
+        return (data.from[0][0] + surround(data.from[0][1]));
+    } catch (e) {
+        console.log(e);
+        return ('');
+    }
+    
 }
 
 function getSubject(data) {
@@ -521,13 +567,14 @@ function getSubject(data) {
 }
 
 function prepHTML (data) {
+    if (data === undefined) return ('');
     return $('<div/>').text(data).html();
 }
 
 function getSent(data) {
-  //  var sent = new Date(Date.parse(data.sent));
-  //  return (sent.toLocaleString());
-    return(data.sent);
+    var sent = new Date(Date.parse(data.sent));
+    return (sent.toLocaleDateString() + " " + sent.toLocaleTimeString());
+   // return(data.sent);
 }
 
 function getContent(data) {
@@ -569,10 +616,11 @@ function getAttachments(data, item, auth) {
             (data.extrapass ? ('&encrextrapass=' + encodeURIComponent(data.extrapass)): "") +
             '&lang=' + language +
             '&attname=' + encodeURIComponent(attach.filename);
-            var link = root + 'oxguard/guest?action=getattach' + params;
+            var name = attach.filename.replace('.grd2','').replace('.grd','');
+            var link = root + 'oxguard/guest/' + encodeURIComponent(name) + '?action=getattach' + params;
             var atlink = $('<a>', {
                 "class": "attachment-link",
-                text: attach.filename.replace('.grd2','').replace('.grd',''),
+                text: name,
                 "link": link,
                 "href" : "#"
             }).css({
@@ -614,13 +662,16 @@ function getReplyData (data) {
             replyAll.push(data.to[i]);
         }
     }
-    for (var j = 0; j < data.cc.length; j++) {
-        if (data.cc[j][1] === thisEmail) {
-            from = data.cc[j];
-        } else {
-            replyAll.push(data.cc[j]);
+    if (data.cc !== undefined) {
+        for (var j = 0; j < data.cc.length; j++) {
+            if (data.cc[j][1] === thisEmail) {
+                from = data.cc[j];
+            } else {
+                replyAll.push(data.cc[j]);
+            }
         }
     }
+
     return {
         replyTo : replyTo,
         replyAll : replyAll,
@@ -763,17 +814,17 @@ function sendreply(data, all) {
 }
 
 function reply(data) {
-    initReply(data);
+    initReply(data, false);
 	$('#rto').html('<span>' + prepHTML(getFrom(data)) + '</span>').show();
 }
 
 function replyall(data) {
-    initReply (data);
+    initReply (data, true);
     var replydata = getReplyData (data);
     $('#rto').html('<span>' + prepHTML(getReplyAllTo(replydata)) + '</span>').show();
 }
 
-function initReply (data) {
+function initReply (data, replyall) {
     $('[name="format"][value="html"]').prop("checked", true);
     $('.plaintexteditor').val('');
     attachlist = [];
@@ -785,7 +836,7 @@ function initReply (data) {
     tinyMCE.activeEditor.focus();
     view('replydiv');
     $('#SendReply').unbind().click(function() {
-        sendreply(data, true);
+        sendreply(data, replyall);
     });
     $('.plaintexteditor').hide();
     $('.mce-tinymce').show();
@@ -876,7 +927,7 @@ function createJson(to, from, subject, content, origMail, format) {
 
 function forgotPass() {
     var email = $('#EmailAddr').val();
-    if (email.indexOf("@") > 1) {
+    if (email.indexOf("@") >= 1) {
         $('#wait').show();
         var url = root + 'oxguard/guest?action=getquestion&email=' + email;
         $.get(url, function (data) {
@@ -919,7 +970,7 @@ function forgotPass2() {
     var answer = $('#forgotAnswer').val();
     $('#ForgotError2').hide();
     $('#wait2').show();
-    var url = root + 'oxguard/guest?action=resetpassword&email=' + email + '&answer=' + answer + '&language=' + language;
+    var url = root + 'oxguard/guest?action=resetpassword&email=' + email + '&answer=' + answer + '&language=' + language + '&templid=' + templid;
     $.get(url, function (data) {
         log("forgot2", data);
         $('#wait2').hide();
@@ -956,6 +1007,10 @@ function view (element) {
 
 }
 
+function back () {
+    view('Reader');
+}
+
 // Convert HTML email to plaintext
 function convert (html, start) {
     try {
@@ -1050,7 +1105,7 @@ function uploadEmail() {
     $('#uploadProgress').attr('aria-valuenow', 0)
     .css('width', '0%');
     $('#uploading').show();
-    var url = root + 'oxguard/guest?action=upload' +
+    var url = root + 'oxguard/guest?action=upload' + '&itemid=' + itemid +
     '&session=' + sessionID + '&userid=' + userid + '&cid=' + cid + "&auth=" + encodeURIComponent(auth);
     $.ajax({
         url: url,
@@ -1098,6 +1153,138 @@ function uploadEmail() {
     });
 }
 
+// PGP Management
+
+function pgpManage() {
+    view('pgpManage');
+    updateCheckBox();
+}
+
+function downloadPGPPublic () {
+        var params = '&id=' + userid + '&cid=' + cid;
+        window.open(root + '/oxguard/pgpmail/public.asc?action=getkeybyid' + params, '_blank');
+}
+
+function downloadPGPPrivate () {
+    var params = '&id=' + userid + '&cid=' + cid + '&auth=' + encodeURIComponent(auth) + '&guest=true';
+    var link = root + '/oxguard/pgp/private.asc?action=getprivkeybyid' + params;
+    window.open(link, '_blank');
+}
+
+function updateCheckBox () {
+    if (settings.inline) {
+        $('#inline').prop('checked', true);
+    } else $('#inline').prop('checked', false);
+    if (settings.direct) {
+        $('#direct').prop('checked', true);
+    } else $('#direct').prop('checked', false);
+}
+
+function updateSettings () {
+    var inline = $('#inline').is('::checked');
+    var direct = $('#direct').is('::checked');
+    var data = {
+            inline : inline,
+            direct : direct
+    };
+    settings.inline = inline;
+    settings.direct = direct;
+    
+    var json = {
+            id : userid,
+            cid : cid,
+            data : data
+    };
+    $.post(root + '/oxguard/settings?action=set&session=' + sessionID + '&auth=' + encodeURIComponent(auth), JSON.stringify(json), function () {
+        log('updated settings');
+    })
+    .fail (function () {
+        alert (i18n.t('Failed to update settings'))
+    });
+}
+
+function rKeys (data) {
+    var keys = data.keys;
+    for (var i = 0; i < keys.length; i++) {
+        var key = keys[i];
+        var tr = $('<tr>');
+        var td1 = $('<td>');
+        var select = $('<input type="checkbox" class="keycheck" id="key' + i + '" value="' + i + '">');
+        td1.append(select);
+        var td2 = $('<td>');
+        var ids = key.ids;
+        for (var j = 0; j < ids.length; j++) {
+            td2.append(ids[j].replace('<', '&lt').replace('>', '&gt') + '<br>');
+        }
+        var td3 = $('<td>');
+        var fingerprints = key.fingerprints;
+        for (var k = 0; k < fingerprints.length; k++) {
+            td3.append(fingerprints[k] + '<br>');
+        }
+        var td4 = $('<td>');
+        var pass = $('<input type="password" id="pass' + i + '">');
+        td4.append(pass);
+        tr.append(td1).append(td2).append(td3).append(td4);
+        $('#keytable').append(tr);
+    }
+    view('listkeys');
+}
+
+function downloadKeys() {
+    var keys = [];
+    $('.keycheck').each(function (e,v) {
+       if ($(v).is(':checked')) {
+           var index = $(v).val();
+           key = {
+                   username : username, 
+                   index : index,
+                   password : $('#pass' + index).val()
+           }
+           keys.push(key);
+       };
+    });
+    if (keys.length === 0) {
+        alert(i18n.t('No keys selected'));
+        return;
+    }
+    $('#keys').html(i18n.t('Click to download') + '<br><br>');
+    for (var i = 0; i < keys.length; i++) {
+        $.post(root + '/oxguard/retrieve?action=getkeys&session=' + sessionID + '&auth=' + encodeURIComponent(auth), JSON.stringify(keys[i]), function (data) {
+            var filename = i18n.t('Key');
+            if (data.keydata.ids.length > 0) {
+                filename = data.keydata.ids[0];
+            }
+            var a = $('<a href="data:application/pgp;charset=utf-8,' + data.data + '" download="' + filename + '.asc" class="downloadkey">' + filename.replace('<','&lt').replace('>','&gt') + '</a>');
+            $('#keys').append(a).append('<br><br>');
+            $('#keys').show();
+        })
+        .fail (function (err) {
+            if (err.responseText.trim() === 'bad password') {
+                alert(i18n.t('Bad Password'));
+                return;
+            }
+            if (err.responseText.trim() === 'lockout') {
+                alert(i18n.t('Account locked out try later'));
+                return;
+            }
+            alert(i18n.t('Fail'));
+            console.log(err);
+        });
+    }
+
+    
+}
+
+function logOut () {
+    $.get(root + '/oxguard/login?action=logout')
+    .done(function () {
+        view('logout');
+    })
+    .fail(function () {
+        alert(i18n.t('Fail'));
+    });
+    
+}
 
 function log (where, data) {
     if (!debug) return;
