@@ -56,32 +56,30 @@ import java.nio.charset.StandardCharsets;
 import java.nio.file.Files;
 import java.nio.file.Path;
 import java.nio.file.Paths;
+import java.util.ArrayList;
 import java.util.List;
-
 import javax.crypto.Cipher;
-
 import org.apache.commons.codec.binary.Base64;
+import org.apache.commons.net.util.SubnetUtils;
+import org.apache.commons.net.util.SubnetUtils.SubnetInfo;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
 import ch.qos.logback.classic.Level;
-
-import com.openexchange.guard.cacheitems.Cache;
 import com.openexchange.guard.database.DbCommand;
 import com.openexchange.guard.database.DbQuery;
 import com.openexchange.guard.dbpool.ContextDBConnection;
 import com.openexchange.guard.dbpool.Manager;
-import com.openexchange.guard.server.HandleArgs;
+import com.openexchange.guard.encr.EncrLib;
 
 public class Config {
 	private static Logger logger = LoggerFactory.getLogger(Config.class);
 
 	public static int version = 7;  // Version for upgrader
-	
+
 	public static String serverVersion = "n/a";  // Value of server version from Java Manifest
 
     public static Level logLevel = Level.ALL;
-    
+
     public static Level jettyLogLevel = Level.ERROR;
 
     public enum LogType {
@@ -97,6 +95,10 @@ public class Config {
     public static String ogMySqlReadIP = null;
 
     public static String ogMySqlIP = "";
+    
+    public static String ogShardReadIP = null;
+    
+    public static String ogShardIP = null;
 
     public static String mysqlUsername = "";
 
@@ -107,39 +109,43 @@ public class Config {
     public static String mysqlPassword = "";
 
     public static String storageType = "file";
-    
+
     public static String configdbName = "configdb";
-    
+
     public static String rootFileDir = "/var/spool/open-xchange/guard/uploads";
 
 	public static String S3Endpoint = "";
-	
+
 	public static String S3BucketName = "";
-    
+
     public static String S3Region = "us-west-2";
-    
+
     public static String S3AccessKey = "";
-    
+
     public static String S3SecretKey = "";
-    
+
     public static String OXBackend = "";
-    
+
     public static String OXBackendPath = "/ajax/";
-    
+
     public static int OXBackendIdleTime = 62;  // Close idle connections to backend after time in seconds
-    
+
     public static int keyCacheCheckInterval = 29;   // Time between check of RSA pool count in seconds
-    
+
     public static String OXBackendUser = "";
-    
+
     public static String OXBackendPassword = "";
 
+    public static String SupportApiUser = "";
+
+    public static String SupportApiPassword = "";
+
     public static String OXUrl = "";
-    
+
     public static String externalReaderPath = "/appsuite/api/oxguard/reader/reader.html";
 
     public static String remoteLog = "";
-    
+
     public static boolean deleteOnRetract = true;
 
     public static String logDir = "/var/log/open-xchange/guard";
@@ -187,14 +193,16 @@ public class Config {
     public static int max_idle_db = 2;
 
     public static int max_open_db = 10;
-    
+
     public static int max_http_connections = 50;	// Max HTTP connections at once to a given route (backend, etc)
-    
+
     public static int max_total_connections = 50;   // Max total HTTP pooled connections to all destinations
 
     public static boolean no_recovery = false;
-    
-    public static boolean delete_recovery = true;  // Users allowed to delete their password recovery info -- pending implementation
+
+    public static boolean no_delete_recovery = true;  // Users allowed to delete their password recovery info
+
+    public static boolean no_delete_keys = false;  // Users allowed to delete their private keys or not
 
     public static boolean ok_private = true;
 
@@ -203,7 +211,7 @@ public class Config {
     public static int bad_minute_lock = 10;
 
     public static int bad_password_count = 5;
-    
+
     public static int max_remote = 100;  // Rate limit, max number of emails received remotely in lockout period
 
     public static int bad_ip_count = 10;
@@ -211,9 +219,11 @@ public class Config {
     public static boolean rsa_cache = true;
 
     public static int rsa_cache_count = 100;
-    
+
     public static int rsa_certainty = 256;
     
+    public static int key_valid_days = 3650;
+
     public static int min_password_length = 6;
 
     public static String default_lang = "en_US";
@@ -225,27 +235,42 @@ public class Config {
     public static String etcpath = "/opt/open-xchange/guard/etc";
 
     public static int ox_backend_port = 8009;
-    
-    public static boolean backend_ssl = false; 
+
+    public static boolean backend_ssl = false;
 
     public static int shard_size = 10000;
 
     public static String db_prefix = "oxguard_";
 
     public static boolean demo = false;
-    
+
     public static String upsellTemplate = "/opt/open-xchange/guard/templates/upsell.html";
-    
+
     public static String passwordFromAddress = null;  // Address and name for the FROM header for new password emails
-    
-    public static String passwordFromName = null;  
-    
+
+    public static String passwordFromName = null;
+
     public static boolean useStartTls = true;
-    
+
     public static boolean showStatus = true;
 
+    public static int pgpCacheDays = 7;
+
     public static String mailResolverUrl = null;  // MailResolverURL, must be defined so when email address is added to the end it will be valid
+
+    public static ArrayList<String> pgpLocations = new ArrayList<String> ();
+
+    public static String mailIdDomain = "";
+
+    public static ArrayList<SubnetInfo> pgpWhitelist = new ArrayList<SubnetInfo>();
     
+    public static String apiUserAgent = "Open-Xchange Guard Server";
+
+    /**
+     * The amount of hours how long the deleted keys will be stayed "exposed" for downloading
+     */
+    public static int ExposedKeyDurationInHours = 168 /* 1 Week */;
+
     /**
      * Parse config file and update variables
      */
@@ -273,14 +298,16 @@ public class Config {
                             case "com.openexchange.guard.usessl":
                                 if (data[1].trim().toLowerCase().contains("true")) {
                                     usessl = true;
-                                } else
+                                } else {
                                     usessl = false;
+                                }
                                 break;
                             case "com.openexchange.guard.rsacache":
                                 if (data[1].trim().toLowerCase().contains("true")) {
                                     rsa_cache = true;
-                                } else
+                                } else {
                                     rsa_cache = false;
+                                }
                                 break;
                             case "com.openexchange.guard.rsacachecount":
                                 rsa_cache_count = Integer.parseInt(data[1].trim());
@@ -294,32 +321,44 @@ public class Config {
                             case "com.openexchange.guard.norecovery":
                                 if (data[1].trim().toLowerCase().contains("true")) {
                                     no_recovery = true;
-                                } else
+                                } else {
                                     no_recovery = false;
+                                }
                                 break;
-                            case "com.openexchange.guard.deleterecovery":
+                            case "com.openexchange.guard.nodeleteprivate":
                                 if (data[1].trim().toLowerCase().contains("true")) {
-                                    delete_recovery = true;
-                                } else
-                                    delete_recovery = false;
+                                    no_delete_keys = true;
+                                } else {
+                                    no_delete_keys = false;
+                                }
+                                break;
+                            case "com.openexchange.guard.nodeleterecovery":
+                                if (data[1].trim().toLowerCase().contains("true")) {
+                                    no_delete_recovery = true;
+                                } else {
+                                    no_delete_recovery = false;
+                                }
                                 break;
                             case "com.openexchange.guard.okprivate":
                                 if (data[1].trim().toLowerCase().contains("true")) {
                                     ok_private = true;
-                                } else
+                                } else {
                                     ok_private = false;
+                                }
                                 break;
                             case "com.openexchange.guard.okextrapassword":
                                 if (data[1].trim().toLowerCase().contains("true")) {
                                     ok_extra_password = true;
-                                } else
+                                } else {
                                     ok_extra_password = false;
+                                }
                                 break;
                             case "com.openexchange.guard.demo":
                                 if (data[1].trim().toLowerCase().contains("true")) {
                                     demo = true;
-                                } else
+                                } else {
                                     demo = false;
+                                }
                                 break;
                             case "com.openexchange.guard.deleteonretract":
                             	if (data[1].trim().toLowerCase().contains("true")) {
@@ -338,8 +377,9 @@ public class Config {
                             case "com.openexchange.guard.defaultoxguardenabled":
                                 if (data[1].trim().toLowerCase().contains("true")) {
                                     default_oxguard_enabled = true;
-                                } else
+                                } else {
                                     default_oxguard_enabled = false;
+                                }
                                 break;
                             case "com.openexchange.guard.newpasslength":
                                 newPassLength = Integer.parseInt(data[1].trim());
@@ -409,9 +449,15 @@ public class Config {
                             case "com.openexchange.guard.oxguarddatabasehostname":
                                 ogMySqlIP = data[1].trim();
                                 break;
-                            case "com.openexchange.guard.guardreadip":
+                            case "com.openexchange.guard.oxguarddatabaseread":
                                 ogMySqlReadIP = data[1].trim();
                                 break;
+                            case "com.openexchange.guard.oxguardsharddatabase":
+                                ogShardIP = data[1].trim();
+                                break;
+                            case "com.openexchange.guard.oxguardshardread":
+                                ogShardReadIP = data[1].trim();
+                                break;
                             case "com.openexchange.guard.storage.type":
                                 storageType = data[1].trim();
                                 break;
@@ -447,6 +493,12 @@ public class Config {
                             case "com.openexchange.guard.restapipassword":
                                 OXBackendPassword = data[1].trim();
                                 break;
+                            case "com.openexchange.guard.supportapiusername":
+                                SupportApiUser = data[1].trim();
+                                break;
+                            case "com.openexchange.guard.supportapipassword":
+                                SupportApiPassword = data[1].trim();
+                                break;
                             case "com.openexchange.guard.upselltemplate":
                             	upsellTemplate = data[1].trim();
                             	break;
@@ -480,6 +532,9 @@ public class Config {
                             case "com.openexchange.guard.cachedays":
                                 cachedays = Integer.parseInt(data[1].trim());
                                 break;
+                            case "com.openexchange.guard.pgpcachedays":
+                                pgpCacheDays = Integer.parseInt(data[1].trim());
+                                break;
                             case "com.openexchange.guard.badipcount":
                                 bad_ip_count = Integer.parseInt(data[1].trim());
                                 break;
@@ -502,28 +557,37 @@ public class Config {
                             	shard_size = Integer.parseInt(data[1].trim());
                             	break;
                             case "com.openexchange.guard.logtype":
-                                if (data[1].contains("udp"))
+                                if (data[1].contains("udp")) {
                                     logType = LogType.udp;
-                                if (data[1].contains("terminal"))
+                                }
+                                if (data[1].contains("terminal")) {
                                     logType = LogType.terminal;
-                                if (data[1].contains("file"))
+                                }
+                                if (data[1].contains("file")) {
                                     logType = LogType.file;
+                                }
                                 break;
                             case "com.openexchange.guard.loglevel":
-                                if (data[1].contains("debug"))
+                                if (data[1].contains("debug")) {
                                     logLevel = Level.DEBUG;
-                                if (data[1].contains("error"))
+                                }
+                                if (data[1].contains("error")) {
                                     logLevel = Level.ERROR;
-                                if (data[1].contains("normal"))
+                                }
+                                if (data[1].contains("normal")) {
                                     logLevel = Level.INFO;
+                                }
                                 break;
                             case "com.openexchange.guard.jettyloglevel":
-                            	if (data[1].contains("debug"))
+                            	if (data[1].contains("debug")) {
                                     jettyLogLevel = Level.DEBUG;
-                                if (data[1].contains("error"))
-                                	jettyLogLevel = Level.ERROR;
-                                if (data[1].contains("normal"))
-                                	jettyLogLevel = Level.INFO;
+                                }
+                                if (data[1].contains("error")) {
+                                    jettyLogLevel = Level.ERROR;
+                                }
+                                if (data[1].contains("normal")) {
+                                    jettyLogLevel = Level.INFO;
+                                }
                                 break;
                             case "com.openexchange.guard.aeskeylength":
                                 AESKeyLen = Integer.parseInt(data[1].trim());
@@ -540,14 +604,20 @@ public class Config {
                                     System.exit(1);
                                 }
                                 break;
+                            case "com.openexchange.guard.keyvaliddays":
+                                key_valid_days = Integer.parseInt(data[1].trim());
+                                break;
                             case "com.openexchange.guard.externalemailurl":
-                                OXUrl = data[1].trim();
+                                OXUrl = data[1].trim().replace("http://", "").replace("https://", "");;
                                 if (data.length > 2) { // may have port in command line
                                     OXUrl = OXUrl + ":" + data[2].trim();
                                 }
                                 break;
                             case "com.openexchange.guard.externalreaderpath":
-                            	externalReaderPath = data[1].trim();
+                            	externalReaderPath = data[1].trim().replace("http://", "").replace("https://", "");
+                            	if (data.length > 2) { // may have port in command line
+                            	    externalReaderPath = externalReaderPath + ":" + data[2].trim();
+                                }
                                 break;
                             case "com.openexchange.guard.etcpath":
                                 etcpath = data[1].trim();
@@ -567,7 +637,43 @@ public class Config {
                             case "com.openexchange.guard.defaultlanguage":
                                 default_lang = data[1].trim();
                                 break;
+                            case "com.openexchange.guard.mailiddomain":
+                                mailIdDomain = data[1].trim();
+                                break;
+                            case "com.openexchange.guard.apiuseragent":
+                                apiUserAgent = data[1].trim();
+                                break;
+                            case "com.openexchange.guard.publicpgpdirectory":
+                            	String urlStrings = data[1].trim();
+                            	String [] urls = urlStrings.split(",");
+                            	for (String url : urls) {
+                            		url = url.trim();
+                            		if (url.startsWith("htt") || url.startsWith("hkp")) {
+                            			pgpLocations.add(url);
+                            		}
+                            	}
+                            	break;
+                            case "com.openexchange.guard.publickeywhitelist":
+                            	String list = data[1].trim();
+                            	String [] items = list.split(",");
+                            	for (String item : items) {
+                            		try {
+                            			SubnetUtils util = new SubnetUtils(item.trim());
+                            			pgpWhitelist.add(util.getInfo());
+                            		} catch (Exception e) {
+                            			logger.error("Bad IP address in whitelist", e);
+                            		}
 
+                            	}
+                            case "com.openexchange.guard.exposedkeydurationinhours":
+                                    try {
+                                        ExposedKeyDurationInHours = Integer.parseInt(data[1].trim());
+                                    } catch (Exception e) {
+                                        logger.error(
+                                            "Error while loading configuration com.openexchange.guard.exposedKeyDurationInHours. Using default value.",
+                                            e);
+
+                                    }
                             default:
                             	logger.info("Unknown configuration tag " + variable);
                                 break;
@@ -583,7 +689,6 @@ public class Config {
             	System.out.println("Possible malformed config file, or missing mysql username data");
             }
         } catch (IOException e) {
-            e.printStackTrace();
             logger.error("Error with config file " + path);
             System.exit(1);
         }
@@ -599,7 +704,7 @@ public class Config {
                 }
             }
         } catch (Exception ex) {
-            logger.error("Unable to load password file");
+            logger.error("Unable to load password file: " + ex.toString());
         }
 
     }
@@ -608,7 +713,7 @@ public class Config {
         ContextDBConnection dbcon = null;
         try {
 
-            DbCommand com = new DbCommand("SELECT * FROM global_settings");
+            DbCommand com = new DbCommand("SELECT global_settings.key, global_settings.value FROM global_settings");
             DbQuery db = new DbQuery();
             db.readOG(com);
             while (db.next()) {
@@ -646,7 +751,7 @@ public class Config {
     }
 
     private static void check() {
-        if (mysqlIP == "") {
+        if (mysqlIP.equals("")) {
             loadConfig();
         }
     }
@@ -675,12 +780,12 @@ public class Config {
         check();
         return (storageType);
     }
-    
+
     public static String getRootFileDirectory() {
         check();
         return (rootFileDir);
     }
-    
+
     public static String getS3Endpoint() {
     	check();
 		return S3Endpoint;
@@ -690,7 +795,7 @@ public class Config {
     	check();
 		return S3BucketName;
 	}
-    
+
 	public static String getS3Region() {
 		check();
 		return S3Region;
@@ -710,7 +815,7 @@ public class Config {
         check();
         return (OXBackend);
     }
-    
+
     public static String getBasicAuthenticationEncoding() throws UnsupportedEncodingException {
     	String userPassword = OXBackendUser + ":" + OXBackendPassword;
         return new String(Base64.encodeBase64(userPassword.getBytes("UTF-8")));
@@ -719,4 +824,16 @@ public class Config {
     public static String getConfigDbConnectionString() {
         return ("jdbc:mysql://" + mysqlIP + "/" + configdbName + "?user=" + mysqlUsername + "&password=" + mysqlPassword);
     }
+
+    public static String getMessageID () {
+        if (mailIdDomain.equals("")) {
+            if (OXUrl.contains("/")) {
+                mailIdDomain = OXUrl.substring(0, OXUrl.indexOf("/"));
+            }
+            if (mailIdDomain.contains(":")) {
+                mailIdDomain = mailIdDomain.substring(0, mailIdDomain.indexOf(":"));
+            }
+        }
+        return ("<" + EncrLib.getUUID() + "@" + mailIdDomain + ">");
+    }
 }
