@@ -4,7 +4,6 @@ import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.InputStreamReader;
 import java.io.UnsupportedEncodingException;
-
 import org.apache.http.HttpResponse;
 import org.apache.http.client.methods.CloseableHttpResponse;
 import org.apache.http.client.methods.HttpGet;
@@ -14,26 +13,31 @@ import org.apache.http.protocol.HttpContext;
 import org.apache.http.util.EntityUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
 import com.google.gson.JsonObject;
 import com.google.gson.JsonParser;
-import com.openexchange.guard.database.Utilities;
-import com.openexchange.guard.server.OxDbConn;
+import com.openexchange.guard.server.connectionPooling.HttpConnectionPoolService;
 
 public class OxConfiguration {
 	private static Logger logger = LoggerFactory.getLogger(OxConfiguration.class);
 	final static String templid = "com.openexchange.guard.templateID";
 	final static String fromEmail = "com.openexchange.guard.fromEmail";
-	
+	final static String externalReader = "com.openexchange.guard.externalReaderURL";
+	final static String deleteDraft = "com.openexchange.mail.deleteDraftOnTransport";
+	final static String oxUI = "com.openexchange.guard.externalOxUI";
+	final static String defaultReaderURL = "https://" + Config.externalReaderPath;
+	final static String defaultOxUI = Config.OXUrl;
+
 
     private static JsonObject getOgRequest(String prop, int cid, int id, boolean required) throws Exception {
 
         JsonObject result = null;
-        CloseableHttpClient httpClient = OxDbConn.httpClient;
+        //CloseableHttpClient httpClient = OxDbConn.httpClient;
+        CloseableHttpClient httpClient = HttpConnectionPoolService.getClient();
         HttpContext context = HttpClientContext.create();
         String url = "http" + (Config.backend_ssl ? "s" : "") + "://" + Config.getOxBackend() + ":" + Config.ox_backend_port + "/preliminary/configuration/v1/property/" + prop + "/" + cid + "/" + id;
         HttpGet getRequest = new HttpGet(url);
         getRequest.addHeader("accept", "application/json");
+        getRequest.setHeader("User-Agent", Config.apiUserAgent);
         getRequest.addHeader("Authorization", "Basic " + Config.getBasicAuthenticationEncoding());
         CloseableHttpResponse response = httpClient.execute(getRequest, context);
         if (response.getStatusLine().getStatusCode() != 200) {
@@ -62,11 +66,11 @@ public class OxConfiguration {
         return (result);
 
     }
-    
+
 
     /**
      * Format response into a JSON object
-     * 
+     *
      * @param response
      * @return
      * @throws UnsupportedEncodingException
@@ -84,36 +88,120 @@ public class OxConfiguration {
     public static JsonObject getProp(String prop, int cid, int id, boolean required) throws Exception {
         return (getOgRequest(prop, cid, id, required));
     }
-    
+
     public static int getTemplateId (int cid, int id) {
     	try {
-    		if (cid < 0) return (0);
+    		if (cid < 0) {
+                return (0);
+            }
     		JsonObject tjson = getProp(templid, cid, id, true);
-    		if (tjson == null) return(0);
+    		if (tjson == null) {
+                return(0);
+            }
     		if (tjson.has(templid)) {
     			return (tjson.get(templid).getAsInt());
-    		} else return (0);
+    		} else {
+                return (0);
+            }
     	} catch (Exception ex) {
     		logger.error("Error getting custom template ID ", ex);
     		return (0);
     	}
     }
-    
+
     public static String getFromEmail (int id, int cid) {
     	try {
-    		if (cid == 0) return (null);
+    		if (cid == 0) {
+                return (null);
+            }
     		if (cid < 0) {
     			cid = -cid;
     			id = -1;
     		}
     		JsonObject tjson = getProp(fromEmail, cid, id, false);
-    		if (tjson == null) return(null);
+    		if (tjson == null) {
+                return(null);
+            }
     		if (tjson.has(fromEmail)) {
     			return (tjson.get(fromEmail).getAsString());
-    		} else return (null);
+    		} else {
+                return (null);
+            }
     	} catch (Exception ex) {
     		logger.error("Error getting from Email address for password template ", ex);
     		return (null);
     	}
     }
+
+    public static String getExternalReaderURL (int id, int cid) {
+        try {
+            if (cid == 0) {
+                return (null);
+            }
+            if (cid < 0) {
+                cid = -cid;
+                id = -1;
+            }
+            JsonObject tjson = getProp(externalReader, cid, id, false);
+            if (tjson == null) {
+                return(defaultReaderURL);
+            }
+            if (tjson.has(externalReader)) {
+                return (tjson.get(externalReader).getAsString());
+            } else {
+                return (defaultReaderURL);
+            }
+        } catch (Exception ex) {
+            logger.error("Error getting External Email Reader URL ", ex);
+            return (defaultReaderURL);
+        }
+    }
+
+    public static String getOXUI (int id, int cid) {
+        try {
+            if (cid == 0) {
+                return (null);
+            }
+            if (cid < 0) {
+                cid = -cid;
+                id = -1;
+            }
+            JsonObject tjson = getProp(oxUI, cid, id, false);
+            if (tjson == null) {
+                return(defaultOxUI);
+            }
+            if (tjson.has(oxUI)) {
+                return (tjson.get(oxUI).getAsString());
+            } else {
+                return (defaultOxUI);
+            }
+        } catch (Exception ex) {
+            logger.error("Error getting OX UI URL ", ex);
+            return (defaultOxUI);
+        }
+    }
+
+    public static boolean deleteDraft (int id, int cid) {
+        try {
+            if (cid == 0) {
+                return (false);
+            }
+            if (cid < 0) {
+                cid = -cid;
+                id = -1;
+            }
+            JsonObject tjson = getProp(deleteDraft, cid, id, false);
+            if (tjson == null) {
+                return(false);
+            }
+            if (tjson.has(deleteDraft)) {
+                return (tjson.get(deleteDraft).getAsBoolean());
+            }
+            return(false);
+        } catch (Exception ex) {
+            logger.error("Error getting External Email Reader URL ", ex);
+            return (false);
+        }
+
+    }
 }
