@@ -49,47 +49,49 @@
 package com.openexchange.guard.database;
 
 import java.io.ByteArrayInputStream;
+import java.io.IOException;
+import java.security.NoSuchAlgorithmException;
+import java.security.spec.InvalidKeySpecException;
 import java.sql.SQLException;
+import java.util.ArrayList;
 import java.util.Date;
-
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
-
 import org.bouncycastle.openpgp.PGPPublicKeyRing;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.w3c.dom.Element;
 import org.w3c.dom.NodeList;
 import org.xml.sax.InputSource;
-
 import com.google.gson.JsonObject;
-import com.openexchange.guard.cacheitems.Cache;
 import com.openexchange.guard.config.Config;
 import com.openexchange.guard.encr.EncrLib;
 import com.openexchange.guard.encr.GuardKeys;
+import com.openexchange.guard.pgp.PgpKeys;
+import com.openexchange.guard.util.Core;
 
 /**
  * Class for access of mysql database for key and content ID retrieval
- * 
+ *
  * @author greg
  */
 public class Access {
 
-    private String mysqlIP = com.openexchange.guard.config.Config.getMysqlIP();// IP address of configdb
+    private final String mysqlIP = com.openexchange.guard.config.Config.getMysqlIP();// IP address of configdb
 
-    private String mysqlUsername = com.openexchange.guard.config.Config.getMysqlUsername();
+    private final String mysqlUsername = com.openexchange.guard.config.Config.getMysqlUsername();
 
-    private String mysqlPassword = com.openexchange.guard.config.Config.getMysqlPassword();
+    private final String mysqlPassword = com.openexchange.guard.config.Config.getMysqlPassword();
 
-    private String OGmysqlIP = com.openexchange.guard.config.Config.getMysqlIPOxGuard();
+    private final String OGmysqlIP = com.openexchange.guard.config.Config.getMysqlIPOxGuard();
 
-    private String ctxUsername = ""; // Username for access ox database that contains context
+    private final String ctxUsername = ""; // Username for access ox database that contains context
 
-    private String ctxPassword = ""; // Password
+    private final String ctxPassword = ""; // Password
 
-    private String url = ""; // URL of the mysql server containing the context data
+    private final String url = ""; // URL of the mysql server containing the context data
 
-    private String db = ""; // Database name
+    private final String db = ""; // Database name
 
     private String OGmysqlConnectionString = "";
 
@@ -132,8 +134,9 @@ public class Access {
     }
 
     public static String dbformat(String data) {
-        if (data == null)
+        if (data == null) {
             return (data);
+        }
         data = data.replace("'", "\'");
         data = data.replace("\u00ef\u00bf\u00bd", "\'");
         data = data.replace("\\", "\\\\");
@@ -142,7 +145,7 @@ public class Access {
 
     /**
      * Execute a mysql command
-     * 
+     *
      * @param command
      * @throws Exception
      */
@@ -164,7 +167,7 @@ public class Access {
 
     /**
      * Execute a mysql command
-     * 
+     *
      * @param command
      * @throws Exception
      */
@@ -190,16 +193,18 @@ public class Access {
             Sharding.checkTables(shard, false);
         } else {
             if (cid > 0)
+             {
                 Sharding.checkTables(cid, true); // Check these OG tables exist
+            }
         }
 
-        String command = "INSERT INTO og_KeyTable (id, cid, PGPSecret, PGPPublic, RSAPrivate, RSAPublic, Recovery, Salt, email, lastMod, lang) VALUES ( ?, ?, ?, ?, ?, ?, ?, ?, ?, ";
+        String command = "INSERT INTO og_KeyTable (id, cid, PGPSecret, PGPPublic, RSAPrivate, RSAPublic, Recovery, Salt, email, lastMod, lang, keyid) VALUES ( ?, ?, ?, ?, ?, ?, ?, ?, ?, ";
         if (userCreated) {
             command += "NOW()";
         } else {
             command += "NULL";
         }
-        command += ", ?)";
+        command += ", ?, ?)";
         DbCommand com = new DbCommand(command);
         com.addVariables((id == 0 ? null : id));
         com.addVariables(cid);
@@ -211,13 +216,15 @@ public class Access {
         com.addVariables(key.getSalt());
         com.addVariables(email);
         com.addVariables(language);
+        com.addVariables(key.pubring.getPublicKey().getKeyID());
         try {
             DbQuery db = new DbQuery();
             if (cid >= 0) {// OX users go in OX database, otherwise guest shard
                 if (id < 0) {// Master key is negative, otherwise place in proper shard
                     db.writeOxGuard(com);
-                } else
+                } else {
                     db.writeToDB(com, cid);
+                }
             } else {
                 db.writeToDb(com, Sharding.database_name(shard));
             }
@@ -230,8 +237,9 @@ public class Access {
                 if (cid >= 0) {
                     if (id > 0) {
                         db.readFromDB(com, cid);
-                    } else
+                    } else {
                         db.readOG(com);
+                    }
                 } else {
                     db.readFromDB(com, Sharding.database_name(shard));
                 }
@@ -239,10 +247,16 @@ public class Access {
                     userid = db.rs.getInt("id");
                 }
             }
-            if (userid > 0)
+            if (userid > 0) {
                 addEmail(email, userid, cid, shard);
+            }
             db.close();
 
+            key.contextid = cid;
+            if (userid > 0) {
+                PgpKeys.addPublicKeyIndex(key);
+            }
+
             return (userid);
         } catch (SQLException e) {
             logger.error("SQL error while storing key", e);
@@ -251,9 +265,72 @@ public class Access {
 
     }
 
+    public boolean updatePubring (GuardKeys key) throws IOException {
+        String command = "UPDATE og_KeyTable SET PGPPublic = ? WHERE keyid = ?";
+        DbCommand com = new DbCommand(command);
+        com.addVariables(key.exportPGPPublicKeyRing());
+        com.addVariables(key.keyid);
+        DbQuery db = new DbQuery();
+        try {
+            db.write(com, key.userid, key.contextid);
+        } catch (Exception e) {
+            logger.error("Error updating PGP Public KeyRing " , e);
+            return (false);
+        }
+        return (true);
+    }
+
+    public boolean addNewKey (GuardKeys newkey) throws Exception {
+
+    	int version = 0;
+    	// Check for highest version of key
+    	String getversion = "SELECT MAX(version) FROM og_KeyTable WHERE id = ? and cid = ?";
+    	DbCommand ver = new DbCommand(getversion);
+    	ver.addVariables(newkey.userid);
+    	ver.addVariables(newkey.contextid);
+        String update = "UPDATE og_KeyTable SET current = 0 WHERE id = ? and cid = ?";
+        DbCommand up = new DbCommand(update);
+        up.addVariables(newkey.userid);
+        up.addVariables(newkey.contextid);
+        DbQuery db = new DbQuery();
+        if (newkey.contextid >= 0) {// OX users go in OX database, otherwise guest shard
+            db.writeToDB(up, newkey.contextid);
+        } else {
+            db.write(up, newkey.userid, newkey.contextid);
+        }
+        db.close();
+        String command = "INSERT INTO og_KeyTable (id, cid, PGPSecret, PGPPublic, RSAPrivate, RSAPublic, Recovery, Salt, email, lastMod, lang, keyid, version, question, answer, settings) VALUES ( ?, ?, ?, ?, ?, ?, ?, ?, ?, " +
+            "NOW(), ?, ?, ?, ?, ?, ?)";
+        DbCommand com = new DbCommand(command);
+        com.addVariables(newkey.userid);
+        com.addVariables(newkey.contextid);
+        com.addVariables(newkey.getEncodedPGPSecret());
+        com.addVariables(newkey.exportPgpPublicKey());
+        com.addVariables(newkey.getEncodedPrivate());
+        com.addVariables(newkey.getEncodedPublic());
+        com.addVariables(newkey.recovery);
+        com.addVariables(newkey.getSalt());
+        com.addVariables(newkey.email);
+        com.addVariables(newkey.language);
+        com.addVariables(newkey.pubring.getPublicKey().getKeyID());
+        com.addVariables(newkey.version);
+        com.addVariables(newkey.question);
+        com.addVariables(newkey.answer);
+        com.addVariables(newkey.settings == null ? null : newkey.settings.toString());
+        db = new DbQuery();
+        if (newkey.contextid >= 0) {// OX users go in OX database, otherwise guest shard
+            db.writeToDB(com, newkey.contextid);
+        } else {
+            db.write(com, newkey.userid, newkey.contextid);
+        }
+        db.close();
+        PgpKeys.addPublicKeyIndex(newkey);
+        return (true);
+    }
+
     public void updateKeys(int id, int cid, GuardKeys key, String recovery, boolean reset) throws Exception {
-        String command = "UPDATE og_KeyTable Set PGPSecret = '" + key.getEncodedPGPSecret() + "', RSAPrivate = '" + key.getEncodedPrivate() + "', Recovery = '" + recovery + 
-        		"', lastMod = " + (reset ? "NULL" : "NOW()") + " WHERE id = " + id + " AND cid = " + cid + ";";
+        String command = "UPDATE og_KeyTable Set PGPSecret = '" + key.getEncodedPGPSecret() + "', RSAPrivate = " + (key.privKeyNull() ? "NULL" : ("'" + key.getEncodedPrivate() + "'")) + ", Recovery = '" + recovery +
+        		"', lastMod = " + (reset ? "NULL" : "NOW()") + " WHERE id = " + id + " AND cid = " + cid + " AND current = 1;";
         try {
             DbCommand com = new DbCommand(command);
             DbQuery db = new DbQuery();
@@ -264,7 +341,52 @@ public class Access {
         }
 
     }
-    
+
+    /**
+     * Update PGP Key record updating PGP Secret key and recovery on duplicate
+     * @param id
+     * @param cid
+     * @param key
+     * @param recovery
+     * @throws Exception
+     */
+    public void updateDuplPGPKey (int id, int cid, GuardKeys key, String recovery, Long dupl) throws Exception {
+        String command = "UPDATE og_KeyTable SET PGPSecret = ?, PGPPublic = ?, Recovery = ?, Salt = ? WHERE id = ? AND cid = ? and keyid = ?";
+        try {
+            DbCommand com = new DbCommand(command);
+            com.addVariables(key.getEncodedPGPSecret());
+            com.addVariables(key.exportPgpPublicKey());
+            com.addVariables(recovery);
+            com.addVariables(key.getSalt());
+            com.addVariables(id);
+            com.addVariables(cid);
+            com.addVariables(dupl);
+            DbQuery db = new DbQuery();
+            db.write(com, id, cid);
+            db.close();
+        } catch (SQLException e) {
+            logger.error("SQL error while updating PGP keys", e);
+        }
+    }
+
+    public void updatePGPKeys (int id, int cid, GuardKeys key) throws Exception {
+    	String command = "UPDATE og_KeyTable Set PGPSecret = ?, PGPPublic = ? WHERE id = ? AND cid = ? AND keyid = ?";
+        try {
+            DbCommand com = new DbCommand(command);
+            com.addVariables(key.getEncodedPGPSecret());
+            com.addVariables(key.exportPgpPublicKey());
+            com.addVariables(id);
+            com.addVariables(cid);
+            com.addVariables(key.keyid);
+            DbQuery db = new DbQuery();
+            db.write(com, id, cid);
+            db.close();
+            PgpKeys.addPublicKeyIndex(key);
+        } catch (SQLException e) {
+            logger.error("SQL error while updating PGP keys", e);
+        }
+    }
+
     public static void updatePin(int id, int cid, String pin) throws Exception {
     	String command = "UPDATE og_KeyTable Set question = 'PIN', answer = ? WHERE id = ? AND cid = ?;";
         try {
@@ -280,55 +402,62 @@ public class Access {
         }
     }
 
+
     public GuardKeys getKeys(int id, int cid) throws Exception {
         int shard = 0;
-        if (cid < 0)
+        if (cid < 0) {
             shard = Sharding.getShard(id, cid);
-        return (getKeys(id, cid, shard));
+        }
+        return (getKeys(id, cid, shard, null, true));
+    }
+
+    public GuardKeys getKeys(int id, int cid, long keyid, boolean current) throws Exception {
+    	int shard = 0;
+        if (cid < 0) {
+            shard = Sharding.getShard(id, cid);
+        }
+        return (getKeys(id, cid, shard, Long.toString(keyid), current));
+    }
+
+    public GuardKeys getKeys (int id, int cid, int shard) throws Exception {
+    	return (getKeys(id, cid, shard, null, true));
     }
 
     /**
      * Retrieve keys from Mater KeyTable
-     * 
+     *
      * @param id member ID
      * @return
      * @throws Exception
      */
-    public GuardKeys getKeys(int id, int cid, int shard) throws Exception {
+    public GuardKeys getKeys(int id, int cid, int shard, String keyid, boolean current) throws Exception {
         GuardKeys key = null;
-        String command = "SELECT * FROM og_KeyTable WHERE id = ? AND cid = ?";
+        String command = "SELECT PGPSecret, PGPPublic, RSAPrivate, RSAPublic, Salt, misc, settings, email, lastMod, question, answer, cid, id, keyid, version, current, lang, Recovery FROM og_KeyTable WHERE id = ? AND cid = ?";
+        if (current) {
+            command += " AND og_KeyTable.current = 1 ";
+        }
+        if (keyid != null) {
+            command += " AND keyid = ?";
+        }
         DbCommand com = new DbCommand(command);
         com.addVariables(id);
         com.addVariables(cid);
+        if (keyid != null) {
+            com.addVariables(keyid);
+        }
         DbQuery db = new DbQuery();
         if (cid >= 0) {
             if (id < 0) {// minus 1 is the recovery password, which is in OG
                 db.readOG(com);
-            } else
+            } else {
                 db.readFromDB(com, cid);
-        } else
+            }
+        } else {
             db.readFromShard(com, shard);
+        }
         try {
             if (db.next()) {
-                OxResultSet rs = db.rs;
-                key = new GuardKeys();
-                key.setPGPSecretFromString(rs.getString("PGPSecret"));
-                key.setPGPPublicKeyRing(rs.getString("PGPPublic"));
-                key.setEncodedPrivate(rs.getString("RSAPrivate"));
-                key.setPublicKeyFrom64String(rs.getString("RSAPublic"));
-                key.setSalt(rs.getString("Salt"));
-                key.misc = rs.getString("misc");
-                key.settings = rs.getString("settings");
-                key.email = rs.getString("email");
-                key.lastup = rs.getDate("lastMod");
-                key.question = rs.getString("question");
-                key.answer = rs.getString("answer");
-                key.contextid = cid;
-                key.userid = id;
-                String recovery = rs.getString("Recovery");
-                if (recovery.equals("-1") || (key.lastup == null)) {
-                    key.passwordNeeded = true;
-                }
+                key = getKeyFromRS (db.rs);
             }
         } catch (Exception ex) {
             logger.error("Error while iterating result set for keys", ex);
@@ -339,14 +468,130 @@ public class Access {
 
     /**
      * Retrieve keys from Mater KeyTable
-     * 
+     *
      * @param id member ID
      * @return
      * @throws Exception
      */
-    public GuardKeys getKeysFromEmail(String email) throws Exception {
+    public GuardKeys getOldGuardKeys(int id, int cid) throws Exception {
         GuardKeys key = null;
-        String command = "SELECT * FROM og_email WHERE email LIKE (?);";
+        String command = "SELECT PGPSecret, PGPPublic, RSAPrivate, RSAPublic, Salt, misc, settings, email, lastMod, question, answer, cid, id, keyid, version, current, lang, Recovery FROM og_KeyTable WHERE id = ? AND cid = ? AND version < 1";  //Version may be 0 or -1 for Guard key (-1 during upgrade)
+        DbCommand com = new DbCommand(command);
+        com.addVariables(id);
+        com.addVariables(cid);
+        DbQuery db = new DbQuery();
+        int shard = 0;
+        if (cid < 0) {
+            shard = Sharding.getShard(id, cid);
+        }
+        if (cid >= 0) {
+            if (id < 0) {// minus 1 is the recovery password, which is in OG
+                db.readOG(com);
+            } else {
+                db.readFromDB(com, cid);
+            }
+        } else {
+            db.readFromShard(com, shard);
+        }
+        try {
+            if (db.next()) {
+                key = getKeyFromRS (db.rs);
+            }
+        } catch (Exception ex) {
+            logger.error("Error while iterating result set for keys", ex);
+
+        }
+        return (key);
+    }
+
+    /**
+     * Get a list of all the Keys associated with the account
+     * @param id
+     * @param cid
+     * @return
+     * @throws Exception
+     */
+    public ArrayList<GuardKeys> getAllKeys (int id, int cid) throws Exception {
+        String command = "SELECT PGPSecret, PGPPublic, RSAPrivate, RSAPublic, Salt, misc, settings, email, lastMod, question, answer, cid, id, keyid, version, current, lang, Recovery FROM og_KeyTable WHERE id = ? AND cid = ? ORDER BY version DESC";
+        DbCommand com = new DbCommand(command);
+        com.addVariables(id);
+        com.addVariables(cid);
+        DbQuery db = new DbQuery();
+        db.read(com, id, cid);
+        ArrayList<GuardKeys> keys = new ArrayList<GuardKeys> ();
+        while (db.next()) {
+            GuardKeys key = getKeyFromRS (db.rs);
+            keys.add(key);
+        }
+        return (keys);
+    }
+
+    public ArrayList<GuardKeys> getAllKeysInContext (int cid) throws Exception {
+        String command = "SELECT PGPSecret, PGPPublic, RSAPrivate, RSAPublic, Salt, misc, settings, email, lastMod, question, answer, cid, id, keyid, version, current, lang, Recovery FROM og_KeyTable WHERE cid = ?";
+        DbCommand com = new DbCommand(command);
+        com.addVariables(cid);
+        DbQuery db = new DbQuery();
+        db.readFromDB(com, cid);
+        ArrayList<GuardKeys> keys = new ArrayList<GuardKeys> ();
+        while (db.next()) {
+            GuardKeys key = getKeyFromRS (db.rs);
+            keys.add(key);
+        }
+        return (keys);
+    }
+
+    /**
+     * Create GuardKey from database result set
+     * @param rs
+     * @param id  ID of user, -1 if unknown
+     * @param cid
+     * @return
+     * @throws InvalidKeySpecException
+     * @throws NoSuchAlgorithmException
+     * @throws Exception
+     */
+    private GuardKeys getKeyFromRS (OxResultSet rs) throws InvalidKeySpecException, NoSuchAlgorithmException, Exception {
+        GuardKeys key = new GuardKeys();
+        key.setPGPSecretFromString(rs.getString("PGPSecret"));
+        key.setPGPPublicKeyRing(rs.getString("PGPPublic"));
+        key.setEncodedPrivate(rs.getString("RSAPrivate"));
+        key.setPublicKeyFrom64String(rs.getString("RSAPublic"));
+        key.setSalt(rs.getString("Salt"));
+        key.misc = rs.getString("misc");
+        key.settings = rs.getJson("settings");
+        key.email = rs.getString("email");
+        key.lastup = rs.getDate("lastMod");
+        key.question = rs.getString("question");
+        key.answer = rs.getString("answer");
+        key.contextid = rs.getInt("cid");
+        key.userid = rs.getInt("id");
+        key.keyid = rs.getLong("keyid");
+        key.version = rs.getInt("version");
+        key.current = rs.getBit("current");
+        key.language = rs.getString("lang");
+        key.recovery = rs.getString("Recovery");
+        key.recoveryAvail = key.recovery == null ? false : !key.recovery.equals("");
+        if (key.recovery != null) {
+            if (key.recovery.equals("-1") || (key.lastup == null)) {
+                key.passwordNeeded = true;
+            }
+        }
+        return (key);
+    }
+
+    public GuardKeys getKeysFromEmail(String email) throws Exception {
+        return (getKeysFromEmail(email, 0L));
+    }
+    /**
+     * Retrieve keys from Mater KeyTable
+     *
+     * @param id member ID
+     * @return
+     * @throws Exception
+     */
+    public GuardKeys getKeysFromEmail(String email, Long keyid) throws Exception {
+        GuardKeys key = null;
+        String command = "SELECT cid, id, db FROM og_email WHERE email LIKE (?);";
         DbCommand com = new DbCommand(command);
         com.addVariables(email);
         DbQuery db = new DbQuery();
@@ -356,6 +601,9 @@ public class Access {
             int id = db.rs.getInt("id");
             int shard = db.rs.getInt("db");
             db.close();
+            if (keyid != 0L) {
+                return (getKeys(id, cid, shard, Long.toString(keyid), false));
+            }
             return (getKeys(id, cid, shard));
         }
         db.close();
@@ -364,7 +612,7 @@ public class Access {
 
     /**
      * Store Item into item table
-     * 
+     *
      * @param ItemID
      * @param owner
      * @param exp
@@ -374,8 +622,9 @@ public class Access {
      */
     public void storeItemID(String ItemID, int recip, int recip_cid, int owner, int owner_cid, long exp, int type, String XML, String salt) throws Exception {
         // Store item. If multiple recips within the same context database, then there will be duplicates we can ignore
-        if (XML.length() > 3072)
+        if (XML.length() > 3072) {
             XML = XML.substring(0, 3071);
+        }
         String command = "INSERT INTO og_encrypted_items (Id, Owner, Expiration, Type, XML, Salt, Owner_cid) VALUES (?, ?, ?, ?, ?, ?, ?) ON DUPLICATE KEY UPDATE Type = Type";
         DbQuery db = new DbQuery();
         DbCommand com = new DbCommand(command);
@@ -403,24 +652,35 @@ public class Access {
         db.write(com, userid, cid);
     }
 
+    public void deleteRecovery (int id, int cid) throws Exception {
+        String command = "UPDATE og_KeyTable SET Recovery = '' WHERE id = ? and cid = ?";
+        DbCommand com = new DbCommand(command);
+        com.addVariables(id);
+        com.addVariables(cid);
+        DbQuery db = new DbQuery();
+        db.write(com, id, cid);
+    }
+
     private String convertDate(java.util.Date dt) {
-        if (dt == null)
+        if (dt == null) {
             return (null);
+        }
         java.text.SimpleDateFormat sdf = new java.text.SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
         String currentTime = sdf.format(dt);
         return (currentTime);
     }
 
     public Boolean itemExists(String ItemId, int user, int cid) throws Exception {
-        String command = "SELECT * FROM og_encrypted_items WHERE og_encrypted_items.ID = ? AND Owner = ? AND Owner_cid = ?;";
+        String command = "SELECT Id FROM og_encrypted_items WHERE og_encrypted_items.ID = ? AND Owner = ? AND Owner_cid = ?;";
         DbQuery db = new DbQuery();
         DbCommand com = new DbCommand(command);
         com.addVariables(ItemId);
         com.addVariables(user);
         com.addVariables(cid);
         db.read(com, user, cid);
-        if (db.hasResults())
+        if (db.hasResults()) {
             return (true);
+        }
         return (false);
     }
 
@@ -435,14 +695,13 @@ public class Access {
             db.write(com, user, cid);
 
         } catch (Exception e) {
-            // TODO Auto-generated catch block
-            e.printStackTrace();
+            logger.error("Error updating status", e);
         }
     }
 
     public void getCount(String ItemID, int userid, int cid) throws Exception {
         String xml = "";
-        String command = "SELECT * FROM og_encrypted_items WHERE ID = ? AND Owner = ? AND Owner_cid = ?;";
+        String command = "SELECT XML FROM og_encrypted_items WHERE ID = ? AND Owner = ? AND Owner_cid = ?;";
         DbQuery db = new DbQuery();
         DbCommand com = new DbCommand(command);
         com.addVariables(ItemID);
@@ -461,8 +720,9 @@ public class Access {
             StringBuilder output = new StringBuilder();
             output.append("[");
             for (int i = 0; i < users.getLength(); i++) {
-                if (i > 0)
+                if (i > 0) {
                     output.append(", ");
+                }
                 Element user = (Element) users.item(i);
                 String email = user.getElementsByTagName("e").item(0).getTextContent();
                 int uid = Integer.parseInt(user.getElementsByTagName("i").item(0).getTextContent());
@@ -496,7 +756,7 @@ public class Access {
     }
 
     public RetrievedItem getItemID(String ItemId, int user, int cid) throws Exception {
-        String command = "SELECT * FROM og_encrypted_items INNER JOIN og_content_keys ON og_encrypted_items.Id = og_content_keys.ItemId WHERE og_encrypted_items.ID = ? AND UserId = ? AND CID = ?;";
+        String command = "SELECT ContentKey, Expiration, Owner, Salt, Type, XML FROM og_encrypted_items INNER JOIN og_content_keys ON og_encrypted_items.Id = og_content_keys.ItemId WHERE og_encrypted_items.ID = ? AND UserId = ? AND CID = ?;";
         DbCommand com = new DbCommand(command);
         com.addVariables(ItemId);
         com.addVariables(user);
@@ -520,7 +780,7 @@ public class Access {
 
     /**
      * Insert new token to oxGuard_sessions for encrypting passwords
-     * 
+     *
      * @param sessionID
      * @param userid
      * @return
@@ -548,11 +808,14 @@ public class Access {
 
     /**
      * Delete token from sessions for logout
-     * 
+     *
      * @param sessionID
      * @throws Exception
      */
     public void deleteToken(String sessionID) throws Exception {
+        if (sessionID == null) {
+            return;
+        }
         String command = "DELETE FROM oxguard_session WHERE sessionid = ?";
         DbCommand com = new DbCommand(command);
         com.addVariables(sessionID);
@@ -570,7 +833,7 @@ public class Access {
 
     /**
      * Get the token associated with the sessionID
-     * 
+     *
      * @param sessionID
      * @return
      */
@@ -612,7 +875,7 @@ public class Access {
 
     /**
      * Get number of times item has been opened. Returns -1 if not found
-     * 
+     *
      * @param userid
      * @param cid
      * @param itemid
@@ -644,7 +907,7 @@ public class Access {
     public boolean retract(int userid, int cid, String itemID, String salt, boolean delete) {
         try {
             DbQuery db = new DbQuery();
-            String command = "SELECT COUNT(*) AS count FROM og_encrypted_items WHERE  og_encrypted_items.Id = ? AND Salt = ?";
+            String command = "SELECT COUNT(Id) AS count FROM og_encrypted_items WHERE  og_encrypted_items.Id = ? AND Salt = ?";
             // Salt must be same to verify authorized sender
             DbCommand com = new DbCommand(command);
             com.addVariables(itemID);
@@ -658,13 +921,15 @@ public class Access {
             }
             // Delete the content key if so configured, otherwise expire the item
             command = "update og_encrypted_items SET Expiration = IF (Expiration = 0, -1, -Expiration) WHERE ID = ? AND Expiration >= 0 AND Salt = ?";
-            
+
             if (delete) {
             	command = "update og_content_keys SET ContentKey = 'retracted' WHERE ItemId = ?";
             }
             com = new DbCommand(command);
             com.addVariables(itemID);
-            if (!delete) com.addVariables(salt);
+            if (!delete) {
+                com.addVariables(salt);
+            }
             db.write(com, userid, cid);
             command = "update og_content_keys SET Status = IF (Status = 0, -1, -Status -1) WHERE ItemId = ?";
             com = new DbCommand(command);
@@ -679,7 +944,7 @@ public class Access {
 
     /**
      * Get XML data from an item. Returns null if not found. User must be owner
-     * 
+     *
      * @param userid
      * @param cid
      * @param itemid
@@ -707,7 +972,9 @@ public class Access {
     public boolean storeQuestion(int id, int cid, String question, String answer) {
         EncrLib encr = new EncrLib();
         String encranswer = null;
-        if (answer != null) encranswer = encr.encryptAES(answer, Config.rpass, question, Config.AESKeyLen);
+        if (answer != null) {
+            encranswer = encr.encryptAES(answer, Config.rpass, question, Config.AESKeyLen);
+        }
         String command = "UPDATE og_KeyTable SET question = ?, answer = ?" + " WHERE id = ? AND cid = ?";
         try {
             DbQuery db = new DbQuery();
@@ -724,26 +991,137 @@ public class Access {
         return(true);
     }
 
-    public PGPPublicKeyRing getPublicKeyRing(String email) {
+    public ArrayList<PGPPublicKeyRing> getPublicKeyRing(String email, int userid, int cid) {
+        return (getPublicKeyRing(email, userid, cid, false));
+    }
+    /**
+     * Search for public keys for email
+     * CID = 0 if search for local only
+     * @param email
+     * @param cid
+     * @param oxOnly  exclude guests
+     * @return
+     */
+    public ArrayList<PGPPublicKeyRing> getPublicKeyRing(String email, int userid, int cid, boolean oxOnly) {
         try {
-            String command = "select PublicKeyRing FROM PublicPGP WHERE email = ?";
-            DbCommand com = new DbCommand(command);
-            com.addVariables(email);
-            DbQuery db = new DbQuery();
-            db.readOG(com);
-            GuardKeys keys = new GuardKeys();
-            // First check if we have in our masterKeyTable
-            if (db.next()) {
-                keys.setPGPPublicKeyRing(db.rs.getString("PublicKeyRing"));
+        	ArrayList<PGPPublicKeyRing> pubRings = new ArrayList<PGPPublicKeyRing>();
+        	String command = "select id, cid, local FROM PGPKeys WHERE email = ?";
+        	if (oxOnly) {
+                command += " AND cid > 0";
             }
-            db.close();
-            return (keys.pubring);
+        	DbCommand com = new DbCommand(command);
+        	com.addVariables(email);
+        	DbQuery db = new DbQuery();
+        	db.readOG(com);
+        	while (db.next()) {
+        		String id = db.rs.getString("id");
+        		int key_cid = db.rs.getInt("cid");
+        		boolean local = db.rs.getBit("local");
+        		if (local) {  // If local, we just use the one we have
+        			GuardKeys key = getKeysFromEmail(email);
+        			pubRings.add(key.pubring);
+        			db.close();
+        			return (pubRings);
+        		} else {
+        			if (cid == 0) {
+                        return (null);
+                    }
+        			command = "SELECT PGPPublic FROM og_pgp_keys WHERE Email = ? AND ids LIKE ? AND cid = ? AND (userid = ? OR share_level > 0)";
+        			DbCommand com2 = new DbCommand(command);
+        			com2.addVariables(email);
+        			com2.addVariables("%" + id + "%");
+        			com2.addVariables(cid);
+        			com2.addVariables(userid);
+        			DbQuery db2 = new DbQuery();
+        			db2.readFromDB(com2, key_cid);
+        			if (db2.next()) {
+        				GuardKeys key = new GuardKeys();
+        				key.setPGPPublicKeyRing(db2.rs.getString("PGPPublic"));
+        				pubRings.add(key.pubring);
+        			}
+        			db2.close();
+        		}
+        	}
+        	return (pubRings);
         } catch (Exception ex) {
             logger.error("Error while retrieving public key ring for " + email, ex);
             return (null);
         }
     }
 
+    /**
+     * Get the public keyring associated with the id.  Can be long or Hex String (8 or 16 byte)
+     * @param req_id
+     * @param userid
+     * @param cid
+     * @return
+     */
+    public PGPPublicKeyRing getPublicKeyRingById(Object req_id, int userid, int cid) {
+        try {
+        	String command = "";
+        	String hexid = "";
+        	DbCommand com;
+        	if (req_id.getClass() == String.class) {
+        		command = "select email, cid, keyid, local FROM PGPKeys WHERE hexid = ?";
+        		hexid = (String) req_id;
+        		if (hexid.length() > 8) {  // If this is a 16 byte key, then lets convert it back to the long id
+        			req_id = Core.getLongPGP(hexid);
+        		}
+        	}
+        	if (req_id.getClass() == Long.class) {
+        		command = "select email, cid, keyid, local FROM PGPKeys WHERE id = ?";
+        		hexid = Core.PGPKeyId((Long) req_id);
+        	}
+        	DbQuery db = new DbQuery();
+        	com = new DbCommand(command);
+    		com.addVariables(req_id);
+        	db.readOG(com);
+        	if (db.next()) {
+        		String email = db.rs.getString("email");
+        		int key_cid = db.rs.getInt("cid");
+        		long keyid = db.rs.getLong("keyid");
+        		boolean local = db.rs.getBit("local");
+        		if (local) {  // If local, we just use the one we have
+        			GuardKeys key = getKeysFromEmail(email, keyid);
+        			db.close();
+        			return (key.pubring);
+        		} else {
+        			db.close();
+        			if (cid == 0) {
+                        return (null);
+                    }
+        			return (getUploadedPublicKeyRingById (hexid, userid, cid));
+        		}
+        	}
+        	db.close();
+        	if (cid == 0) {
+                return (null);
+            }
+        	return (getUploadedPublicKeyRingById (hexid, userid, cid));
+        } catch (Exception ex) {
+            logger.error("Error while retrieving public key ring for " + req_id, ex);
+            return (null);
+        }
+    }
+
+    private PGPPublicKeyRing getUploadedPublicKeyRingById (String req_id, int userid, int cid) throws Exception {
+    	String command = "SELECT PGPPublic FROM og_pgp_keys WHERE cid = ? AND (userid = ? OR share_level = 1) AND ids LIKE ?";
+		DbCommand com2 = new DbCommand(command);
+		com2.addVariables(cid);
+		com2.addVariables(userid);
+		com2.addVariables("%" + req_id + "%");
+		DbQuery db2 = new DbQuery();
+		db2.readFromDB(com2, cid);
+		if (db2.next()) {
+			GuardKeys key = new GuardKeys();
+			key.setPGPPublicKeyRing(db2.rs.getString("PGPPublic"));
+			db2.close();
+			return(key.pubring);
+		}
+		db2.close();
+		return (null);
+    }
+
     public static void addEmail(String email, int id, int cid, int shard) throws Exception {
         String command = "INSERT INTO og_email VALUES (?, ?, ?, ?) ON DUPLICATE KEY UPDATE id = ?, cid = ?, db = ?";
         DbCommand com = new DbCommand(command);
