@@ -49,7 +49,7 @@
 
 /*
  * Class for query database through OX HTTP Api
- * 
+ *
  */
 
 package com.openexchange.guard.database;
@@ -58,28 +58,24 @@ import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.InputStreamReader;
 import java.io.UnsupportedEncodingException;
-
 import org.apache.http.HttpEntity;
 import org.apache.http.HttpResponse;
-import org.apache.http.client.ClientProtocolException;
 import org.apache.http.client.methods.CloseableHttpResponse;
 import org.apache.http.client.methods.HttpPut;
 import org.apache.http.client.protocol.HttpClientContext;
 import org.apache.http.entity.StringEntity;
 import org.apache.http.impl.client.CloseableHttpClient;
-import org.apache.http.protocol.HTTP;
 import org.apache.http.protocol.HttpContext;
 import org.apache.http.util.EntityUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
 import com.google.gson.JsonArray;
 import com.google.gson.JsonElement;
 import com.google.gson.JsonObject;
 import com.google.gson.JsonParser;
 import com.openexchange.guard.config.Config;
 import com.openexchange.guard.encr.EncrLib;
-import com.openexchange.guard.server.OxDbConn;
+import com.openexchange.guard.server.connectionPooling.HttpConnectionPoolService;
 
 public class DbQuery {
 	private static Logger logger = LoggerFactory.getLogger(DbQuery.class);
@@ -89,19 +85,19 @@ public class DbQuery {
     public OxResultSet rs = null;
 
     public boolean keepOpen = false;
-    
+
     public int updated = -1;
 
     String transaction = null;
 
     int index = -1;
-    
+
 
     String query_id = "";
 
     /**
      * Actual send of data to OX Backend
-     * 
+     *
      * @param data String of JSON data to send
      * @param database The ID of the database
      * @param cid ContextId if pertinent. 0 if not
@@ -117,8 +113,9 @@ public class DbQuery {
         } else {
             url = "http" + (Config.backend_ssl ? "s" : "") + "://" + Config.getOxBackend() + ":" + Config.ox_backend_port + "/preliminary/database/v1/pool/r/" + OGInit.read_map.get(database) + "/w/" + OGInit.write_map.get(database) + "/" + database + "/" + (write ? "writable" : "readOnly");
         }
-        if (keepOpen)
+        if (keepOpen) {
             url += "?keepOpen=true";
+        }
         if (transaction != null) {
             url = "http" + (Config.backend_ssl ? "s" : "") + "://" + Config.getOxBackend() + ":" + Config.ox_backend_port + "/preliminary/database/v1/transaction/" + transaction;
         }
@@ -128,7 +125,7 @@ public class DbQuery {
 
     /**
      * Actual send of data to OX Backend
-     * 
+     *
      * @param data String of JSON data to send
      * @param database The ID of the database
      * @param cid ContextId if pertinent. 0 if not
@@ -140,31 +137,35 @@ public class DbQuery {
 
 
         String url = "http" + (Config.backend_ssl ? "s" : "") + "://" + Config.getOxBackend() + ":" + Config.ox_backend_port + "/preliminary/database/v1/pool/r/" + OGInit.read_id + "/w/" + OGInit.write_id + "/oxguard/" + (write ? "writable" : "readOnly");
-        if (keepOpen)
+        if (keepOpen) {
             url += "?keepOpen=true";
+        }
         if (transaction != null) {
             url = "http" + (Config.backend_ssl ? "s" : "") + "://" + Config.getOxBackend() + ":" + Config.ox_backend_port + "/preliminary/database/v1/transaction/" + transaction;
         }
         return (putRequest (data, url));
 
     }
-    
+
     private JsonObject putRequest (String data, String url) throws Exception {
-    	CloseableHttpClient httpClient = OxDbConn.httpClient;
+    	//CloseableHttpClient httpClient = OxDbConn.httpClient;
+    	CloseableHttpClient httpClient = HttpConnectionPoolService.getClient();
         HttpContext context = HttpClientContext.create();
         HttpPut putRequest = new HttpPut(url);
         StringEntity entity = new StringEntity(data, "UTF-8");
-        entity.setContentType("text/javascript;charset=UTF-8");
+        entity.setContentType("application/json;charset=UTF-8");
         putRequest.addHeader("accept", "application/json");
         putRequest.addHeader("Authorization", "Basic " + Config.getBasicAuthenticationEncoding());
+        putRequest.setHeader("User-Agent", Config.apiUserAgent);
         putRequest.setEntity(entity);
         CloseableHttpResponse response = null;
         try {
         	response = httpClient.execute(putRequest, context);
         } catch (IllegalStateException | org.apache.http.conn.ConnectTimeoutException e) {
         	logger.error("Connection issue with backend.  Trying reset pool" , e);
-        	OxDbConn.reset();
-        	response = OxDbConn.httpClient.execute(putRequest, context);
+        	//OxDbConn.reset();
+        	HttpConnectionPoolService.reset();
+        	response = httpClient.execute(putRequest, context);
         } catch (org.apache.http.conn.HttpHostConnectException ex) {
         	logger.error("Unable to connect to backend.  Connection refused.  ?? Down ??");
         	result = null;
@@ -213,7 +214,9 @@ public class DbQuery {
     }
 
     private void checkUpdated () {
-    	if (result == null) return;
+    	if (result == null) {
+            return;
+        }
     	if (result.has("results")) {
     		JsonObject sub = result.get("results").getAsJsonObject();
     		if (sub.has(query_id)) {
@@ -226,7 +229,7 @@ public class DbQuery {
     }
     /**
      * Format the query into proper JSON for backend
-     * 
+     *
      * @param com
      * @return
      */
@@ -255,8 +258,9 @@ public class DbQuery {
                 query_id = id;
             }
             sb.append("}");
-            if (i < (com.count() - 1))
+            if (i < (com.count() - 1)) {
                 sb.append(',');
+            }
         }
 
         sb.append("}");
@@ -265,7 +269,7 @@ public class DbQuery {
 
     /**
      * Format response into a JSON object
-     * 
+     *
      * @param response
      * @return
      * @throws UnsupportedEncodingException
@@ -294,7 +298,7 @@ public class DbQuery {
 
     /**
      * Send command to ConfigDb
-     * 
+     *
      * @param command
      * @return
      * @throws Exception
@@ -335,7 +339,7 @@ public class DbQuery {
 
     /**
      * Get request from Database
-     * 
+     *
      * @param command
      * @param cid Context ID for the database
      * @return
@@ -351,7 +355,7 @@ public class DbQuery {
 
     /**
      * Get request from Database
-     * 
+     *
      * @param command
      * @param cid Context ID for the database
      * @return
@@ -359,8 +363,9 @@ public class DbQuery {
      */
     public JsonObject readFromShard(DbCommand command, int db) throws Exception {
         try {
-            if (db < 0)
+            if (db < 0) {
                 return (null);
+            }
             return (putRequest(createQuery(command), Sharding.database_name(db), 0, false));
         } catch (Exception e) {
             throw (e);
@@ -377,7 +382,7 @@ public class DbQuery {
 
     /**
      * Get request from OxGuard
-     * 
+     *
      * @param command
      * @param cid Context ID for the database
      * @return
@@ -393,15 +398,19 @@ public class DbQuery {
 
     /**
      * Write to database
-     * 
+     *
      * @param command
      * @param cid Context ID for the database
      * @return
      * @throws Exception
      */
-    public JsonObject writeToDB(DbCommand command, int cid) throws Exception {
+    public JsonObject writeToDB(Object command, int cid) throws Exception {
         try {
-            return (putRequest(createQuery(command), "oxdb", cid, true));
+            if (command.getClass() == DbCommand.class) {
+                return (putRequest(createQuery((DbCommand) command), "oxdb", cid, true));
+            } else {
+                return (putRequest(createComplexQuery((MultiCommand) command), "oxdb", cid, true));
+            }
         } catch (Exception e) {
             throw (e);
         }
@@ -409,23 +418,28 @@ public class DbQuery {
 
     /**
      * Write to database
-     * 
+     *
      * @param command
      * @param cid Context ID for the database
      * @return
      * @throws Exception
      */
-    public JsonObject writeToShard(DbCommand command, int db) throws Exception {
+    public JsonObject writeToShard(Object command, int db) throws Exception {
         try {
-            if (db < 0)
+            if (db < 0) {
                 return (null);
-            return (putRequest(createQuery(command), Sharding.database_name(db), 0, true));
+            }
+            if (command.getClass() == DbCommand.class) {
+                return (putRequest(createQuery((DbCommand) command), Sharding.database_name(db), 0, true));
+            } else {
+                return (putRequest(createComplexQuery((MultiCommand) command), Sharding.database_name(db), 0, true));
+            }
         } catch (Exception e) {
             throw (e);
         }
     }
 
-    public JsonObject write(DbCommand command, int id, int cid) throws Exception {
+    public JsonObject write(Object command, int id, int cid) throws Exception {
         if (cid > 0) {
             return (writeToDB(command, cid));
         } else {
@@ -435,7 +449,7 @@ public class DbQuery {
 
     /**
      * Write to oxguard
-     * 
+     *
      * @param command
      * @param cid Context ID for the database
      * @return
@@ -455,23 +469,25 @@ public class DbQuery {
 
     /**
      * Return if has error
-     * 
+     *
      * @return
      */
     public boolean hasError() {
-        if (result == null)
+        if (result == null) {
             return (true);
+        }
         return (result.get("error") != null);
     }
 
     /**
      * Get the error string
-     * 
+     *
      * @return
      */
     public String getError() {
-        if (result == null)
+        if (result == null) {
             return ("Null result");
+        }
         if (result.get("error") != null) {
             return (result.get("error").getAsString());
         }
@@ -480,7 +496,7 @@ public class DbQuery {
 
     /**
      * Has results?
-     * 
+     *
      * @return
      */
     public boolean hasResults() {
@@ -489,11 +505,13 @@ public class DbQuery {
 
     /**
      * Get next row of data
-     * 
+     *
      * @return
      */
     public boolean next() {
-    	if (result == null) return(false);
+    	if (result == null) {
+            return(false);
+        }
     	try {
     		JsonArray rows = result.get("results").getAsJsonObject().get(query_id).getAsJsonObject().get("rows").getAsJsonArray();
     		index++;
@@ -509,7 +527,7 @@ public class DbQuery {
 
     /**
      * Get the row of data at index position
-     * 
+     *
      * @param i index of row
      * @return
      */
@@ -519,7 +537,7 @@ public class DbQuery {
 
     /**
      * Get count of rows
-     * 
+     *
      * @return
      */
     public int getRowCount() {
@@ -528,7 +546,7 @@ public class DbQuery {
 
     /**
      * Get result at an index
-     * 
+     *
      * @param name
      * @param index
      * @return
