@@ -52,13 +52,10 @@ package com.openexchange.guard.database;
 import java.security.NoSuchAlgorithmException;
 import java.security.spec.InvalidKeySpecException;
 import java.util.ArrayList;
-
 import javax.servlet.http.HttpServletRequest;
-
 import org.apache.commons.codec.binary.Base64;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
 import com.google.gson.JsonObject;
 import com.openexchange.guard.cacheitems.Cache;
 import com.openexchange.guard.config.Config;
@@ -69,6 +66,9 @@ import com.openexchange.guard.mailcreator.Mail;
 import com.openexchange.guard.ox.Api;
 import com.openexchange.guard.ox.InterOx;
 import com.openexchange.guard.ox.Languages;
+import com.openexchange.guard.pgp.PGPPublicHandler;
+import com.openexchange.guard.pgp.PgpKeyLookup;
+import com.openexchange.guard.pgp.PgpKeys;
 import com.openexchange.guard.server.OxCookie;
 import com.openexchange.guard.server.Sender;
 import com.openexchange.guard.validator.EmailValidator;
@@ -83,7 +83,7 @@ public class GetPubKey {
 	private static Logger logger = LoggerFactory.getLogger(GetPubKey.class);
 
     public RecipKey getPubKey(String email, int senderID, int senderCID, String from, boolean sending, boolean remote) throws Exception {
-        return (getPubKey(email, senderID, senderCID, from, sending, remote, "", "", null));
+        return (getPubKey(email, senderID, senderCID, from, sending, remote, "", "", null, null));
     }
 
     /**
@@ -96,18 +96,31 @@ public class GetPubKey {
      * @return
      * @throws Exception
      */
-    public RecipKey getPubKey(String email, int senderID, int senderCID, String from, boolean sending, boolean remote, String guestlang, String fromName, String pin) throws Exception {
+    public RecipKey getPubKey(String email, int senderID, int senderCID, String from, boolean sending, boolean remote, String guestlang, String fromName, String pin, EmailInfo info) throws Exception {
 
     	if (!EmailValidator.validate(email)) return (null); // make sure email address
         // Get connection
         RecipKey result = null;
-        // First check if we have in our masterKeyTable
-        result = checkMaster(email);
-
-        // If not found, check to see if we have it in cache for outside user
-        if ((result == null) && !remote) {
-            result = checkCache(email);
+        
+        // First, we'll check if there is a key uploaded by the user
+        GuardKeys key = null;
+        // If there is a senderID, check for upload
+        
+        if (senderID != 0) key = PGPPublicHandler.getPGPKeysFromEmail(email, senderID, senderCID);
+        if (key == null && info != null)  // If no key found, check from the original sender if this is a reply
+            key = PGPPublicHandler.getPGPKeysFromEmail(email, info.owner_id, info.owner_cid);
+        if (key!=null) {
+            result = new RecipKey(key);
+            return (result);
         }
+        
+        // Next check if we have in our masterKeyTable
+        result = checkMaster(email);
+        if (result != null) return(result);
+        // If not found, check to see if we have it in cache for outside user
+   //     if ((result == null) && !remote) {
+    //        result = checkCache(email);
+    //    }
 
         if (result == null) {
             // If not in our keytable, and not in cache, then let's search the OX user database to see if member
@@ -139,9 +152,14 @@ public class GetPubKey {
 	                            result.lang,
 	                            templId);
 		                if (!sender.send(pwEmail, null)) return(null);
-		                GuardKeys keys = keyman.createMasterKeys(result.userid, result.cid, email, result.name, result.newGuestPass, result.lang);
+		                // When creating a new key for a recipient, we will always create a recovery in case of lost email, etc
+		                GuardKeys keys = keyman.createMasterKeys(result.userid, result.cid, email, result.name, result.newGuestPass, result.lang, false, true);
+		                result.pgpring = keys.pubring;
 		                result.pubkey = (keys == null) ? null : keys.getPublic();
+		                
 	                }
+	                result.pgp = true;
+	                return(result);
                 }
             } catch (Exception ex) {
             	logger.error("Error getting public key for " + email, ex);
@@ -149,10 +167,12 @@ public class GetPubKey {
                 return (result);
             }
         }
-
-        // OK, local has failed. Lookup remote oxguard server
-        if ((result == null) && !remote) {
-            result = checkRemote(email, from);
+        
+        
+        
+        if (!remote) {
+            result = PgpKeyLookup.getRemoteKeys(email);
+            if (result != null) return(result);
         }
 
         if (remote)
@@ -169,7 +189,7 @@ public class GetPubKey {
     public static RecipKey checkMaster(String email) throws Exception {
         RecipKey result = null;
         // First check if we have in our masterKeyTable
-        String command = "SELECT * FROM og_email WHERE email = ?";
+        String command = "SELECT id, cid, db FROM og_email WHERE email = ?";
         DbCommand com = new DbCommand(command);
         com.addVariables(email);
         DbQuery db = new DbQuery();
@@ -178,7 +198,7 @@ public class GetPubKey {
             int id = db.rs.getInt("id");
             int cid = db.rs.getInt("cid");
             int shard = db.rs.getInt("db");
-            command = "select cid, id, RSAPublic, lang FROM og_KeyTable WHERE email = ?";
+            command = "select cid, id, RSAPublic, PGPPublic, settings, lang FROM og_KeyTable WHERE email = ? AND current = 1";
             com = new DbCommand(command);
             com.addVariables(email);
             db.read(com, id, cid);
@@ -196,6 +216,12 @@ public class GetPubKey {
                 result.userid = rs.getInt("id");
                 result.cid = rs.getInt("cid");
                 result.guest = result.cid < 0;
+                result.settings = rs.getJson("settings");
+                result.pgpring = PGPPublicHandler.getKeyRing(rs.getString("PGPPublic"));
+                if (result.pgpring != null) {
+                    result.expired = PgpKeys.isExpired(result.pgpring.getPublicKey());
+                    result.pgp = true;
+                }
                 result.lang = rs.getString("lang");
                 if (result.lang == null)
                     result.lang = Languages.default_code;
@@ -257,7 +283,7 @@ public class GetPubKey {
             result = new RecipKey();
             Keymanager manager = new Keymanager();
             result.newGuestPass = EncrLib.randomPass(Config.newPassLength);
-            GuardKeys keys = manager.createMasterKeys(0, (senderCID * -1), email, email, result.newGuestPass, Languages.isavail(guestlang));// Guest
+            GuardKeys keys = manager.createMasterKeys(0, (senderCID * -1), email, email, result.newGuestPass, Languages.isavail(guestlang), false, true);// Guest
                                                                                                                                             // account
                                                                                                                                             // is
                                                                                                                                             // going
@@ -269,6 +295,7 @@ public class GetPubKey {
                                                                                                                                             // first
                                                                                                                                             // sender
             result.pubkey = (keys == null) ? null : keys.getPublic();
+            result.pgpring = (keys == null) ? null : keys.pubring;
             result.userid = (keys == null) ? -1 : keys.userid;
             result.cid = (senderCID * -1);
             result.guest = true;
