@@ -50,17 +50,18 @@
 package com.openexchange.guard.database;
 
 import java.sql.Connection;
-
+import javax.servlet.http.HttpServletRequest;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
 import com.openexchange.guard.cacheitems.Cache;
 import com.openexchange.guard.config.Config;
 import com.openexchange.guard.dbpool.ContextDBConnection;
 import com.openexchange.guard.encr.EncrLib;
-import com.openexchange.guard.encr.PgpKeys;
 import com.openexchange.guard.encr.RSAGen;
 import com.openexchange.guard.encr.GuardKeys;
+import com.openexchange.guard.pgp.PgpKeys;
+import com.openexchange.guard.server.Auth;
+import com.openexchange.guard.server.Settings;
 
 public class Keymanager {
 	private static Logger logger = LoggerFactory.getLogger(Keymanager.class);
@@ -68,19 +69,15 @@ public class Keymanager {
 //    public GuardKeys createMasterKeys(int id, int cid, String email, String name, String language) {
  //       return (createMasterKeys(id, cid, email, name, "", language));
  //   }
-	public GuardKeys createMasterKeys(int id, int cid, String email, String name, String password, String language) {
-		return(createMasterKeys (id, cid, email, name, password, language, false));
-	}
-    public GuardKeys createMasterKeys(int id, int cid, String email, String name, String password, String language, boolean userCreated) {
-        // PgpKeys pgp = new PgpKeys();
+
+    public GuardKeys createMasterKeys(int id, int cid, String email, String name, String password, String language, boolean userCreated, boolean createRecovery) {
         RSAGen gen = new RSAGen();
         try {
-            // oxGuardKeys keys = pgp.generateKeys(name, email, password);
             GuardKeys keys = gen.generateKeyPair(name, email, password);
             if (keys != null) {
                 Access acc = new Access();
                 keys.email = email;
-                keys.userid = acc.storeKeys(id, cid, email, keys, createRecovery(keys, password), language, userCreated);
+                keys.userid = acc.storeKeys(id, cid, email, keys, createRecovery ? createRecovery(keys, password) : "", language, userCreated);
                 if (keys.userid == 0) return(null); //Failed to create keys
             } else {
             	return(createMasterKeysNC(id, cid, email, name, password, language, userCreated));// Try failover to generated realtime
@@ -95,11 +92,11 @@ public class Keymanager {
         }
     }
     
-    public GuardKeys createMasterKeysNC(int id, int cid, String email, String name, String password, String language) {
-    	return (createMasterKeysNC (id, cid, email, name, password, language, false));
+    public GuardKeys createMasterKeysNC(int id, int cid, String email, String name, String password, String language, boolean createRecovery) {
+    	return (createMasterKeysNC (id, cid, email, name, password, language, false, createRecovery));
     }
 
-    public GuardKeys createMasterKeysNC(int id, int cid, String email, String name, String password, String language, boolean userCreated) {
+    public GuardKeys createMasterKeysNC(int id, int cid, String email, String name, String password, String language, boolean userCreated, boolean createRecovery) {
         logger.debug("Not using cache, Creating master keys for " + email);
     	PgpKeys pgp = new PgpKeys();
     	Access acc = new Access();
@@ -107,7 +104,7 @@ public class Keymanager {
             GuardKeys keys = pgp.generateKeys(name, email, password);
             if (keys != null) {
                 
-                keys.userid = acc.storeKeys(id, cid, email, keys, createRecovery(keys, password), language, userCreated);
+                keys.userid = acc.storeKeys(id, cid, email, keys, createRecovery ? createRecovery(keys, password) : "", language, userCreated);
                 if (keys.userid == 0) return(null); //Failed to create keys
             }
             return (keys);
@@ -126,26 +123,36 @@ public class Keymanager {
         }
     }
 
+    public static boolean checkCreateRecovery (com.openexchange.guard.server.OxCookie cookie, HttpServletRequest request) {
+        Settings settings = Auth.getPermissions(cookie, request);
+        return (!settings.norecovery);
+    }
 
-    public String createRecovery(GuardKeys keys, String password) throws Exception {
+    public static String createRecovery(GuardKeys keys, String password) throws Exception {
         if (password.equals(""))
             return ("-1");
         if (Config.no_recovery)
             return (""); // If selected for no recovery, then just return blank.
         password = EncrLib.getSHA(password, keys.getSalt());
+        return EncryptRecoveryHash (keys, password);
+        
+    }
+    
+    public static String EncryptRecoveryHash (GuardKeys keys, String password) throws Exception {
         String recovery = "";
         EncrLib encr = new EncrLib();
-        String command = "SELECT RSAPublic FROM og_KeyTable WHERE id = -1 AND cid = 0";
+        String command = "SELECT RSAPublic, PGPPublic FROM og_KeyTable WHERE id = -1 AND cid = 0";
         DbCommand com = new DbCommand(command);
         DbQuery db = new DbQuery();
         db.readOG(com);
         if (db.next()) {
             String pub = db.rs.getString("RSAPublic");
+            String pgppub = db.rs.getString("PGPPublic");
             GuardKeys masterKeys = new GuardKeys();
             masterKeys.setPublicKeyFrom64String(pub);
             String encrPass = encr.encryptAES(
                 password,
-                Config.rpass + Integer.toString(keys.getEncodedPublic().hashCode()),
+                Config.rpass + (keys.getPublic() == null ? "" : Integer.toString(keys.getEncodedPublic().hashCode())),
                 keys.getSalt(),
                 Config.AESKeyLen);
             recovery = encr.encryptRSA(encrPass, masterKeys.getPublic());
@@ -153,20 +160,19 @@ public class Keymanager {
         }
         db.close();
         if (recovery.equals("")) {
-        	if (keys.userid > 0) {  // make sure not for system email
-        		logger.error("Unable to create recovery for " + keys.email);
-        		throw (new Exception ("Unable to create recovery"));
-        	}
+            if (keys.userid > 0) {  // make sure not for system email
+                logger.error("Unable to create recovery for " + keys.email);
+                throw (new Exception ("Unable to create recovery"));
+            }
         }
         return (recovery);
     }
 
     public String recoverPassHash(int id, int cid) {
         String hash = "";
-        ContextDBConnection conn = null;
         try {
             Access acc = new Access();
-            String command = "SELECT Recovery, Salt, RSAPublic FROM og_KeyTable WHERE id = ? AND cid = ?";
+            String command = "SELECT Recovery, Salt, RSAPublic, PGPPublic FROM og_KeyTable WHERE id = ? AND cid = ? AND current = 1";
             DbCommand com = new DbCommand(command);
             com.addVariables(id);
             com.addVariables(cid);
@@ -177,6 +183,7 @@ public class Keymanager {
                 if (!recovery.equals("")) {
                 	String salt = db.rs.getString("Salt");
                     String rsapub = db.rs.getString("RSAPublic");
+                    String PGPPublic = db.rs.getString("PGPPublic");
                     GuardKeys masterKeys = acc.getKeys(-1, 0);
                     EncrLib encr = new EncrLib();
                     String encrhash = encr.decryptRSA(recovery, masterKeys.getDecodedPrivate(Config.mpass));
@@ -186,7 +193,7 @@ public class Keymanager {
                     }
                     hash = encr.decryptAES(
                         encrhash,
-                        Config.rpass + Integer.toString(rsapub.hashCode()),
+                        Config.rpass + (rsapub == null ? "" : Integer.toString(rsapub.hashCode())),
                         salt,
                         Config.AESKeyLen);
                 }
@@ -199,7 +206,7 @@ public class Keymanager {
     }
 
     public String resetPassword(String email) {
-        String newpass = EncrLib.randomPass(Config.newPassLength);
+        String newpass = EncrLib.randomPass(Config.newPassLength >= 4 ? Config.newPassLength : 4);
         return (resetPassword(email, newpass, true));
     }
 
@@ -218,7 +225,7 @@ public class Keymanager {
             Access acc = new Access();
             GuardKeys oldkeys = acc.getKeysFromEmail(email);
             String hash = recoverPassHash(oldkeys.userid, oldkeys.contextid);
-            if (hash == "") {
+            if (hash.equals("")) {
                 return ("NR");  // Return no recovery
             }
 
