@@ -63,13 +63,11 @@ import java.util.ArrayList;
 import java.util.Date;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
-
 import javax.crypto.Cipher;
 import javax.crypto.EncryptedPrivateKeyInfo;
 import javax.crypto.SecretKeyFactory;
 import javax.crypto.spec.PBEKeySpec;
 import javax.crypto.spec.PBEParameterSpec;
-
 import org.apache.commons.lang.StringUtils;
 import org.apache.commons.lang3.StringEscapeUtils;
 import org.apache.commons.net.util.Base64;
@@ -82,8 +80,6 @@ import org.bouncycastle.util.io.pem.PemObject;
 import org.bouncycastle.util.io.pem.PemWriter;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
-import com.openexchange.guard.cacheitems.Cache;
 import com.openexchange.guard.config.Config;
 import com.openexchange.guard.database.Access;
 import com.openexchange.guard.database.RecipKey;
@@ -91,9 +87,10 @@ import com.openexchange.guard.exceptions.NoContentKeyException;
 import com.openexchange.guard.exceptions.NoItemException;
 import com.openexchange.guard.exceptions.NoMasterKeysException;
 import com.openexchange.guard.ox.InterOx;
+import com.openexchange.guard.pgp.PgpKeys;
 import com.openexchange.guard.server.CheckBad;
+import com.openexchange.guard.server.OxCookie;
 import com.openexchange.guard.storage.Storage;
-import com.openexchange.guard.storage.file.FileStorage;
 
 public class Crypto {
 
@@ -118,7 +115,9 @@ public class Crypto {
             return(null); // XML can only be less than 10000 varchar
         }
         acc.storeItemID(itemID, owner, owner_cid, owner, owner_cid, exp, type, finalXML, salt);
-        if (!saveContentKey(itemID, owner, owner_cid, contentKey, acc)) return(null);
+        if (!saveContentKey(itemID, owner, owner_cid, contentKey, acc)) {
+            return(null);
+        }
         for (RecipKey user : recipients) {
             if (user.remote) {// If remote user, we're going to send the data.
                 EncrLib crypto = new EncrLib();
@@ -133,10 +132,14 @@ public class Crypto {
                 {
                     // user.cid = -user.cid;
                     logger.debug("Non OX user " + user.cid + " sending to filesystem");
- 
-                    if (!Storage.saveEncrObj(user.userid, user.cid, itemID, addHeader(encrypted, itemID, data.length()))) return (null);
+
+                    if (!Storage.saveEncrObj(user.userid, user.cid, itemID, addHeader(encrypted, itemID, data.length()))) {
+                        return (null);
+                    }
+                }
+                if (!saveContentKey(itemID, user.userid, user.cid, user.pubkey, contentKey, acc)) {
+                    return (null);
                 }
-                if (!saveContentKey(itemID, user.userid, user.cid, user.pubkey, contentKey, acc)) return (null);
                 acc.storeItemID(itemID, user.userid, user.cid, owner, owner_cid, exp, type, XML, salt);
                 if (user.cid != owner_cid) { // We need to save the item in each context, but as not owner, userid is 0
                     acc.storeItemID(itemID, user.userid, user.cid, owner, owner_cid, exp, type, XML, salt);
@@ -155,7 +158,7 @@ public class Crypto {
 
     /**
      * Save an item without specifying a content key
-     * 
+     *
      * @param owner owner of object
      * @param share who is on the share list
      * @param data the data of the object
@@ -173,7 +176,7 @@ public class Crypto {
 
     /**
      * Save an item without specifying a content key
-     * 
+     *
      * @param owner owner of object
      * @param share who is on the share list
      * @param data the data of the object
@@ -215,16 +218,20 @@ public class Crypto {
             String salt = EncrLib.getsalt();
             if (!attachment) {// Emails already saved, so we don't need to repeat for attachments
                 acc.storeItemID(itemID, owner, cid, owner, cid, exp, 0, XML, salt);
-                if (!saveContentKey(itemID, owner, cid, contentKey, acc)) return(null);
+                if (!saveContentKey(itemID, owner, cid, contentKey, acc)) {
+                    return(null);
+                }
             }
 
             if (recipients != null) {
                 for (RecipKey user : recipients) {
                     if (!attachment) {
-                        if (!saveContentKey(itemID, user.userid, user.cid, contentKey, acc)) return (null);
+                        if (!saveContentKey(itemID, user.userid, user.cid, contentKey, acc)) {
+                            return (null);
+                        }
                         acc.storeItemID(itemID, user.userid, user.cid, owner, cid, exp, 0, XML, salt);
                     }
-                    if (user.cid < 0) // Guest account
+                    if (user.cid < 0) // Guest   account
                     {
                     	Storage.saveEncrObj(
                             user.userid,
@@ -248,7 +255,7 @@ public class Crypto {
 
     /**
      * Decode and encrypted object. Requires userid and password
-     * 
+     *
      * @param user Id of the user
      * @param obj The EncryptedObj
      * @param password
@@ -266,7 +273,9 @@ public class Crypto {
             }
             com.openexchange.guard.database.Access acc = new com.openexchange.guard.database.Access();
             com.openexchange.guard.database.RetrievedItem item = acc.getItemID(obj.ItemID, user, cid); // Get the item data
-            if (item == null) throw new NoItemException("Item not found");
+            if (item == null) {
+                throw new NoItemException("Item not found");
+            }
             if (item.expiration > 0) {// If our expiration not set to unlimited, check
                 Date now = new Date();
                 if (item.expiration < now.getTime()) {
@@ -279,8 +288,16 @@ public class Crypto {
             }
             EncrLib encryptor = new EncrLib();
             if ((contentkey == null) || (contentkey.equals("undefined"))) {
-                com.openexchange.guard.encr.GuardKeys keys = acc.getKeys(user, cid); // Load the users private key from store
-                if (keys == null) throw new NoMasterKeysException ("Unable to find/retrieve user keys");
+                com.openexchange.guard.encr.GuardKeys keys = acc.getOldGuardKeys(user, cid); // Load the users private key from store
+                if (keys != null) {
+                    if (keys.version < 0) {   // If version < 0, then needs upgrade checking
+                        PgpKeys.checkBadUID(keys, password);
+                    }
+                }
+
+                if (keys == null) {
+                    throw new NoMasterKeysException ("Unable to find/retrieve user keys");
+                }
                 contentkey = encryptor.decryptRSA(item.contentKey, keys.getDecodedPrivate(password));
                 if (contentkey == null) {
                     CheckBad.addBad(obj.ItemID);
@@ -335,7 +352,7 @@ public class Crypto {
 
         return (null);
     }
-    
+
 
     public static byte[] addHeader(String strdata, String itemID, int length) {
         String filehead = StringUtils.rightPad("ID: " + itemID + ", Key: " + Config.AESKeyLen + (length > 0 ? (", Len: " + length) : "") + ", START-----", 100, "-");// Create header
@@ -411,10 +428,26 @@ public class Crypto {
             StringWriter sigfos = new StringWriter(); // write PKCS#8 EncryptedPrivateKeyInfo to file
             sigfos.write("-----BEGIN ENCRYPTED PRIVATE KEY-----\r\n" + Base64.encodeBase64String(encinfo.getEncoded()) + "-----END ENCRYPTED PRIVATE KEY-----");
             sigfos.close();
-            System.out.println(sigfos.toString());
         } catch (Exception ex) {
             logger.error("Error while exporting private key", ex);
         }
         return ("");
     }
+
+    public static String decodeEPass (String epass, OxCookie cookie) {
+    	Access acc = new Access();
+        String token = acc.getToken(cookie.JSESSIONID);
+        EncrLib enc = new EncrLib();
+        return(enc.decryptAES(epass, token));
+    }
+
+    public static String encodeEPass (String password, OxCookie cookie, int userid, int cid) throws Exception {
+    	EncrLib encr = new EncrLib();
+        Access acc = new Access();
+        String token = acc.getToken(cookie.JSESSIONID);
+        if (token == null) {
+            token = acc.newToken(cookie.JSESSIONID, userid, cid);
+        }
+        return (encr.encryptAES(password, token));
+    }
 }
