@@ -52,7 +52,6 @@ package com.openexchange.guard.encr;
 import org.apache.commons.codec.binary.Base64;
 import org.bouncycastle.bcpg.ArmoredOutputStream;
 import org.bouncycastle.openpgp.PGPKeyRingGenerator;
-
 import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
@@ -68,20 +67,24 @@ import java.security.spec.InvalidKeySpecException;
 import java.security.spec.PKCS8EncodedKeySpec;
 import java.security.spec.X509EncodedKeySpec;
 import java.sql.Date;
-
 import org.bouncycastle.openpgp.PGPException;
+import org.bouncycastle.openpgp.PGPPrivateKey;
 import org.bouncycastle.openpgp.PGPPublicKeyRing;
 import org.bouncycastle.openpgp.PGPPublicKey;
 import org.bouncycastle.openpgp.PGPSecretKey;
 import org.bouncycastle.openpgp.PGPSecretKeyRing;
+import org.bouncycastle.openpgp.operator.PGPDigestCalculatorProvider;
 import org.bouncycastle.openpgp.operator.bc.BcKeyFingerprintCalculator;
+import org.bouncycastle.openpgp.operator.jcajce.JcePBESecretKeyDecryptorBuilder;
 import org.bouncycastle.openpgp.PGPUtil;
 import org.bouncycastle.openpgp.PGPObjectFactory;
+import org.bouncycastle.operator.bc.BcDigestCalculatorProvider;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
+import com.google.gson.JsonObject;
 import com.openexchange.guard.config.Config;
 import com.openexchange.guard.database.RecipKey;
+import com.openexchange.guard.database.model.DeletedKey;
 import com.openexchange.guard.exceptions.BadPasswordException;
 
 /**
@@ -112,16 +115,30 @@ public class GuardKeys {
 
     public String misc;
 
-    public String settings;
+    public JsonObject settings;
 
     public String email;
     
     public String question;
     
     public String answer;
+    
+    public String recovery;
+    
+    public long keyid;
+     
+    public boolean current;
+    
+    public boolean inline = false;
+    
+    public int version;
+    
+    public String language;
+    
+    public boolean recoveryAvail = false;
 
     public GuardKeys() {
-
+        salt = EncrLib.getsalt();
     }
 
     public GuardKeys(RecipKey key) {
@@ -130,6 +147,17 @@ public class GuardKeys {
         pubkey = key.pubkey;
         email = key.email;
     }
+    
+    /**
+     * Populate guardkey based on deleted key
+     * @param key
+     */
+    public GuardKeys(DeletedKey key) {
+        keyring = key.getSecretKeyRing();
+        userid = key.getUserId();
+        contextid = key.getCid();
+        salt = key.getSalt();
+    }
 
     /**
      * @param gen PGPKeyRingGenerator
@@ -150,6 +178,9 @@ public class GuardKeys {
         keyring = ring;
     }
 
+    public boolean privKeyNull () {
+        return (privkey == null);
+    }
     /**
      * Set the private RSA key, need password and salt to encrypt
      * 
@@ -161,7 +192,7 @@ public class GuardKeys {
     public Boolean setPrivate(PrivateKey priv, String password) {
         EncrLib crypto = new EncrLib();
         privkey = crypto.encryptAES(Base64.encodeBase64String(priv.getEncoded()), EncrLib.getSHA(password, salt), salt, Config.AESKeyLen);
-        if (privkey == "") { // Return false if failed to save
+        if (privkey.equals("")) { // Return false if failed to save
             return (false);
         } else
             return (true);
@@ -180,6 +211,26 @@ public class GuardKeys {
     public PrivateKey getDecodedPrivate(String password) throws BadPasswordException, UnsupportedEncodingException {
         return (getDecodedPrivate(password, true));
     }
+    
+    public boolean verifyPassword (String password) throws BadPasswordException, PGPException, UnsupportedEncodingException {
+        if (password == null) return(false);
+        if (privkey != null) {
+        	try {
+        		if (getDecodedPrivate (password, true) != null) {
+        			return(true);
+        		} else return (false);
+        	} catch (BadPasswordException e) {
+        		logger.warn("Bad password");
+        		return(false);
+        	}
+        }
+        try {
+            if (getPGPEncrKey (password) != null) return (true);
+        } catch (Exception e) {
+            return(false);
+        }
+        return (false);
+    }
 
     /**
      * Retrieve the Private RSA key using password and salt
@@ -247,6 +298,10 @@ public class GuardKeys {
      * @throws NoSuchAlgorithmException
      */
     public void setPublicKeyFrom64String(String data) throws InvalidKeySpecException, NoSuchAlgorithmException {
+        if (data == null) {
+            pubkey = null;
+            return;
+        }
         KeyFactory kf = KeyFactory.getInstance("RSA");
         // for private keys use PKCS8EncodedKeySpec; for public keys use X509EncodedKeySpec
         X509EncodedKeySpec ks = new X509EncodedKeySpec(Base64.decodeBase64(data));
@@ -269,6 +324,7 @@ public class GuardKeys {
      * @return
      */
     public String getEncodedPublic() {
+        if (pubkey == null) return (null);
         return (Base64.encodeBase64String(pubkey.getEncoded()));
     }
 
@@ -296,7 +352,7 @@ public class GuardKeys {
      * @return
      */
     public PGPPublicKey getPGPPublicKey() {
-        Iterator keys = keyring.getPublicKeys();
+        Iterator keys = pubring.getPublicKeys();
         PGPPublicKey key = null;
         while (keys.hasNext()) {
             key = (PGPPublicKey) keys.next();
@@ -304,7 +360,7 @@ public class GuardKeys {
                 return key;
             }
         }
-        return (keyring.getPublicKey());
+        return (pubring.getPublicKey());
 
     }
 
@@ -324,8 +380,21 @@ public class GuardKeys {
      * @return
      */
     public PGPSecretKey getPGPSecretKey() {
+        if (keyring == null) return(null);
         return (keyring.getSecretKey());
     }
+    
+    public PGPSecretKey getPGPSigningKey() {
+    	Iterator it = keyring.getSecretKeys();
+    	while (it.hasNext()) {
+    		PGPSecretKey key = (PGPSecretKey) it.next();
+    		if (key.isMasterKey()) {
+    		    return (key);
+    		} else 
+    			if (key.isSigningKey()) return(key);
+    	}
+    	return (null);
+    }
 
     public PGPSecretKeyRing getPGPSecretKeyRing() {
         return (keyring);
@@ -338,6 +407,7 @@ public class GuardKeys {
      * @throws IOException
      */
     public String getEncodedPGPSecret() throws IOException {
+        if (keyring == null) return(null);
         return (Base64.encodeBase64String(keyring.getEncoded()));
     }
 
@@ -350,7 +420,10 @@ public class GuardKeys {
      * @throws PGPException
      */
     public void setPGPSecretFromString(String keystring) throws IOException, PGPException {
-
+        if (keystring == null) {
+            keyring = null;
+            return;
+        }
         keyring = new PGPSecretKeyRing(Base64.decodeBase64(keystring), new BcKeyFingerprintCalculator());
 
     }
@@ -387,16 +460,26 @@ public class GuardKeys {
         return (getEncryptionKey(getKeyring(ascdata)));
     }
 
+    public void setPGPKeyRingFromAsc (String ascdata) throws IOException {
+        pubring = getKeyring(ascdata);
+    }
+    
     private PGPPublicKeyRing getKeyring(String AscData) throws IOException {
         // PGPUtil.getDecoderStream() will detect ASCII-armor automatically and decode it,
         // the PGPObject factory then knows how to read all the data in the encoded stream
-        PGPObjectFactory factory = new PGPObjectFactory(PGPUtil.getDecoderStream(new java.io.ByteArrayInputStream(AscData.getBytes("UTF-8"))));
-        // these files should really just have one object in them,
-        // and that object should be a PGPPublicKeyRing.
-        Object o = factory.nextObject();
-        if (o instanceof PGPPublicKeyRing) {
-            return (PGPPublicKeyRing) o;
+        try {
+            PGPObjectFactory factory = new PGPObjectFactory(PGPUtil.getDecoderStream(new java.io.ByteArrayInputStream(AscData.getBytes("UTF-8"))), new BcKeyFingerprintCalculator());
+            // these files should really just have one object in them,
+            // and that object should be a PGPPublicKeyRing.
+            Object o = factory.nextObject();
+            if (o instanceof PGPPublicKeyRing) {
+                return (PGPPublicKeyRing) o;
+            }
+        } catch (Exception ex) {
+            logger.error("Error creating keyring " ,ex);
+             
         }
+
         throw new IllegalArgumentException("Input text does not contain a PGP Public Key");
     }
 
@@ -435,13 +518,14 @@ public class GuardKeys {
         // arm.write(pbkeyring.getEncoded());
         Iterator<PGPPublicKey> keys = pubring.getPublicKeys();
         ByteArrayOutputStream bout = new ByteArrayOutputStream();
-        List<PGPPublicKey> pkeys = new ArrayList<PGPPublicKey>();
         while (keys.hasNext()) {
             PGPPublicKey k = keys.next();
             k.encode(bout);
         }
         arm.write(bout.toByteArray());
         arm.close();
+        bout.close();
+        out.close();
         return (new String(out.toByteArray()));
     }
 
@@ -450,6 +534,7 @@ public class GuardKeys {
         ArmoredOutputStream arm = new ArmoredOutputStream(out);
         arm.write(pubring.getEncoded());
         arm.close();
+        out.close();
         return (new String(out.toByteArray()));
     }
 
@@ -460,6 +545,99 @@ public class GuardKeys {
             logger.error("Failed to store in public key ring", e);
         }
     }
+    
+    /**
+     * Get PGP Private Signing Key
+     * @param password
+     * @return
+     * @throws PGPException 
+     */
+    public PGPPrivateKey getPGPSignKey (String password) throws PGPException {
+    	PGPSecretKey sec_key = null;
+    	Iterator<PGPSecretKey> it = keyring.getSecretKeys();
+    	while (sec_key == null && it.hasNext()) {
+    		PGPSecretKey key = it.next();
+    		if (key.isSigningKey()) {
+    			return (getDecodedPrivate(password, key));
+    		}
+    	}
+    	logger.error("Unable to find PGP signing key");
+    	return (null);
+    }
+    
+    public PGPSecretKey getMasterPGPSecretKey () {
+    	PGPSecretKey sec_key = null;
+    	Iterator<PGPSecretKey> it = keyring.getSecretKeys();
+    	while (sec_key == null && it.hasNext()) {
+    		PGPSecretKey key = it.next();
+    		if (key.isMasterKey()) {
+    			return (key);
+    		}
+    	}
+    	logger.error("Unable to find PGP signing key");
+    	return (null);
+    }
+    
+    /**
+     * Get PGP private key for encrypting
+     * @param password
+     * @return
+     * @throws PGPException 
+     */
+    public PGPPrivateKey getPGPEncrKey (String password) throws PGPException {
+    	PGPSecretKey sec_key = null;
+    	Iterator<PGPSecretKey> it = keyring.getSecretKeys();
+    	PGPSecretKey master = null;
+    	while (sec_key == null && it.hasNext()) {
+    		PGPSecretKey key = it.next();
+    		if (!key.isMasterKey()) { // We prefer to not return the master.  Only return master if no other encr keys found
+    			return (getDecodedPrivate(password, key));
+    		} else master = key;
+    	}
+    	if (master != null) return (getDecodedPrivate(password, master));
+    	logger.error("Unable to find PGP encrypting key");
+    	return (null);
+    }
+    
+    /**
+     * Get PGP Private key associated with ID
+     * @param password
+     * @param id
+     * @return
+     * @throws PGPException 
+     */
+    public PGPPrivateKey getPGPKey (String password, long id) throws PGPException {
+    	PGPSecretKey sec_key = null;
+    	Iterator<PGPSecretKey> it = keyring.getSecretKeys();
+    	while (sec_key == null && it.hasNext()) {
+    		PGPSecretKey key = it.next();
+    		if (key.getKeyID() == id) {
+    			return (getDecodedPrivate(password, key));
+    		}
+    	}
+    	logger.error("Unable to find PGP key for id " + Long.toString(id));
+    	return (null);
+    }
+    
+    private PGPPrivateKey getDecodedPrivate (String password, PGPSecretKey sec_key) throws PGPException
+    {
+    	char [] pass = EncrLib.getSHA(password, salt).toCharArray();
+    	PGPPrivateKey decryptor = sec_key.extractPrivateKey(new JcePBESecretKeyDecryptorBuilder().setProvider("BC").build(pass));
+    	//org.bouncycastle.openpgp.operator.PBESecretKeyDecryptor decryptor = new org.bouncycastle.openpgp.operator.bc.BcPBESecretKeyDecryptorBuilder(new org.bouncycastle.openpgp.operator.bc.BcPGPDigestCalculatorProvider()).build(pass);
+        return (decryptor);
+    }
+    
+    public PGPPublicKey getPGPPublicKey (long id) {
+    	
+    	Iterator<PGPPublicKey> it = pubring.getPublicKeys();
+    	while (it.hasNext()) {
+    		PGPPublicKey key = it.next();
+    		if (key.getKeyID() == id) {
+    			return (key);
+    		}
+    	}
+    	return (null);
+    }
 
     public int getKeyLen() {
         RSAPublicKey rsapub = (RSAPublicKey) pubkey;
