@@ -12,13 +12,12 @@ import java.util.List;
 import java.util.Properties;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
-
+import javax.mail.Message.RecipientType;
 import javax.mail.MessagingException;
 import javax.mail.Session;
 import javax.mail.internet.MimeMessage;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
-
 import org.apache.commons.fileupload.FileItem;
 import org.apache.commons.fileupload.FileItemFactory;
 import org.apache.commons.fileupload.FileUploadException;
@@ -28,11 +27,11 @@ import org.apache.commons.io.IOUtils;
 import org.bouncycastle.util.encoders.Base64;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
 import com.google.gson.Gson;
 import com.google.gson.JsonArray;
 import com.google.gson.JsonObject;
 import com.google.gson.JsonParser;
+import com.google.gson.JsonPrimitive;
 import com.openexchange.guard.config.OxConfiguration;
 import com.openexchange.guard.database.Access;
 import com.openexchange.guard.database.EmailInfo;
@@ -43,12 +42,18 @@ import com.openexchange.guard.encr.DecodedObj;
 import com.openexchange.guard.encr.EncrLib;
 import com.openexchange.guard.encr.EncryptedObj;
 import com.openexchange.guard.exceptions.GuardMissingParameter;
+import com.openexchange.guard.logging.LogAction;
 import com.openexchange.guard.mailcreator.Attachment;
 import com.openexchange.guard.mailcreator.Mail;
 import com.openexchange.guard.ox.Api;
+import com.openexchange.guard.pgp.PGPResult;
+import com.openexchange.guard.pgp.PGPSender;
+import com.openexchange.guard.pgp.PGPUtils;
+import com.openexchange.guard.pgp.PgpHandler;
 import com.openexchange.guard.server.Auth;
 import com.openexchange.guard.server.MailHandler;
 import com.openexchange.guard.server.OxCookie;
+import com.openexchange.guard.server.Settings;
 import com.openexchange.guard.server.UserData;
 import com.openexchange.guard.server.Settings.permissions;
 import com.openexchange.guard.util.Core;
@@ -78,10 +83,9 @@ public class Incoming {
 	   public void incomingEmailForm(HttpServletRequest request, HttpServletResponse response) throws Exception {
 	        FileItemFactory fif = new DiskFileItemFactory();
 	        ServletFileUpload sfu = new ServletFileUpload(fif);
-	        String itemID = EncrLib.getUUID(); // One itemID for email and it's attachments
-	        String salt = EncrLib.getsalt();// Create salt
 	        int userid = Core.getIntParameter(request, "userid", true);
 	        int cid = Core.getIntParameter(request, "cid", true);
+	        LogAction.setUser(userid, cid);
 	        String sentfolder = Core.getStringParameter(request, "sentfolder");
 	        if (sentfolder == null) sentfolder = "default0/Sent Items";
 	        String guest = Core.getStringParameter(request, "guest");
@@ -89,11 +93,12 @@ public class Incoming {
 	        	if (!guest.toLowerCase().trim().equals("true")) guest = null;
 	        }
 	        String language = Core.getStringParameter(request, "lang");
-	        String guestlanguage = Core.getStringParameter(request, "guestlangauge");
+	        String guestlanguage = Core.getStringParameter(request, "guestlanguage");
 	        String attach_password = Core.getStringParameter(request, "password");// encrypted attachment password
-	        String attach_extrapass = Core.getStringParameter(request, "extrapass");  // encrypted attachment extra password
 	        String auth = Core.getStringParameter(request, "auth");
+	        String draftFolder = Core.getStringParameter(request, "draftfolder");
 	        String pin = null;
+	        boolean draft = Core.getBooleanParameter(request, "draft");
 	        int templId = Core.getIntParameter(request, "templid");
 	        if (language == null)
 	            language = "en-US";
@@ -103,13 +108,15 @@ public class Incoming {
 	        if (guest == null) {
 	        	Api ap = new Api(cookie, request);
 	        	if (!ap.verifyLogin()) {
-	        		logger.error("not logged into ui");
+	        	    LogAction.setFail();
+	        		logger.info("not logged into ui");
 	                Core.sendFail(response, "not authorized");
 	                return;
 	        	}
 	        	   // Load custom template Id for this user
 	        } else {
 	        	if (sr == null) {  // Reply ID must be sent if not part of UI
+	        	    LogAction.setFail();
 	        		logger.error("Reply ID not sent");
 	        		Core.sendFail(response, "not authorized");
 	                return;
@@ -118,7 +125,8 @@ public class Incoming {
 	        	UserData userdata = new UserData(auth, cookie);
 	        	cid = userdata.cid;
 	        	if (userdata.userid < 0) {
-	        		logger.debug("Bad authorization for guest login");
+	        	    LogAction.setFail();
+	        		logger.info("Bad authorization for guest login");
 	        		Core.sendFail(response, "not authorized");
 	                return;
 	        	}
@@ -129,12 +137,20 @@ public class Incoming {
 	        boolean sendPermission = false;
 	        // Check for permission for sending email
 	        if (guest == null) {
-		        if (!Auth.checkPermission(cookie, request, permissions.MAIL)) {
+	            Settings settings = Auth.getPermissions(cookie, request);
+		        if (!Auth.check(settings, permissions.MAIL)) {
 		            // If not permission, check if secure reply
 		            if (sr == null) {
-		                logger.error("not authorized");
-		                Core.sendFail(response, "not authorized");
-		                return;
+		                if (Auth.checkMinimum(settings)) {  // Make sure they have minimal Guard permission
+		                    // Assuming secure reply from user without guard-mail settings
+		                    // ToDo in future...rate limit, recip limit, other
+		                    sendPermission = true;
+		                } else {
+    		                LogAction.setFail();
+    		                logger.info("not authorized");
+    		                Core.sendFail(response, "not authorized");
+    		                return;
+		                }
 		            }
 		        } else sendPermission = true;
 	        }
@@ -145,8 +161,6 @@ public class Incoming {
 	            templId = OxConfiguration.getTemplateId(cid, userid);
 	        }
 	        
-	        Crypto crypt = new Crypto();
-	        String contentKey = crypt.createContentKey();
 
 	            ArrayList<Attachment> attachments = new ArrayList<Attachment>();// Get list of all attachments
 	            String emaildata = "";
@@ -171,7 +185,7 @@ public class Incoming {
 	                } else {
 	                    name = fi.getName(); // Otherwise, assume it is an attachment
 	                    if (name != null) {// Make sure the name of the file exists, may just be blank field
-	                        if (name.trim() != "") {
+	                        if (!name.trim().equals("")) {
 	                            if (name.lastIndexOf("\\") > -1) {
 	                                name = name.substring(name.lastIndexOf("\\"));
 	                            }
@@ -184,8 +198,7 @@ public class Incoming {
 	                            Attachment attach = new Attachment();
 	                            attach.content = out.toByteArray(); // Populate attachment data
 	                            attach.filename = fi.getName();
-
-	                            attach.type = "grd/" + fi.getContentType().replace("/", "--"); // Add grd prefix
+	                            attach.type = fi.getContentType();
 	                            attachments.add(attach); // Add to our array of attachments
 	                            out.close();
 	                            input.close();
@@ -193,15 +206,16 @@ public class Incoming {
 	                    }
 	                }
 	            }
-	            if (emaildata != "") { // We must have the basic email data
+	            if (!emaildata.equals("")) { // We must have the basic email data
 	                JsonParser jsonParser = new JsonParser(); // Put it into json
 	                JsonObject json = (JsonObject) jsonParser.parse(emaildata);
-	                if (json.get("attachextra") != null) attach_extrapass = json.get("attachextra").getAsString();
 	                if (json.get("attachpassword") != null) attach_password = json.get("attachpassword").getAsString();
 	                JsonObject data = json.getAsJsonObject("data");
-	                ArrayList<RecipKey> recipients = getUsers(data, userid, cid, guestlanguage, pin); // get the users and populate the recipient keys
+	                ArrayList<RecipKey> recipients = getUsers(data, userid, cid, guestlanguage, pin, info); // get the users and populate the recipient keys
+
 	                if (recipients == null) {
-	                    logger.error("Unable to get keys for all users");
+	                    LogAction.setFail();
+	                    logger.info("Unable to get keys for all users");
 	                    Core.sendFail(response, "keyproblem");
 	                    return;
 	                }
@@ -209,7 +223,8 @@ public class Incoming {
 	                if (!sendPermission) {
 	                	JsonArray from = data.get("from").getAsJsonArray().get(0).getAsJsonArray();
 	                	if (!checkReply(from, recipients, info)) {
-	                		logger.error("Attempt to reply to users not in original email list");
+	                	    LogAction.setFail();
+	                		logger.info("Attempt to reply to users not in original email list");
 	                		Core.sendFail(response, "unauth recip");
 	                		return;
 	                	}
@@ -224,7 +239,9 @@ public class Incoming {
 	                		header.remove("X-OxGuard-ExtraPass");
 	                		header.addProperty("X-OxGuard-ExtraPass", "true");
 	                	}
+	                	
 	                } catch (Exception e) {
+	                    LogAction.setFail();
 	                    logger.error("Problem getting extra pass " + e.toString());
 	                }
 
@@ -239,151 +256,40 @@ public class Incoming {
 	                                	attachments.add(attach);
 	                            }
 	                        }
+	                        data.remove("infostore_ids");  // We've added them to our attachments, now remove from the JSON
 	                    }
 	                } catch (Exception ex2) {
+	                    LogAction.setFail();
 	                    logger.error("Problem extracting infostore items.", ex2);
 
 	                }
 	                // Search for inline images, convert to attachments
 	                data = getInlineImages (data, attachments, request, cookie);
 	                
+	                data = getNestedAttachments (data, attachments);
+	                
 	                data = getNestedMessages (data, attachments, request, cookie);
 	                
 	                // Search for attachments that reference a draft email, and aren't actually supplied.
 	                data = getDraftAttachments (data, attachments, request, cookie, userid, cid);
 	                
-	                Gson gson = new Gson();
-	                EncrLib encr = new EncrLib();
-	                String mailId = "<OxGuard-" + EncrLib.getUUID() + ">";
-	                try {
-	                    // Let's encrypt the email
-	                    long exp = getExpDate(data);
-	                    String oxemail = Mail.PrepareOXEmail(data, attachments, exp, mailId);
-	                    if (!extrapass.equals("")) {// if extra password specified, then encrypt first with aes
-	                        logger.debug("encrypting with extra password");
-	                        // oxemail = encr.encryptAES(oxemail, extrapass, itemID, Config.AESKeyLen);
-	                        if (extrapass.length() > 30) {
-		                		if (extrapass.startsWith("E!")) {
-		                			extrapass = encr.decryptRSA(extrapass.substring(2), Core.getPasswordKey());
-		                		}
-		                	}
-	                        if (extrapass != null) oxemail = crypt.addExtraPass(oxemail, extrapass, itemID);
-	                    }
-	                    String encrypted = crypt.EncryptEmail(userid, cid, recipients, oxemail, exp, 1, "", itemID, contentKey, salt);
-	                    if (encrypted == null) {
-	                        com.openexchange.guard.util.Core.sendFail(response, "Failed to send");
-	                        return;
-	                    }
-	                    Attachment main = new Attachment();
-	                    main.encrContent = crypt.addHeader(encrypted, itemID, -1); // Add the header
-	                    main.filename = itemID + ".grd"; // Our email filename is the itemID + .loc
-	                    main.type = "text/plain";
-	                    attachments.add(main); // Add it to our list of attachments for the email
-
-	                } catch (SQLException e) {
-	                    logger.error("Error while trying to encrypt email for item " + itemID, e);
-	                    com.openexchange.guard.util.Core.sendFail(response, "Failed to send");
+	                addVcard (data, attachments, request, cookie, userid, cid);
+	                
+	                if (!getAttachPGPFiles (data, attachments, request, cookie, userid, cid, attach_password)) {
+	                    Core.sendFail(response, "Unable to decode attachments");
 	                    return;
-
 	                }
+	                
+	                // Check if PGP email
+	                
+	                	PGPSender pgpsend = new PGPSender(draft, draftFolder);
+	                	pgpsend.sendPGPMail(json, attachments, attach_password, recipients, guestmessage, pin, templId, request, response, cookie);
 
-	                // Encrypt the attachments, using the same content key as for the email.
-	                for (int i = 0; i < attachments.size(); i++) {
-	                    Attachment attach = attachments.get(i);
-	                    if (attach.encrContent == null) {// Make sure this isn't the email
-	                        // Check if attachments are encrytped objects
-
-	                        if (com.openexchange.guard.util.Core.isEncrItem(attach.content)) {
-	                            try {
-	                                logger.debug("already encrypted");
-	                                EncryptedObj encrAttach = new EncryptedObj(attach.content);
-	                                String decoded = crypt.decodeEncrObj(userid, cid, encrAttach, attach_password);
-	                                if (decoded.startsWith("EXTRA")) {
-	                                    if (attach_extrapass != null) {
-	                                        decoded = crypt.decodeExtraPass(decoded, attach_extrapass, encrAttach.ItemID, encrAttach.keyLength);
-	                                    }
-	                                }
-	                                attach.content = Base64.decode(decoded);
-	                            } catch (Exception ex) {
-	                                logger.debug("Failed to decode attachments");
-	                                com.openexchange.guard.util.Core.sendFail(response, "FailDecodeAttach");
-	                                return;
-	                            }
-
-	                        }
-
-	                        attach.filename += ".grd";
-
-	                        if (!extrapass.equals("")) {
-	                            attach.encrContent = crypt.encrItem(
-	                                userid,
-	                                cid,
-	                                recipients,
-	                                crypt.addExtraPass(Base64.toBase64String(attach.content), extrapass, itemID),
-	                                getExpDate(data),
-	                                2,
-	                                "",
-	                                itemID,
-	                                contentKey,
-	                                true,
-	                                attach.filename);
-	                            // crypt.encrItem(userid, cid, recipients,encr.encryptAES(
-	                            // Base64.toBase64String(attach.content),extrapass,itemID, Config.AESKeyLen) , getExpDate(data), 2, "", itemID,
-	                            // contentKey, true, attach.filename);
-	                            attach.filename = attach.filename.replace(".grd", ".grd2"); // mark the attachment as extra password
-	                        } else {
-	                            attach.encrContent = crypt.encrItem(
-	                                userid,
-	                                cid,
-	                                recipients,
-	                                Base64.toBase64String(attach.content),
-	                                getExpDate(data),
-	                                2,
-	                                "",
-	                                itemID,
-	                                contentKey,
-	                                true,
-	                                attach.filename);
-	                        }
-
-	                        attachments.set(i, attach);
-	                    }
-	                }
-	                // Send the email link message
-	                SendNotice sender = new SendNotice(cookie, request, sessionId);
-	                if (sender.sendEmailNotice(
-	                    recipients,
-	                    data,
-	                    itemID,
-	                    ua,
-	                    attachments,
-	                    !(extrapass.equals("")),
-	                    guest,
-	                    info,
-	                    language,
-	                    cid,
-	                    guestmessage,
-	                    pin,
-	                    header,
-	                    sentfolder,
-	                    mailId,
-	                    templId)) {
-	                    logger.info("Success email send");
-	                    // If was draft email, delete it from the draft folder
-	                    if (data.has("msgref")) {
-	                    	Api ap = new Api(cookie, request);
-	                    	ap.deleteDraft(data.get("msgref").getAsString());
-	                    }
-	                    com.openexchange.guard.util.Core.sendOK(response, "OK");// If OK, send response
-	                } else {
-	                    com.openexchange.guard.util.Core.sendFail(response, "Failed to send");
-	                    logger.debug("Failed to send email");
-	                }
 
 	            } else {// Our reply that the email was blank
+	                LogAction.setFail();
 	                com.openexchange.guard.util.Core.sendOK(response, "No Email Data");
 	            }
-
 	    }
 	   
 	    /**
@@ -393,38 +299,38 @@ public class Incoming {
 	     * @return
 	     * @throws Exception
 	     */
-	    public ArrayList<RecipKey> getUsers(JsonObject json, int senderId, int senderCID, String guestlanguage, String pin) throws Exception {
+	    public ArrayList<RecipKey> getUsers(JsonObject json, int senderId, int senderCID, String guestlanguage, String pin, EmailInfo info) throws Exception {
 
 	        ArrayList<RecipKey> recipients = new ArrayList<RecipKey>();
 
 	        JsonArray recips = json.get("to").getAsJsonArray();
 	        String from = json.get("from").getAsJsonArray().get(0).getAsJsonArray().get(1).getAsString();
 	        String fromName = json.get("from").getAsJsonArray().get(0).getAsJsonArray().get(0).getAsString();
-	        recipients = addUsers(recipients, recips, senderId, senderCID, from, fromName, RecipKey.reciptype.to, guestlanguage, pin);
+	        recipients = addUsers(recipients, recips, senderId, senderCID, from, fromName, RecipientType.TO, guestlanguage, pin, info);
 	        if (recipients == null)
 	            return (null);
 	        recips = json.get("cc").getAsJsonArray();
-	        recipients = addUsers(recipients, recips, senderId, senderCID, from, fromName, RecipKey.reciptype.cc, guestlanguage, pin);
+	        recipients = addUsers(recipients, recips, senderId, senderCID, from, fromName, RecipientType.CC, guestlanguage, pin, info);
 	        if (recipients == null)
 	            return (null);
 	        recips = json.get("bcc").getAsJsonArray();
-	        recipients = addUsers(recipients, recips, senderId, senderCID, from, fromName, RecipKey.reciptype.bcc, guestlanguage, pin);
+	        recipients = addUsers(recipients, recips, senderId, senderCID, from, fromName, RecipientType.BCC, guestlanguage, pin, info);
 
 	        return (recipients);
 
 	    }
 	    
-	    private ArrayList<RecipKey> addUsers(ArrayList<RecipKey> recipients, JsonArray recips, int senderId, int senderCID, String from, String fromName, RecipKey.reciptype type, String guestlanguage, String pin) throws Exception {
+	    private ArrayList<RecipKey> addUsers(ArrayList<RecipKey> recipients, JsonArray recips, int senderId, int senderCID, String from, String fromName, RecipientType type, String guestlanguage, String pin, EmailInfo info) throws Exception {
 	        for (int i = 0; i < recips.size(); i++) {
 	            JsonArray recip = recips.get(i).getAsJsonArray();
 	            String email = recip.get(1).getAsString();
 	            GetPubKey gk = new GetPubKey();
 	            try {
-	                RecipKey key = gk.getPubKey(email, senderId, senderCID, from, true, false, guestlanguage, fromName, pin);
+	                RecipKey key = gk.getPubKey(email, senderId, senderCID, from, true, false, guestlanguage, fromName, pin, info);
 	                if (key != null) {
 	                    if (key.pubkey == null) {
 	                        // Fail to create key
-	                        return (null);
+	                        if (key.pgp == false) return (null);
 	                    }
 	                    key.email = email;
 	                    key.type = type;
@@ -483,7 +389,7 @@ public class Incoming {
 	    
 	    public static Attachment getInfoAttach(JsonObject jsonElement, HttpServletRequest request, OxCookie ck) {
 	    	try {
-		        int id = jsonElement.get("id").getAsInt();
+		        String id = jsonElement.get("id").getAsString();
 		        int version = jsonElement.has("version") ? jsonElement.get("version").getAsInt() : -1;
 		        int folder = jsonElement.get("folder_id").getAsInt();
 		        Api ap = new Api(ck, request);
@@ -503,19 +409,30 @@ public class Incoming {
 	     * @param request
 	     * @return
 	     */
-	    public static JsonObject getDraftAttachments (JsonObject data, ArrayList<Attachment> attachments, HttpServletRequest request, OxCookie ck, int userid, int cid) {
+	    public JsonObject getDraftAttachments (JsonObject data, ArrayList<Attachment> attachments, HttpServletRequest request, OxCookie ck, int userid, int cid) {
 	    	try {
 		    	JsonArray contents = data.get("attachments").getAsJsonArray();
 		    	JsonArray newcontents = new JsonArray();
 		    	for (int i = 0; i < contents.size(); i++) {
 		    		boolean modified = false;
 		    		JsonObject att = contents.get(i).getAsJsonObject();
-		    		if (att.has("ogAttach")) {
-		    			DecodedObj ogAttach = getEncrAttach (att, userid, cid, request, ck);
+		    		if (att.has("epass")) {
+                        String password = Crypto.decodeEPass(att.get("epass").getAsString(), ck);
+		    		    PgpHandler pgph = new PgpHandler();
 		    			Attachment decattach = new Attachment();
-		    			decattach.content = Base64.decode(ogAttach.decoded64);
+		    		    String emailid = att.get("mail").getAsJsonObject().get("id").getAsString();
+		    		    String folder = att.get("mail").getAsJsonObject().get("folder_id").getAsString();
+		    		    boolean pgpmime = true;
+		    		    if (att.has("pgpFormat")) {
+		    		        if (att.get("pgpFormat").getAsString().contains("inline")) {
+		    		            pgpmime = false;
+		    		        }
+		    		    }
+		    			decattach.content = pgpmime ? 
+		    			    pgph.getAttach(request, ck, emailid, folder, userid, cid, att.get("filename").getAsString(), password) :
+		    			        pgph.decodeInlineAttach(emailid, folder, att.get("id").getAsString(), att.get("filename").getAsString(), userid, cid, password, request, ck);
 		    			decattach.type = att.get("content_type").getAsString();
-		    			decattach.filename = att.get("filename").getAsString();
+		    			decattach.filename = att.get("filename").getAsString().replace(".pgp", "");
 		    			attachments.add(decattach);
 		    			modified = true;
 		    		} else {
@@ -524,7 +441,7 @@ public class Incoming {
 			    			String item = att.get("atmsgref").getAsString();
 			    			int j = item.lastIndexOf("/");
 			    			String folder = item.substring(0,j);
-			    			int emailid = Integer.parseInt(item.substring(j+1).trim());
+			    			String emailid = item.substring(j+1).trim();
 			    			Api ap = new Api (ck, request);
 							Attachment attach = ap.getPlainAttachment(emailid, attid, folder);
 							attach.filename = att.get("filename").getAsString();
@@ -543,13 +460,88 @@ public class Incoming {
 	    	return (data);
 	    }
 	    
+	    /**
+	     * If the user forwards an item that was already encrypted (filestore item), then needs to be decoded before re-encrypting
+	     * @param data
+	     * @param attachments
+	     * @param request
+	     * @param cookie
+	     * @param userid
+	     * @param cid
+	     * @param password
+	     * @return
+	     * @throws Exception
+	     */
+	    private boolean getAttachPGPFiles (JsonObject data, ArrayList<Attachment> attachments, HttpServletRequest request, OxCookie cookie, int userid, int cid, String password) throws Exception {
+	        boolean success = true;
+	        for (Attachment att : attachments) {
+	            if (att.filename != null && att.filename.endsWith(".pgp")) {
+	                ByteArrayInputStream in = new ByteArrayInputStream (att.content);
+	                PGPResult result = PGPUtils.decryptFile(in, userid, cid, password);
+	                if (result.error == null) {
+	                    att.content = result.decoded;
+	                    att.filename = att.filename.replace(".pgp", "");
+	                } else {
+	                    success = false;
+	                }
+	            }
+	        }
+	        return (success);
+	        
+	    }
+	    
+	    /**
+	     * Check if vcard should be added.  If so, add to attachments.
+	     * This is only supported 7.62 after July patch
+	     * @param data
+	     * @param attachments
+	     * @param request
+	     * @param cookie
+	     * @param userid
+	     * @param cid
+	     */
+	    private void addVcard (JsonObject data, ArrayList<Attachment> attachments, HttpServletRequest request, OxCookie cookie, int userid, int cid) {
+            if (data.has("vcard")) {
+                if (data.get("vcard").getAsInt() == 1) {
+                    Api ap = new Api(cookie, request);
+                    byte[] vcard = ap.getVcard();
+                    if (vcard != null) {
+                        String filename = "vcard";  // backup name is vcard
+                        String vs = new String(vcard);
+                        String[] lines = vs.split("\n");
+                        for (String line : lines) {  // Parse the lines for the Full Name for the filename of the vcard
+                            if (line.startsWith("FN:")) {
+                                filename = line.replace("FN:", "");
+                            }
+                        }
+                        Attachment vcardAtt = new Attachment(); // Add to attachments
+                        vcardAtt.content = vcard;
+                        vcardAtt.filename = filename + ".vcf";
+                        vcardAtt.type = "text/vcard";
+                        attachments.add(vcardAtt);
+                    }
+                    data.remove("vcard");
+                    data.add("vcard", new JsonPrimitive(0));  // We need to remove the flag so the backend doesn't add it unencrypted.
+                }
+            }
+	    }
+	    
+	    /**
+	     * Depreciated.  Get decode Guard attachment being forwarded
+	     * @param attach
+	     * @param userid
+	     * @param cid
+	     * @param request
+	     * @param ck
+	     * @return
+	     */
 	    private static DecodedObj getEncrAttach (JsonObject attach, int userid, int cid, HttpServletRequest request, OxCookie ck) {
 	    	
 	    	int attachid = attach.get("id").getAsInt();
 	    	String filename = attach.get("filename").getAsString();
-	    	int emailid = attach.get("parent").getAsJsonObject().get("id").getAsInt();
+	    	String emailid = attach.get("parent").getAsJsonObject().get("id").getAsString();
 	    	String folder = attach.get("parent").getAsJsonObject().get("folder_id").getAsString();
-	    	String encrextrapass = attach.get("extrapass").getAsString();
+	    	String encrextrapass = attach.has("extrapass") ? attach.get("extrapass").getAsString() : null;
 	    	String extrapass = null;
 	    	String cc = attach.get("cc") == null ? null : attach.get("cc").getAsString();
 	    	UserData data = new UserData();;
@@ -580,7 +572,7 @@ public class Incoming {
 	    }
 	    
 	    
-	    public static JsonObject getNestedMessages (JsonObject data, ArrayList<Attachment> attachments, HttpServletRequest request, OxCookie ck) {
+	    public  JsonObject getNestedMessages (JsonObject data, ArrayList<Attachment> attachments, HttpServletRequest request, OxCookie ck) {
 	    	try {
 	    		if (!data.has("nested_msgs")) return (data);
 	    		JsonArray nested = data.get("nested_msgs").getAsJsonArray();
@@ -593,7 +585,7 @@ public class Incoming {
 	    			String mime = ap.getMime(message_ref);
 	    			Attachment newatt = new Attachment();
 	    			newatt.filename = subject + ".eml";
-	    			newatt.content = mime.getBytes();
+	    			newatt.content = mime.getBytes("UTF-8");
 	    			newatt.type = "text/html";
 	    			attachments.add(newatt);
 	    			MimeParser mparse = new MimeParser(newatt.content);
@@ -607,42 +599,70 @@ public class Incoming {
 	    	}
 	    }
 	    
-	    public static JsonObject getInlineImages (JsonObject data, ArrayList<Attachment> attachments, HttpServletRequest request, OxCookie ck) {
+	    public  JsonObject getNestedAttachments (JsonObject data, ArrayList<Attachment> attachments) {
+	        try {
+	        JsonArray contents = data.get("attachments").getAsJsonArray();
+	        for (int i = 0; i < contents.size(); i++) {
+	            JsonObject content = contents.get(i).getAsJsonObject();
+	            if (content.has("disp")) {
+	                if (content.get("disp").getAsString().equals("attachment")) {
+	                    if (content.get("content").getAsString().length() > 0) {
+	                        Attachment att = new Attachment();
+	                        att.filename = content.get("filename").getAsString();
+	                        att.content = content.get("content").getAsString().getBytes("UTF-8");
+	                        att.type = content.get("content_type").getAsString();
+	                        attachments.add(att);
+	                    }
+	                }
+	            }
+	        }
+	        } catch (Exception e) {
+	            logger.error("Problem getting nested attachments", e);
+	        }
+	        return (data);
+	    }
+	    
+	    public  JsonObject getInlineImages (JsonObject data, ArrayList<Attachment> attachments, HttpServletRequest request, OxCookie ck) {
 	    	try {
 		    	JsonArray contents = data.get("attachments").getAsJsonArray();
 		    	JsonArray newcontents = new JsonArray();
 		    	for (int i = 0; i < contents.size(); i++) {
-		    		if (contents.get(i).getAsJsonObject().get("content").isJsonNull()) {
+		    		if (contents.get(i).getAsJsonObject().has("content") && contents.get(i).getAsJsonObject().get("content").isJsonNull()) {
 		    			newcontents.add(contents.get(i));
 		    		}
 		    		else {
 		    			JsonObject content = contents.get(i).getAsJsonObject();
-		    			String cont = content.get("content").getAsString();
-		    			int start = cont.indexOf("<img");
-		    			boolean changed = false;
-		    			String newcont = new String(cont);
-		    			while ((start > 0) && (start < cont.length())) {
-		    				int end = cont.indexOf(">", start);
-		    				if (end < 0) break;
-		    				String image = cont.substring(start, end);
-		    				Pattern pattern = Pattern.compile("src=\"[^\"]*");
-		    				Matcher match = pattern.matcher(image);
-		    				if (match.find()) {
-		    					String url = image.substring(match.start() +5, match.end());
-		    					if (url.length() > 1) {
-			    					Api ap = new Api (ck, request);
-			    					Attachment attach = ap.getInlineImage(url);
-			    					attach.type = "grd/" + attach.type.replace("/", "--");
-			    					if (attach != null) attachments.add(attach);
-			    					newcont = newcont.replace(url, "attached");
-			    					changed = true;
-		    					}
-		    				}
-		    				start = cont.indexOf("<img ", end);
-		    			}
-		    			if (changed) {
-		        			content.remove("content");
-		        			content.addProperty("content", newcont);
+		    			if (content.has("content")) {
+    		    			String cont = content.get("content").getAsString();
+    		    			int start = cont.indexOf("<img");
+    		    			boolean changed = false;
+    		    			String newcont = new String(cont);
+    		    			while ((start > 0) && (start < cont.length())) {
+    		    				int end = cont.indexOf(">", start);
+    		    				if (end < 0) break;
+    		    				String image = cont.substring(start, end);
+    		    				Pattern pattern = Pattern.compile("src=\"[^\"]*");
+    		    				Matcher match = pattern.matcher(image);
+    		    				if (match.find()) {
+    		    					String url = image.substring(match.start() +5, match.end());
+    		    					if (url.length() > 1) {
+    		    					    // TODO Handle forwarding of inline Guard images
+    			    					Api ap = new Api (ck, request);
+    			    					Attachment attach = ap.getInlineImage(url);
+    			    					if (attach != null) {
+                                            attach.content_id = EncrLib.getUUID() + "@GUARD";
+                                            attachments.add(attach);
+                                            newcont = newcont.replace(url, "cid:" + attach.content_id);
+    			    					}
+    			    					changed = true;
+    		    					}
+    		    				}
+    		    				start = cont.indexOf("<img ", end);
+    		    			}
+    		    			if (changed) {
+    		        			content.remove("content");
+    		        			content.addProperty("content", newcont);
+    		    			}
 		    			}
 		    			newcontents.add(content);
 		    			}
