@@ -52,15 +52,12 @@ package com.openexchange.guard.mailcreator;
 import java.io.StringReader;
 import java.util.ArrayList;
 import java.util.Date;
-
+import javax.mail.Message.RecipientType;
 import javax.mail.internet.AddressException;
 import javax.mail.internet.InternetAddress;
-
 import net.htmlparser.jericho.Source;
-
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
 import com.google.gson.Gson;
 import com.google.gson.JsonArray;
 import com.google.gson.JsonObject;
@@ -112,6 +109,14 @@ public class Mail {
             return (null);
         return (setContent(email, content));
     }
+    
+    public JsonObject createBlankGuestMail (String lang, int templId, String messageID) {
+        JsonObject mail = createBlankGuestMail(lang, templId);
+        JsonObject header = new JsonObject();
+        header.addProperty("Message-ID", messageID);
+        mail.get("data").getAsJsonObject().add("headers", header);
+        return (mail);
+    }
 
     public JsonObject addPlainText(JsonObject email, String data) {
         Gson gson = new Gson();
@@ -131,16 +136,10 @@ public class Mail {
         return (g.fromJson(emailText, JsonObject.class));
     }
 
-    public JsonObject addTo(JsonObject mail, String name, String email, RecipKey.reciptype type) {
-        switch (type) {
-        case bcc:
-            return (addBCC(mail, name, email));
-        case cc:
-            return (addCC(mail, name, email));
-        case to:
-            return (addTo(mail, name, email));
-
-        }
+    public JsonObject addTo(JsonObject mail, String name, String email, RecipientType type) {
+        if (type == RecipientType.TO) return (addTo(mail, name, email));
+        if (type == RecipientType.CC) return (addCC(mail, name, email));
+        if (type == RecipientType.BCC) return (addBCC(mail, name, email));
         return (null);
     }
 
@@ -307,7 +306,8 @@ public class Mail {
     public static ArrayList <String> getFromAddress(String name, String email, int userid, int cid) {
     	ArrayList <String> answer = new ArrayList <String> ();
     	// Check if Backend has a from address for this person in config cascade
-    	String fromemail = OxConfiguration.getFromEmail(userid, cid);
+    	String fromemail = null;
+    	if (userid != 0) fromemail = OxConfiguration.getFromEmail(userid, cid);
     	// If no config cascade
     	if (fromemail == null) {
     		// Check if we have global configuration
@@ -342,5 +342,16 @@ public class Mail {
     	
     }
     
+    public static String getPlainText (String html) {
+        try {
+            StringReader sr = new StringReader(html);
+            Source src = new Source (sr);
+            return(src.getRenderer().toString());
+        } catch (Exception e) {
+            logger.error("Error converting HTML to plaintext ", e);
+            return(html);
+        }
+    }
+    
 
 }
