@@ -50,34 +50,31 @@
 package com.openexchange.guard.ox;
 
 import java.io.BufferedReader;
-import java.io.FileInputStream;
-import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.UnsupportedEncodingException;
+import java.net.URISyntaxException;
 import java.net.URLEncoder;
 import java.nio.charset.Charset;
 import java.util.ArrayList;
 import java.util.Date;
 import java.util.List;
-
 import javax.servlet.http.HttpServletRequest;
-
 import org.apache.http.HttpEntity;
 import org.apache.http.NameValuePair;
 import org.apache.http.client.ClientProtocolException;
 import org.apache.http.client.CookieStore;
-import org.apache.http.client.HttpClient;
+import org.apache.http.client.NonRepeatableRequestException;
 import org.apache.http.client.entity.UrlEncodedFormEntity;
 import org.apache.http.client.methods.CloseableHttpResponse;
 import org.apache.http.client.methods.HttpGet;
 import org.apache.http.client.methods.HttpPost;
 import org.apache.http.client.methods.HttpPut;
 import org.apache.http.client.protocol.ClientContext;
+import org.apache.http.client.utils.URIBuilder;
 import org.apache.http.entity.ContentType;
 import org.apache.http.entity.StringEntity;
-import org.apache.http.entity.mime.Header;
 import org.apache.http.entity.mime.HttpMultipartMode;
 import org.apache.http.entity.mime.MultipartEntityBuilder;
 import org.apache.http.entity.mime.content.ByteArrayBody;
@@ -90,26 +87,31 @@ import org.apache.http.protocol.HttpContext;
 import org.apache.http.util.EntityUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
 import com.google.gson.Gson;
 import com.google.gson.GsonBuilder;
+import com.google.gson.JsonArray;
 import com.google.gson.JsonObject;
 import com.google.gson.JsonParser;
 import com.openexchange.guard.config.Config;
+import com.openexchange.guard.config.OxConfiguration;
 import com.openexchange.guard.encr.EncryptedObj;
 import com.openexchange.guard.mailcreator.Attachment;
+import com.openexchange.guard.ox.streaming.JsonStreamHandler;
+import com.openexchange.guard.ox.streaming.StreamHandler;
+import com.openexchange.guard.ox.streaming.StreamedContentBody;
 import com.openexchange.guard.server.OxCookie;
-import com.openexchange.guard.server.OxDbConn;
+import com.openexchange.guard.server.connectionPooling.HttpConnectionPoolService;
 import com.openexchange.guard.util.Core;
+import com.openexchange.guard.util.JsonExtractor;
 
 /**
  * API class for communicating with OX servers
- * 
+ *
  * @author greg
  */
 public class Api {
 	private static Logger logger = LoggerFactory.getLogger(Api.class);
-	
+
     HttpContext httpContext = null;
 
     CloseableHttpClient httpClient = null;
@@ -122,7 +124,7 @@ public class Api {
 
     /**
      * Constructor for api. Logs in, and gets httpContext and Session ID
-     * 
+     *
      * @param username
      * @param password
      */
@@ -132,7 +134,7 @@ public class Api {
 
     /**
      * Constructor from cookie
-     * 
+     *
      * @param ck cookie
      * @param sess sessionId
      * @param ua User-agent
@@ -160,14 +162,15 @@ public class Api {
 
     /**
      * Login and populate the httpContext and SessionID
-     * 
+     *
      * @param username
      * @param password
      * @return
      */
     public String login(String username, String password) {
         // httpClient = new DefaultHttpClient();
-    	CloseableHttpClient httpClient = OxDbConn.httpClient;
+    	//CloseableHttpClient httpClient = OxDbConn.httpClient;
+        CloseableHttpClient httpClient = HttpConnectionPoolService.getCookielessClient();
         CookieStore cookieStore = new BasicCookieStore();
         httpContext = new BasicHttpContext();
         httpContext.setAttribute(ClientContext.COOKIE_STORE, cookieStore);
@@ -199,11 +202,16 @@ public class Api {
         return ("");
     }
 
+    public OxCookie getCookie(){
+        return this.cookie;
+    }
+
     public CloseableHttpResponse getResponse(String params) throws IllegalStateException, IOException {
 
         if (httpClient == null) {
             // httpClient = new DefaultHttpClient();
-        	httpClient = OxDbConn.httpClient;
+        	//httpClient = OxDbConn.httpClient;
+        	httpClient = HttpConnectionPoolService.getCookielessClient();
         }
 
         HttpGet getRequest = new HttpGet("http" + (Config.backend_ssl ? "s" : "") + "://" + Config.getOxBackend() + ":" + Config.ox_backend_port + Config.OXBackendPath + params + "&session=" + session);
@@ -214,6 +222,7 @@ public class Api {
             getRequest.addHeader("Cookie", cookie.getCookieHeader());
             getRequest.setHeader("User-Agent", useragent);
             response = httpClient.execute(getRequest);
+
         } else { // If logging into by previous login httpcontext, then use that
             response = httpClient.execute(getRequest, httpContext);
         }
@@ -233,7 +242,8 @@ public class Api {
 
         if (httpClient == null) {
             // httpClient = new DefaultHttpClient();
-            httpClient = OxDbConn.httpClient;
+            //httpClient = OxDbConn.httpClient;
+            httpClient = HttpConnectionPoolService.getCookielessClient();
         }
 
         HttpPut putRequest = new HttpPut("http" + (Config.backend_ssl ? "s" : "") + "://" + Config.getOxBackend() + ":" + Config.ox_backend_port + Config.OXBackendPath + params + "&session=" + session);
@@ -269,7 +279,8 @@ public class Api {
 
         if (httpClient == null) {
             // httpClient = new DefaultHttpClient();
-            httpClient = OxDbConn.httpClient;
+            //httpClient = OxDbConn.httpClient;
+            httpClient = HttpConnectionPoolService.getCookielessClient();
         }
         HttpPost postRequest = new HttpPost("http" + (Config.backend_ssl ? "s" : "") + "://" + Config.getOxBackend() + ":" + Config.ox_backend_port + Config.OXBackendPath + params);
 
@@ -307,7 +318,7 @@ public class Api {
 
     /**
      * For uploading encrypted files to server
-     * 
+     *
      * @param data The json with folder, etc
      * @param params
      * @param attachments List of files, encrypted
@@ -319,7 +330,8 @@ public class Api {
 
         if (httpClient == null) {
             // httpClient = new DefaultHttpClient();
-            httpClient = OxDbConn.httpClient;
+            //httpClient = OxDbConn.httpClient;
+            httpClient = HttpConnectionPoolService.getCookielessClient();
         }
         MultipartEntityBuilder entity = MultipartEntityBuilder.create();
         entity.setMode(HttpMultipartMode.BROWSER_COMPATIBLE);
@@ -351,7 +363,7 @@ public class Api {
 
     /**
      * Multipart post for email
-     * 
+     *
      * @param data
      * @param params
      * @param attachments
@@ -364,18 +376,21 @@ public class Api {
         long start = System.currentTimeMillis();
         if (httpClient == null) {
             // httpClient = new DefaultHttpClient();
-            httpClient = OxDbConn.httpClient;
+            //httpClient = OxDbConn.httpClient;
+            httpClient = HttpConnectionPoolService.getCookielessClient();
         }
         MultipartEntityBuilder entity = MultipartEntityBuilder.create();
         entity.setMode(HttpMultipartMode.BROWSER_COMPATIBLE);
-        entity.addTextBody("json_0", data, ContentType.APPLICATION_JSON);
+        // Add the email json
+        if (data != null) {
+            entity.addTextBody("json_0", data, ContentType.APPLICATION_JSON);
+        }
         entity.setCharset(Charset.forName("UTF-8"));
         int attachcount = 0;
-        if (attachments != null) {
+        if (attachments != null) {  // Add all the attachments to the body
             for (Attachment attach : attachments) {
-                ContentType type = ContentType.create(attach.type);
+                ContentType type = ContentType.create(attach.type + "\r\nX-Forced-MIME-Type:true");
                 ContentBody file = new ByteArrayBody(attach.encrContent, type, attach.filename);
-                // ContentBody file = new ByteArrayBody(attach.encrContent, attach.type, attach.filename);
                 entity.addPart("file_" + attachcount++, file);
             }
         }
@@ -429,6 +444,30 @@ public class Api {
         return (null);
     }
 
+    public String sendMimeMail (String data, String folder) throws UnsupportedEncodingException {
+    	String params = "mail?action=new";
+        params += "&session=" + session;
+        if (folder != null) {
+        	params += "&folder=" + URLEncoder.encode(folder, "UTF-8");
+        	params += "&flags=32"; //Marks the email as "read" while putting it in the folder
+        }
+        Gson gson = new GsonBuilder().disableHtmlEscaping().create();
+
+        try {
+        	CloseableHttpResponse response = putRequest(params, data);
+            HttpEntity ent = response.getEntity();
+            String dat = EntityUtils.toString(ent);
+            response.close();
+        	return (dat);
+        } catch (ClientProtocolException e) {
+            logger.error("Error sending mail", e);
+
+        } catch (IOException e) {
+        	logger.error("Error sending mail", e);
+        }
+        return (null);
+    }
+
     public JsonObject saveSentMail(JsonObject mail, ArrayList<Attachment> attachments, String sentfolder) {
 	        String params = "mail?action=new";
 	        try {
@@ -446,7 +485,7 @@ public class Api {
 	        }
         return (null);
     }
-    
+
     public JsonObject saveDraftMail(JsonObject mail, ArrayList<Attachment> attachments) {
         String params = "mail?action=new&session=" + session;
         Gson gson = new Gson();
@@ -476,9 +515,9 @@ public class Api {
     }
 
 
-    public EncryptedObj getAttachment(int id, int attachment, String folder) {
+    public EncryptedObj getAttachment(String id, int attachment, String folder) throws UnsupportedEncodingException {
         long start = System.currentTimeMillis();
-        String params = "mail?action=attachment" + "&folder=" + URLEncoder.encode(folder) + "&id=" + id + "&attachment=" + attachment;
+        String params = "mail?action=attachment" + "&folder=" + URLEncoder.encode(folder) + "&id=" + URLEncoder.encode(id, "UTF-8") + "&attachment=" + attachment;
         try {
             CloseableHttpResponse response = getResponse(params);
             HttpEntity ent = response.getEntity();
@@ -497,10 +536,10 @@ public class Api {
         }
         return (null);
     }
-    
-    public Attachment getPlainAttachment (int id, String attachment, String folder) {
+
+    public Attachment getPlainAttachment (String id, String attachment, String folder) throws UnsupportedEncodingException {
     	long start = System.currentTimeMillis();
-        String params = "mail?action=attachment" + "&folder=" + URLEncoder.encode(folder) + "&id=" + id + "&attachment=" + attachment;
+        String params = "mail?action=attachment" + "&folder=" + URLEncoder.encode(folder, "UTF-8") + "&id=" + URLEncoder.encode(id, "UTF-8") + "&attachment=" + attachment;
         try {
             CloseableHttpResponse response = getResponse(params);
             HttpEntity ent = response.getEntity();
@@ -517,23 +556,18 @@ public class Api {
         return (null);
     }
 
-    public String getMime (String ref) {
+    public String getMime (String ref) throws UnsupportedEncodingException {
     	int i = ref.lastIndexOf(("/"));
-    	if (i < -1) return (null);
+    	if (i < -1) {
+            return (null);
+        }
     	String folder = ref.substring(0, i);
     	String id = ref.substring(i+1);
-    	int emailid;
-    	try {
-    		emailid = Integer.parseInt(id);
-    	} catch (Exception ex) {
-    		logger.error("Error parsing email id from msgref " + ref);
-    		return (null);
-    	}
-    	return getMime(emailid, folder);
+    	return getMime(id, folder);
     }
-    
-    public String getMime(int id, String folder) {
-        String params = "mail?action=get" + "&folder=" + URLEncoder.encode(folder) + "&id=" + id + "&src=1&save=1";
+
+    public String getMime(String id, String folder) throws UnsupportedEncodingException {
+        String params = "mail?action=get" + "&folder=" + URLEncoder.encode(folder, "UTF-8") + "&id=" + URLEncoder.encode(id, "UTF-8") + "&src=1&save=1";
         try {
             CloseableHttpResponse response = getResponse(params);
             HttpEntity ent = response.getEntity();
@@ -546,8 +580,8 @@ public class Api {
         }
     }
 
-    public String getMail(int id, String folder) {
-        String params = "mail?action=get" + "&folder=" + URLEncoder.encode(folder) + "&id=" + id;
+    public String getMail(String id, String folder) throws UnsupportedEncodingException {
+        String params = "mail?action=get" + "&folder=" + URLEncoder.encode(folder, "UTF-8") + "&id=" + URLEncoder.encode(id, "UTF-8");
         try {
             CloseableHttpResponse response = getResponse(params);
             HttpEntity ent = response.getEntity();
@@ -558,9 +592,9 @@ public class Api {
             logger.error("Error getting mail " + id + " from folder " + folder, ex);
             return ("");
         }
-        
+
     }
-    
+
     public int getContextId () {
     	String params = "config/context_id?session=" + session;
 		try {
@@ -645,8 +679,9 @@ public class Api {
         JsonObject json = (JsonObject) parser.parse(folders);
         if (json.get("data") != null) {
             return (true);
-        } else
+        } else {
             return (false);
+        }
     }
 
     public String updateFile(Attachment file, String data, int id, int folder) throws UnsupportedEncodingException {
@@ -669,17 +704,17 @@ public class Api {
         return ("");
     }
 
-    public JsonObject getInfoItemJson (int id, int folder) {
+    public JsonObject getInfoItemJson (String id, int folder) throws UnsupportedEncodingException {
     	String json = getInfoItemJson (id, folder, -1);
     	JsonParser parser = new JsonParser();
         JsonObject result = (JsonObject) parser.parse(json);
         return(result);
     }
-    
-    public String getInfoItemJson(int id, int folder, int version) {
-    	String params = "files?action=get&id=" + id + "&folder=" + folder + "&version=" + version + "&session=" + session;
+
+    public String getInfoItemJson(String id, int folder, int version) throws UnsupportedEncodingException {
+    	String params = "files?action=get&id=" + URLEncoder.encode(id, "UTF-8") + "&folder=" + folder + "&version=" + version + "&session=" + session;
     	if (version == -1) {
-    		params = "files?action=get&id=" + id + "&folder=" + folder + "&session=" + session;
+    		params = "files?action=get&id=" + URLEncoder.encode(id, "UTF-8") + "&folder=" + folder + "&session=" + session;
     	}
         try {
             CloseableHttpResponse response = getResponse(params);
@@ -693,6 +728,37 @@ public class Api {
         return (null);
     }
 
+    /**
+     * Get the default vcard from the backend
+     * @return
+     */
+    public byte[] getVcard () {
+        String params = "contacts?action=getVcard&session=" + session;
+        try {
+            CloseableHttpResponse response = getResponse(params);
+            HttpEntity ent = response.getEntity();
+            byte[] data = EntityUtils.toByteArray(ent);
+            response.close();
+            return (data);
+        } catch (Exception ex) {
+            logger.error("Error getting vcard ", ex);
+        }
+        return (null);
+    }
+
+    public JsonObject importVcard (Attachment vcard, String folder) {
+        String params = "import?action=VCARD&folder=" + URLEncoder.encode(folder) + "&session=" + session;
+        try {
+            ArrayList<Attachment> atts = new ArrayList<Attachment>();
+            atts.add(vcard);
+            JsonObject response = multipartPost(null, params, atts);
+            return (response);
+        } catch (Exception ex) {
+            logger.error("Error importing vcard ", ex);
+        }
+        return (null);
+    }
+
     public byte[] getPlainTextInfoStore(int id, int folder, int version) {
         String params = "files?action=document&id=" + id + "&folder=" + folder + "&version=" + version + "&session=" + session;
         try {
@@ -707,18 +773,80 @@ public class Api {
         return (null);
     }
 
+    /**
+     * Opens a remote stream to an infostore/drive item
+     * @param id The id of the item to open
+     * @param folder the if of the item's folder
+     * @param version the version of the item
+     * @return a stream to the remote item
+     * @throws URISyntaxException
+     */
+    public InputStream openDriveItem(String fileId,
+        long fileSize,
+        int folderId,
+        int fileVersion) throws URISyntaxException, IOException {
+
+        //building the uri
+        URIBuilder builder = new URIBuilder().
+            setScheme("http" + (Config.backend_ssl ? "s" : "")).
+            setHost(Config.getOxBackend()).
+            setPort(Config.ox_backend_port).
+            setPath("/ajax/files").
+            setParameter("action", "document").
+            setParameter("id", fileId).
+            setParameter("folder",String.valueOf(folderId)).
+            setParameter("session",String.valueOf(session));
+        if(fileVersion > -1) {
+            builder.setParameter("version",String.valueOf(fileVersion));
+        }
+
+        //Sending request
+        HttpGet getRequest = new HttpGet(builder.build());
+        getRequest.addHeader("accept", "application/json");
+        getRequest.setHeader("Connection", "close");
+        CloseableHttpResponse response = null;
+        if(httpContext == null){
+            //using the given cookie
+            getRequest.addHeader("Cookie", cookie.getCookieHeader());
+            getRequest.setHeader("User-Agent", useragent);
+            response = httpClient.execute(getRequest);
+        }
+        else {
+            //using the current HTTP-context
+            response = httpClient.execute(getRequest,httpContext);
+        }
+
+        //Checking response
+        Integer returnStatus = response.getStatusLine().getStatusCode();
+        if(returnStatus.toString().startsWith("2") /* Status 2xx Success */) {
+            HttpEntity responseEntity = response.getEntity();
+            return responseEntity.getContent();
+        }
+        else{
+            //Error handling
+            response.close();
+            getRequest.releaseConnection();
+            throw new IOException(
+                String.format("Error reading input stream from remote drive item id %s folder %d. HTTP-Error: %s",
+                              fileId,
+                              folderId,
+                              response.toString()));
+        }
+    }
+
     /**
      * Retrive file item from infostore
-     * 
+     *
      * @param id ID of the item
      * @param folder folder id
      * @param version version id
      * @return
+     * @throws UnsupportedEncodingException
      */
-    public EncryptedObj getInfoStore(int id, int folder, int version) {
-        String params = "files?action=document&id=" + id + "&folder=" + folder + "&version=" + version + "&session=" + session;
+    public EncryptedObj getInfoStore(String id, int folder, int version) throws UnsupportedEncodingException {
+        String params = "files?action=document&id=" + URLEncoder.encode(id, "UTF-8") + "&folder=" + folder + "&version=" + version + "&session=" + session;
         if (version == -1) {
-        	params = "files?action=document&id=" + id + "&folder=" + folder + "&session=" + session;
+        	params = "files?action=document&id=" + URLEncoder.encode(id, "UTF-8") + "&folder=" + folder + "&session=" + session;
         }
         long start = System.currentTimeMillis();
         try {
@@ -741,21 +869,36 @@ public class Api {
 
     /**
      * Retrieve non-encrypted/encrypted infostore object as attachment
-     * 
+     *
      * @param id
      * @param folder
      * @param version
      * @return
+     * @throws UnsupportedEncodingException
      */
-    public Attachment getInfoStoreAttach(int id, int folder, int version) {
-        String params = "files?action=document&id=" + id + "&folder=" + folder + ((version >= 0) ? ("&version=" + version) : "") + "&session=" + session;
+    public Attachment getInfoStoreAttach(String id, int folder, int version) throws UnsupportedEncodingException {
+        String params = "files?action=document&id=" + URLEncoder.encode(id, "UTF-8") + "&folder=" + folder + ((version >= 0) ? ("&version=" + version) : "") + "&session=" + session;
         try {
             Attachment attach = new Attachment();
             CloseableHttpResponse response = getResponse(params);
             HttpEntity ent = response.getEntity();
             attach.type = ent.getContentType().toString();
             int i = attach.type.indexOf(":");
-            if (i > 0) attach.type = attach.type.substring(i + 1).trim();
+            if (i > 0) {
+                attach.type = attach.type.substring(i + 1).trim();
+                if (attach.type.contains("application/pgp-signature")) {
+                    // pgp attachment.  Type renamed by backend
+                    JsonObject json = getInfoItemJson (id, folder);
+                    try {
+                        JsonObject meta = json.get("data").getAsJsonObject().get("meta").getAsJsonObject();
+                        if (meta.has("OrigMime")) {
+                            attach.type = meta.get("OrigMime").getAsString();
+                        }
+                    } catch (Exception ex) {
+                        logger.error("Error getting mime type of infostore pgp file", ex);
+                    }
+                }
+            }
             attach.content = EntityUtils.toByteArray(ent);
             response.close();
             return (attach);
@@ -770,8 +913,12 @@ public class Api {
     		url = java.net.URLDecoder.decode(url, "UTF-8").replace("&amp;","&");
     		if (url.contains("file?action") || url.contains("image/mail")) {
     			int index = 0;
-    			if (url.contains("file?action")) index = url.indexOf("file?action");
-    			if (url.contains("image/mail")) index = url.indexOf("image/mail");
+    			if (url.contains("file?action")) {
+                    index = url.indexOf("file?action");
+                }
+    			if (url.contains("image/mail")) {
+                    index = url.indexOf("image/mail");
+                }
 	        	String params = url.substring(index);
 	            Attachment attach = new Attachment();
 	            CloseableHttpResponse response = getResponse(params);
@@ -812,7 +959,7 @@ public class Api {
         }
         return (dat);
     }
-    
+
     public String getConfig (String module) {
     	String params = "config/" + module + "?session=" + session;
         String dat = null;
@@ -826,8 +973,44 @@ public class Api {
         }
         return (dat);
     }
-    
-    public void deleteItem(int id, int folder) {
+
+    public String getSentFolder () {
+        String config = getConfig("mail");
+        JsonObject data = Core.getJSON(config);
+        return (data.get("data").getAsJsonObject().get("folder").getAsJsonObject().get("sent").getAsString());
+    }
+
+    public ArrayList<String> getUserEmails () {
+        String params = "account?action=all&columns=1007&session=" + session;
+        String dat = null;
+        ArrayList<String> e = new ArrayList<String> ();
+        try {
+            CloseableHttpResponse response = getResponse(params);
+            HttpEntity ent = response.getEntity();
+            dat = EntityUtils.toString(ent);
+            response.close();
+            JsonObject json = Core.getJSON(dat);
+            JsonArray emails = json.get("data").getAsJsonArray();
+            for (int i = 0; i < emails.size(); i++) {
+                e.add(emails.get(i).getAsString());
+            }
+        } catch (Exception ex) {
+            logger.error("Error retrieving configuration", ex);
+        }
+        return (e);
+    }
+
+    public String getPrimary () {
+        String data = getConfig("mail");
+        JsonObject json = Core.getJSON(data);
+        return (json.get("data").getAsJsonObject().get("defaultaddress").getAsString());
+    }
+
+    public void deleteItem(String id, int folder) {
+        deleteItem (id, Integer.toString(folder));
+    }
+
+    public void deleteItem (String id, String folder) {
         String params = "files?action=delete&hardDelete=true&timestamp=2116800000000";
         try {
             String data = "[{\"id\":\"" + id + "\",\"folder\":\"" + folder + "\"}]";
@@ -839,36 +1022,90 @@ public class Api {
             logger.error("Error deleting item " + id + " in folder " + folder, e);
         }
     }
-    
-    public void deleteDraft(String msgref) {
+
+    public void deleteMail (String id, String folder) {
+        String params = "mail?action=delete&session="+session;
+        try {
+            String data = "[{\"id\":\"" + id + "\",\"folder\":\"" + folder + "\"}]";
+            CloseableHttpResponse response = putRequest(params, data);
+            HttpEntity ent = response.getEntity();
+            String dat = EntityUtils.toString(ent);
+            response.close();
+        } catch (Exception e) {
+            logger.error("Error deleting item " + id + " in folder " + folder, e);
+        }
+    }
+
+    public void handleRef(String msgref, boolean reply, int userid, int cid) {
     	int i = msgref.lastIndexOf("/");
     	if (i>0) {
     		try {
 	    		String defaultDraft = getConfig("mail/folder/drafts");
 	            JsonParser parser = new JsonParser();
 	            JsonObject draft = (JsonObject) parser.parse(defaultDraft);
-	    		if (!msgref.startsWith(draft.get("data").getAsString())) return;  // If not from default draft folder, then exit (reply or forward)
-	    		String id = msgref.substring(i + 1);
-	    		String folder = msgref.substring(0, i);
-	    		
-	    		String params = "mail?action=delete&timestamp=2116800000000";
-	    		JsonObject data = new JsonObject();
-	    		data.addProperty("folder", folder);
-	    		data.addProperty("id", id);
-	    		try {
-					putRequest(params, "[" + data.toString() + "]");
-				} catch (IllegalStateException | IOException e) {
-					logger.error("Problem deleting from draft", e);
-				}
+	            String id = msgref.substring(i + 1);
+	            String folder = msgref.substring(0, i);
+	            if (reply) {
+	                // mark as reply
+	                setReplied (id, folder);
+	                return;
+	            }
+	    		if (msgref.startsWith(draft.get("data").getAsString()))
+                {
+	    		    if (OxConfiguration.deleteDraft(userid, cid)) {
+	    		        // If draft folder, then delete
+	    		        deleteDraft (id, folder);
+	    		    }
+	    		    return;
+                }
+	    		// otherwise, mark as forward
+	    		setForward (id, folder);
+
     		} catch (Exception ex) {
     			logger.error("Problem getting draft folder ", ex);
     			return;
     		}
     	}
     }
-    
+
+    public void setReplied (String id, String folder) throws UnsupportedEncodingException {
+        setFlag (id, folder, 1);
+    }
+
+    public void setForward (String id, String folder) throws UnsupportedEncodingException {
+        setFlag (id, folder, 256);
+    }
+
+    // Set mail flag
+    public void setFlag (String id, String folder, int flag) throws UnsupportedEncodingException {
+        String params = "mail?action=update&id=" + URLEncoder.encode(id, "UTF-8") + "&folder=" + URLEncoder.encode(folder, "UTF-8");
+        JsonObject data = new JsonObject();
+        data.addProperty("flags", flag);
+        data.addProperty("value", true);
+        try {
+            putRequest(params, data.toString());
+        } catch (Exception e) {
+            logger.error("Problem marking as read", e);
+        }
+    }
+
+    // Delete message from draft folder
+    public void deleteDraft (String id, String folder) {
+         String params = "mail?action=delete&timestamp=2116800000000";
+        JsonObject data = new JsonObject();
+        data.addProperty("folder", folder);
+        data.addProperty("id", id);
+        try {
+            putRequest(params, "[" + data.toString() + "]");
+        } catch (IllegalStateException | IOException e) {
+            logger.error("Problem deleting from draft", e);
+        }
+    }
+
     private String checkCR (String data) {
-    	if (data == null) return (data);
+    	if (data == null) {
+            return (data);
+        }
     	try {
     		return (data.replace("\r", "").replace("\n", ""));
     	} catch (Exception ex) {
@@ -877,4 +1114,107 @@ public class Api {
     	}
     }
 
+    /**
+     * Uploads an input stream to drive
+     *
+     * @param inputStream The InputStream to upload
+     * @param streamHandler A handler to define a operation
+     *                      performed on the stream data during upload
+     * @param fileName The new name of the file
+     * @param jsonContent The new meta data content of the file
+     * @param contentType The new content type of the file
+     * @return The JSON response from the backend if the upload was successful
+     * @throws Exception on an error
+     */
+    public JsonObject UploadDriveStreamItem(InputStream inputStream,
+                                             StreamHandler streamHandler,
+                                             String fileName,
+                                             ContentType contentType,
+                                             InputStream jsonStream,
+                                             JsonStreamHandler jsonHandler) throws Exception {
+        try {
+            if (httpClient == null) {
+                //httpClient = OxDbConn.httpClient;
+                httpClient = HttpConnectionPoolService.getCookielessClient();
+            }
+
+            //Multipart entity
+            MultipartEntityBuilder entity = MultipartEntityBuilder.create();
+            entity.setMode(HttpMultipartMode.BROWSER_COMPATIBLE);
+            entity.addPart("file_0",
+                           new StreamedContentBody(inputStream,
+                                                   4096,
+                                                   contentType,
+                                                   streamHandler,
+                                                   fileName));
+            entity.addPart("json",
+                           new StreamedContentBody(jsonStream,
+                                                   4096,
+                                                   ContentType.TEXT_PLAIN,
+                                                   jsonHandler,
+                                                   null));
+
+
+
+            //Posting the data
+            String params = "files?action=new&filename=" +
+                             URLEncoder.encode(fileName) +
+                             "&session=" + this.session;
+            HttpPost postRequest =
+                new HttpPost("http" +
+                             (Config.backend_ssl ? "s" : "") +
+                             "://" +
+                             Config.getOxBackend() +
+                             ":" +
+                             Config.ox_backend_port +
+                             Config.OXBackendPath +
+                             params);
+            postRequest.setEntity(entity.build());
+            postRequest.addHeader("Connection", "Keep-Alive");
+            CloseableHttpResponse response = null;
+            if (httpContext == null) {// If login in by cookie, add header and execute
+                postRequest.addHeader("cookie", cookie.getCookieHeader());
+                postRequest.setHeader("User-Agent", useragent);
+                response = httpClient.execute(postRequest);
+            } else { // If logging into by previous login httpcontext, then use that
+                response = httpClient.execute(postRequest, httpContext);
+            }
+
+            Integer returnStatus = response.getStatusLine().getStatusCode();
+            BufferedReader reader = new BufferedReader(new InputStreamReader(response.getEntity().getContent(), "UTF-8"));
+            String result = reader.readLine();
+            reader.close();
+            response.close();
+            postRequest.releaseConnection();
+            if(!returnStatus.toString().startsWith("2") /* Status 2xx Success */) {
+                throw new IOException(String.format("Error uploading file. HTTP Error %s",returnStatus));
+            }
+            //Getting the new file ID
+            JsonObject jsonResponse = new JsonExtractor().extractObject(result);
+            if(jsonResponse.has("data")) {
+                return jsonResponse;
+            }
+            else if(jsonResponse.has("error")){
+                String errorMessage = jsonResponse.get("error").getAsString();
+                String errorDesc = jsonResponse.get("error_desc").getAsString();
+                String errorCode = jsonResponse.get("code").getAsString();
+                throw new IOException(String.format("Error while uploading file. %s, %s. Error Code %s",
+                                      errorMessage,
+                                      errorDesc,
+                                      errorCode));
+            }
+            else {
+                throw new IOException("No data in json");
+            }
+        } catch (ClientProtocolException clientProtocolException) {
+            Throwable cause = clientProtocolException.getCause();
+            if(cause != null && cause instanceof  NonRepeatableRequestException) {
+                Throwable ioException = cause.getCause();
+                if(ioException != null && ioException instanceof IOException) {
+                    throw (IOException)ioException;
+                }
+            }
+            throw clientProtocolException;
+        }
+    }
 }
