@@ -137,7 +137,7 @@ public class InterOx {
 
     private InterOxInfo incache(String domain) {
         InterOxInfo info = null;
-        String check = "SELECT * FROM iox WHERE DOMAIN = ?";
+        String check = "SELECT domain, url, token, lastcheck FROM iox WHERE DOMAIN = ?";
         try {
             DbCommand com = new DbCommand(check);
             com.addVariables(domain);
