@@ -49,22 +49,16 @@
 
 package com.openexchange.guard.server;
 
-import java.io.BufferedReader;
 import java.io.IOException;
-import java.net.URLEncoder;
-import java.security.PrivateKey;
-
 import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
-
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
+import org.slf4j.MDC;
 import com.google.gson.Gson;
 import com.google.gson.GsonBuilder;
 import com.google.gson.JsonObject;
-import com.google.gson.JsonParser;
 import com.google.gson.JsonPrimitive;
 import com.openexchange.guard.config.Config;
 import com.openexchange.guard.config.OxConfiguration;
@@ -77,19 +71,22 @@ import com.openexchange.guard.database.RecipKey;
 import com.openexchange.guard.database.Sharding;
 import com.openexchange.guard.encr.EncrLib;
 import com.openexchange.guard.encr.GuardKeys;
-import com.openexchange.guard.encr.PgpKeys;
+import com.openexchange.guard.encr.RSAGen;
 import com.openexchange.guard.exceptions.BadPasswordException;
 import com.openexchange.guard.exceptions.GuardMissingParameter;
+import com.openexchange.guard.logging.LogAction;
+import com.openexchange.guard.logging.LogMarkers;
 import com.openexchange.guard.mailcreator.Mail;
 import com.openexchange.guard.ox.Api;
 import com.openexchange.guard.ox.Languages;
 import com.openexchange.guard.ox.Upgrader;
+import com.openexchange.guard.pgp.PgpKeys;
 import com.openexchange.guard.util.Core;
 import com.openexchange.guard.validator.EmailValidator;
 
 
 public class Auth {
-	
+
 	private static Logger logger = LoggerFactory.getLogger(Auth.class);
 
 
@@ -122,7 +119,7 @@ public class Auth {
                 name = json.get("name").getAsString();
             }
             Keymanager keyman = new Keymanager();
-            keys = keyman.createMasterKeys(userid, cid, email, name, password, language, true); // Flag true for user created
+            keys = keyman.createMasterKeys(userid, cid, email, name, password, language, true, Keymanager.checkCreateRecovery(cookie, request)); // Flag true for user created
             if (keys == null) {
                 logger.debug("Failed to create master key");
                 response.getWriter().write(" {\"result\":\"fail \"} ");
@@ -133,9 +130,38 @@ public class Auth {
             			acc.storeQuestion(userid, cid, "e", recEmail);
             		}
             	}
+            	LogAction.setSuccess(true);
+            	logger.info("Created master key for user");
                 logger.debug("Success creating master key");
                 response.getWriter().write(" {\"result\":\"ok\"} ");
             }
+        } else {  // Keys already exist, adding additional
+            boolean adding = Core.getBooleanParameter(request, "add");
+            if (adding) {
+                String email = acc.getEmail(userid, cid);
+                String name = email;
+                if (json.get("name") != null) {
+                    name = json.get("name").getAsString();
+                }
+
+                RSAGen gen = new RSAGen();
+                GuardKeys newkey = gen.generateKeyPair(name, email, password);
+                newkey.contextid = keys.contextid;
+                newkey.userid = keys.userid;
+                String recovery = Keymanager.createRecovery(newkey, password);
+                //acc.addNewKey(newkey, keys, recovery);
+                newkey.recovery = recovery;
+                acc.addNewKey(newkey);
+                LogAction.setSuccess(true);
+                logger.info("Added additional key for user");
+                Core.sendOK(response, " {\"result\":\"ok\"} ");
+            } else {
+                LogAction.setFail();
+                logger.info("Fail creating key, already exists and add flag not set");
+                Core.sendFail(response, "Existing");
+                return;
+            }
+
         }
     }
 
@@ -198,33 +224,43 @@ public class Auth {
         PgpKeys keygen = new PgpKeys();
         keys = keygen.changePassword(oldpass, newpass, keys);
         if (keys == null) {
-            logger.debug("Bad password for change");
+            LogAction.setFail();
+            logger.info("Bad password for change");
             response.getWriter().write("{\"auth\":\"Bad password\"}");
             CheckBad.addBad(cookie.JSESSIONID);
             return (null);
         } else {
-            Keymanager keyman = new Keymanager();
-            String recovery = keyman.createRecovery(keys, newpass);
+            String recovery = "";
+            boolean recoveryAvail = false;
+            if (Keymanager.checkCreateRecovery(cookie, request)) {
+                recoveryAvail = true;
+                recovery = Keymanager.createRecovery(keys, newpass);
+            }
             acc.updateKeys(userid, cid, keys, recovery, false);
             if (!answer.equals("")) {// If sent with question and answer, then store
-                if (question.equals(""))
+                if (question.equals("")) {
                     question = "default";
+                }
                 acc.storeQuestion(userid, cid, question, answer);
             }
             // String sessionID = json.get("sessionID").getAsString();
             String token = acc.getToken(cookie.JSESSIONID);
-            if (token == null) token = acc.newToken(cookie.JSESSIONID, userid, cid);
+            if (token == null) {
+                token = acc.newToken(cookie.JSESSIONID, userid, cid);
+            }
             json.add("encr_password", new JsonPrimitive(newpass));
             EncrLib encr = new EncrLib();
-            String returndata = " {\"auth\":\"" + encr.encryptAES(json.toString(), token) + "\"} ";
-            logger.debug(returndata);
-            Core.sendOK(response, returndata);
+            JsonObject returndata = new JsonObject();
+            returndata.addProperty("auth", encr.encryptAES(json.toString(), token));
+            returndata.addProperty("recovery", recoveryAvail);
+            Core.sendOK(response, returndata.toString());
+            LogAction.setSuccess();
             logger.info("OK Password Change");
-            return (returndata);
+            return (returndata.toString());
         }
 
     }
-    
+
     /**
      * Update secondary email address after verifying OxGuard password
      * @param request
@@ -249,18 +285,25 @@ public class Auth {
         String newemail = Core.getStringFromJson(json, "email", true);
         Access acc = new Access();
         GuardKeys keys = acc.getKeys(userid, cid);
-        if (keys.getDecodedPrivate(password) != null) {
+        if (keys.verifyPassword(password)) {
         	if (updateEmail (userid, cid, newemail)) {
+        	    LogAction.setSuccess();
         		Core.sendOK(response, "OK");
-        	} else Core.sendFail(response, "Fail");
+        	} else {
+        	    LogAction.setFail();
+        	    logger.info("Problem updating secondary email");
+        	    Core.sendFail(response, "Fail");
+        	}
         } else {
         	CheckBad.addBad(Integer.toString(userid) + "-" + Integer.toString(cid));
+        	LogAction.setFail();
+        	logger.info("Bad password for changing secondary email");
         	Core.sendFail(response, "Bad password");
         	return;
         }
-        
+
     }
-    
+
     public void getSecondaryEmail (HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws Exception {
     	Api ap = new Api(cookie, request);
         if (ap.verifyLogin() == false) {// verify logged into UI
@@ -269,6 +312,7 @@ public class Auth {
         }
         int cid = Core.getIntParameter(request, "cid", true);
         int userid = Core.getIntParameter(request, "userid", true);
+        LogAction.setUser(userid, cid);
        	String command = "SELECT answer, email FROM og_KeyTable WHERE id = ? AND CID = ?";
     	DbCommand dcom = new DbCommand(command);
     	dcom.addVariables(userid);
@@ -286,9 +330,10 @@ public class Auth {
     		}
     	}
     	db.close();
+    	LogAction.setSuccess();
     	Core.sendOK(response, "");
     }
-    
+
     private boolean updateEmail (int userid, int cid, String email) {
     	Access acc = new Access();
     	return(acc.storeQuestion(userid, cid, "e", email));
@@ -297,9 +342,16 @@ public class Auth {
     public void resetPass(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws Exception {
     	int cid = Core.getIntParameter(request, "cid", true);
         int userid = Core.getIntParameter(request, "userid", true);
-        resetPass (userid, cid, request, response, cookie, true);
+        LogAction.setUser(userid, cid);
+        Api ap = new Api(cookie, request);
+        if (ap.verifyLogin() != false) {
+            resetPass (userid, cid, request, response, cookie, true);
+        } else {
+            response.setStatus(550);
+            return;
+        }
     }
-    
+
     /*
      * Reset password.  Used from command line tools only
      */
@@ -320,19 +372,27 @@ public class Auth {
 					}
 	    			Sender sender = new Sender();
 	                com.openexchange.guard.mailcreator.Mail mf = new com.openexchange.guard.mailcreator.Mail();
-	                if (!sender.send(mf.getResetEmail(email, Mail.getFromAddress(email, email, keys.userid, keys.contextid), newpass, Config.default_lang, 0), null)) {
+	                int templid = 0;
+	                try {
+	                    templid = OxConfiguration.getTemplateId(keys.contextid, keys.userid);
+	                } catch (Exception e) {
+	                    logger.error("problem getting template id for reset password email");
+	                }
+	                if (!sender.send(mf.getResetEmail(email, Mail.getFromAddress(email, email, keys.userid, keys.contextid), newpass, Config.default_lang, templid), null)) {
 	                	System.out.println("Failed to send");
 	                } else {
 	                	System.out.println("Reset sent");
 	                }
-				} else // OX member
-					resetPass (keys.userid, keys.contextid, null, null, null, false);
+				} else {
+                    resetPass (keys.userid, keys.contextid, null, null, null, false);
+                }
 			} catch (Exception e) {
+			    LogAction.setFail();
 				logger.error("Error resetting password for " + email, e);
 			}
-    	
+
     }
-    
+
     // Remove PIN, for command line use only
     public static void removePin (String email) {
     	Access acc = new Access();
@@ -353,19 +413,49 @@ public class Auth {
 			System.out.println(ex);
 		}
     }
-    
-    
+
+    /**
+     * Delete password recovery for the user
+     * @param request
+     * @param response
+     * @param cookie
+     * @throws Exception
+     */
+    public void deleteRecovery (HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws Exception {
+
+        JsonObject json = Core.getJSON(request);
+        Api ap = new Api(cookie, request);
+        // verify login
+        if (!ap.verifyLogin()) {
+            Core.sendFail(response, "Must be logged in");
+            return;
+        }
+        String password = Core.getStringFromJson(json, "password", true);
+        int userid = Core.getIntFromJson(json, "userid", true);
+        int cid = Core.getIntFromJson(json, "cid", true);
+
+        Access acc = new Access();
+        GuardKeys key = acc.getKeys(userid, cid);
+
+        // verify password
+        if (!key.verifyPassword(password)) {
+            Core.sendFail(response, "Bad password");
+            return;
+        }
+
+        acc.deleteRecovery(userid, cid);
+        Core.sendOK(response, "OK");
+
+
+    }
+
     public void resetPass(int userid, int cid, HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie, boolean web) throws Exception {
-    	int templId = 0;
-    	if (web) { // If from web request, make sure logged onto OX
-        	Api ap = new Api(cookie, request);
-            if (ap.verifyLogin() == false) {
-                response.setStatus(550);
-                return;
-            }
-            templId = OxConfiguration.getTemplateId(cid, userid);
-    	}
-        
+        int templid = 0;
+        try {
+            templid = OxConfiguration.getTemplateId(cid, userid);
+        } catch (Exception e) {
+            logger.error("problem getting template id for reset password email");
+        }
         String lang = request == null ? Config.default_lang : request.getParameter("language");
     	String command = "SELECT answer, email FROM og_KeyTable WHERE id = ? AND CID = ?";
     	DbCommand dcom = new DbCommand(command);
@@ -376,12 +466,12 @@ public class Auth {
     	if (db.next()) {
     		String encr_answer = db.rs.getString("answer");
     		String userEmail = db.rs.getString("email");
-    		String defaultEmail = request.getParameter("default");
     		EncrLib encr = new EncrLib();
     		boolean secondary = true;
-    		String secondEmail = encr.decryptAES(encr_answer, Config.rpass, "e", Config.AESKeyLen, false);
+    		// Mailing to secondary email address if web.  If from command line, then email to primary
+    		String secondEmail = web ? encr.decryptAES(encr_answer, Config.rpass, "e", Config.AESKeyLen, false) : userEmail;
     		if ((secondEmail == null) || secondEmail.equals("")) {
-    			secondEmail = defaultEmail;
+    			secondEmail = userEmail;
     			secondary = false;
     		}
     		if (EmailValidator.validate(secondEmail)) {
@@ -389,40 +479,56 @@ public class Auth {
     			String newpass = keyman.resetPassword(userEmail);
     			if (newpass == null) {
     				if (web) {
+    				    LogAction.setFail();
                     	Core.sendOK(response, "FailChange");
-                    } else System.out.println("Failed to change");
+                    } else {
+                        System.out.println("Failed to change");
+                    }
     				db.close();
     				return;
     			}
     			if (newpass.equals("NR")) {
     				if (web) {
+    				    LogAction.setFail();
+    				    logger.info("Unable to recover password due to no recovery");
                     	Core.sendOK(response, "NoRecovery");
-                    } else System.out.println("No recovery available");
+                    } else {
+                        System.out.println("No recovery available");
+                    }
     				db.close();
     				return;
     			}
     			Sender sender = new Sender();
                 com.openexchange.guard.mailcreator.Mail mf = new com.openexchange.guard.mailcreator.Mail();
-                String from = (defaultEmail == null ? secondEmail : defaultEmail);
-                if (!sender.send(mf.getResetEmail(secondEmail, Mail.getFromAddress(from, from, userid, cid) , newpass, lang, templId), null)) {
+                String from = userEmail;
+                if (!sender.send(mf.getResetEmail(secondEmail, Mail.getFromAddress(from, from, userid, cid) , newpass, lang, templid), null)) {
                     if (web) {
+                        LogAction.setFail();
+                        logger.error("Failed to send email reset");
                     	Core.sendOK(response, "FailNotify");
-                    } else System.out.println("Failed to send nofication");
+                    } else {
+                        System.out.println("Failed to send nofication");
+                    }
                     db.close();
                     return;
                 } else {
                     if (web) {
+                        LogAction.setSuccess();
                     	if (secondary) {
                     		Core.sendOK(response, "ok");
                     	} else {
                     		Core.sendOK(response, "primary");
                     	}
-                    } else System.out.println("OK");
+                    } else {
+                        System.out.println("OK");
+                    }
                 }
     		} else { // Bad secondary email
     			if (web) {
                 	Core.sendOK(response, "NoSecondary");
-                } else System.out.println("No Secondary Email");
+                } else {
+                    System.out.println("No Secondary Email");
+                }
                 db.close();
                 return;
     		}
@@ -431,7 +537,7 @@ public class Auth {
     }
     /**
      * Reset password from within OX UI using OX login password --- Depreciated
-     * 
+     *
      * @param request
      * @param response
      * @param cookie
@@ -468,7 +574,8 @@ public class Auth {
         response.setContentType("application/json");
         response.addHeader("Access-Control-Allow-Origin", "*");
         if (resetpassword == null) {
-            logger.debug("Bad password for change");
+            LogAction.setFail();
+            logger.info("Bad password for change");
             response.getWriter().write("{\"auth\":\"Bad password\"}");
             return (null);
         } else {
@@ -479,6 +586,7 @@ public class Auth {
             String returndata = " {\"auth\":\"" + encr.encryptAES(json.toString(), token) + "\"} ";
             logger.debug(returndata);
             response.getWriter().write(returndata);
+            LogAction.setSuccess();
             logger.info("OK Password Change");
             return (returndata);
         }
@@ -487,10 +595,10 @@ public class Auth {
 
     /**
      * Authorize login from guest account
-     * 
+     *
      * @param request
      * @param response
-     * @throws GuardMissingParameter 
+     * @throws GuardMissingParameter
      */
     public void guestlogin(HttpServletRequest request, HttpServletResponse response) throws GuardMissingParameter {
     	JsonObject login_json = Core.getJSON(request);
@@ -503,82 +611,98 @@ public class Auth {
         String cookieID = EncrLib.getUUID();
         Cookie cookie = new Cookie("OxReaderID", cookieID);
         cookie.setHttpOnly(true);
+        cookie.setPath("/");
         response.addCookie(cookie);
         String username = Core.getStringFromJson(login_json, "username", true);
         String pin = login_json.get("pin") == null ? null : login_json.get("pin").getAsString();
         try {
             if (CheckBad.isBad(username, Config.bad_password_count)) {
             	response.getWriter().write("{\"auth\":\"lockout\", \"time:\": " + Config.bad_minute_lock + "}");
-            	logger.debug("Guest auth lockout for " + username);
+            	LogAction.setFail();
+            	logger.info("Guest auth lockout for " + username);
                 return;
             }
             String password = Core.getStringFromJson(login_json, "password", true);
             GuardKeys keys = acc.getKeysFromEmail(username);
             if (keys == null) {
                 response.getWriter().write("{\"auth\":\"No Key\"}");
-                logger.debug("Guest auth No Key for " + username);
+                LogAction.setFail();
+                logger.info("Guest auth No Key for " + username);
                 return;
             }
-     //       if (keys.passwordNeeded == true) {
-     //           response.getWriter().write(
-      //              "{\"auth\":\"Password Needed\", \"userid\":\"" + keys.userid + "\",\"cid\":\"" + keys.contextid + "\",\"sessionID\" : \"" + sessionID + "\"}");
-      //          return;
-      //      }
+
 
             String token = acc.newToken(cookieID, keys.userid, keys.contextid);
             if (token == null) {
                 response.getWriter().write("{\"auth\":\"Unable to get token\"}");
-                logger.debug("Guest auth Unable to get database token for " + username);
+                LogAction.setFail();
+                logger.error("Guest auth Unable to get database token for " + username);
                 return;
             }
-
-            String json = "{\"user_id\" : \"" + keys.userid + "\", \"encr_password\" : \"" + password + "\", \"sessionID\" : \"" + sessionID + "\", \"cid\" : \"" + keys.contextid + "\" , \"lastMod\" : \"" + keys.lastup + "\"}";
+            // This is the json we're going to encrypt and include in the auth
+            JsonObject json = new JsonObject();
+            json.addProperty("user_id", keys.userid);
+            json.addProperty("encr_password", password);
+            json.addProperty("sessionID", sessionID);
+            json.addProperty("cid", keys.contextid);
             EncrLib encr = new EncrLib();
             String returndata = "Bad Password";
-            PrivateKey key = null;
-            
-            try {
-            	key = keys.getDecodedPrivate(password);
-            } catch (BadPasswordException ex) {
-            	logger.debug("Bad password for " + username);
-            }
-            if (key != null) {
-                // returndata = encr.encryptAES(json, token, cookieID, Config.AESKeyLen);
-                
-                logger.debug("Guest auth good for " + username);
+
+            if (!keys.verifyPassword(password)) {
+                CheckBad.addBad(username);
+                LogAction.setFail();
+                logger.info("Bad password for " + username);
+            } else
+            {
+                LogAction.setSuccess();
+                logger.info("Guest auth good for " + username);
                 // Check if PIN
-                
+
                 if ((keys.lastup == null) && (keys.question == null ? false : keys.question.equals("PIN"))) {
                 	if (pin != null) {
                 		if (pin.equals(keys.answer)) {
-                			returndata = encr.encryptAES(json, token);
+                			returndata = encr.encryptAES(json.toString(), token);
                 		} else {
                 			returndata = "Bad";
                 			CheckBad.addBad(username);
                 		}
                 	} else {
-                		logger.debug("New Guest login, pin required");
+                		logger.info("New Guest login, pin required");
                 		returndata = "PIN";
                 	}
                 } else {
-                	returndata = encr.encryptAES(json, token);
+                	returndata = encr.encryptAES(json.toString(), token);
                 }
-            } else {
-            	CheckBad.addBad(username);
-            	logger.debug("Guest auth bad for " + username);
             }
-            
-            response.getWriter().write(
-                " {\"auth\":\"" + returndata + "\", \"userid\":\"" + keys.userid + "\",\"cid\":\"" + keys.contextid + "\",\"sessionID\" : \"" + sessionID + "\", \"lastMod\" : \"" + keys.lastup + "\"}");
+
+            // The json returned in plain text.  no password here (encrypted within auth)  Include settings
+            JsonObject sendjson = new JsonObject();
+            sendjson.addProperty("auth", returndata);
+            sendjson.addProperty("userid", keys.userid);
+            sendjson.addProperty("sessionID", sessionID);
+            sendjson.addProperty("cid", keys.contextid);
+            sendjson.addProperty("lastMod", keys.lastup == null ? "null" : keys.lastup.toString());
+            try {
+            	Settings settings = new Settings();
+            	sendjson.add("settings", getSettings(keys.settings.toString(), settings));
+            } catch (Exception ex) {
+            	logger.error("problem getting guest settings " , ex);
+            }
+            response.getWriter().write(sendjson.toString());
 
         } catch (Exception e) {
+            LogAction.setFail();
             logger.error("Error during guest login  for user " + username, e);
         }
     }
 
     public void logout(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws Exception {
+        int userid = Core.getIntParameter(request, "userid");
+        int cid = Core.getIntParameter(request, "cid");
+        LogAction.setUser(userid, cid);
         com.openexchange.guard.database.Access acc = new com.openexchange.guard.database.Access();
         acc.deleteToken(cookie.JSESSIONID);
+        acc.deleteToken(cookie.OxReaderID);
         com.openexchange.guard.util.Core.sendOK(response, "OK");
     }
 
@@ -589,6 +713,7 @@ public class Auth {
             Settings settings = getPermissions(cookie, request);
             login(json, request, response, cookie, settings);
         } catch (Exception ex) {
+            LogAction.setFail();
             logger.error("Error reading login JSON information", ex);
             response.getWriter().write("{\"auth\":\"BAD FORMAT\"} ");
             return;
@@ -612,19 +737,21 @@ public class Auth {
         if (pubkey != null) {
         	returnJson.addProperty("pubKey", pubkey);
         }
-        
         int cid = 0;
         try {
 
             int userid = Core.getIntFromJson(json, "user_id", true);
-            cid = Core.getIntFromJson(json, "cid", true);  
+            cid = Core.getIntFromJson(json, "cid", true);
+            MDC.put("ID", Integer.toString(userid));
+            MDC.put("CID", Integer.toString(cid));
             String email = json.get("email") == null ? null : json.get("email").getAsString();
             // Check lockout of bad attempts.  If excessive, don't give encrypted auth key
             if (CheckBad.isBad(Integer.toString(userid) + "-" + Integer.toString(cid), Config.bad_password_count)) {
             	returnJson.addProperty ("auth", "Lockout");
             	returnJson.add ("settings", getSettings(null, settings));
                 response.getWriter().write(returnJson.toString());
-                logger.debug("Lockout auth due to bad attempts");
+                LogAction.setFail();
+                logger.info("Lockout auth due to bad attempts");
                 return;
             }
             if (cid == 0) { // Occasionally, the UI doesn't load the CID.  Get from backend
@@ -636,8 +763,19 @@ public class Auth {
             	logger.debug("Retrieved context_id of " + cid);
             }
             returnJson.addProperty("cid", cid);
-            GuardKeys keys = acc.getKeys(userid, cid);
-            
+            GuardKeys keys = null;
+            String keyid = Core.getStringParameter(request, "keyid");
+            if (keyid != null) {
+                try {
+                    Long kid = Long.parseLong(keyid);
+                    keys = acc.getKeys(userid, cid, kid, false);
+                } catch (Exception ex) {
+                    logger.error("Problem with login for Key ID " + keyid, ex);
+                }
+            } else {
+                keys = acc.getKeys(userid, cid);
+            }
+
             // Nothing found.  Previously a non-ox user and new to us?
             if (keys == null) {
             	if (email != null) {
@@ -648,60 +786,70 @@ public class Auth {
 	            				Upgrader.upgrade(r.userid, r.cid, userid, cid); // try to upgrade the account
 	            				keys = acc.getKeys(userid, cid); // try again
 	            			} catch (Exception ex) {
-	            				logger.error("Unable to upgrade email " + email);
+	            				logger.error(LogMarkers.NOTIFY_MINOR, "Unable to upgrade email " + email);
 	            			}
-	            			
+
 	            		}
 	            	}
             	}
             }
             if (keys == null) {
+                LogAction.setSuccess();
             	returnJson.addProperty ("auth", "No Key");
             	returnJson.add ("settings", getSettings(null, settings));
                 response.getWriter().write(returnJson.toString());
                 logger.debug("Auth sent with No Key for cid = " + cid + " userid = " + userid);
                 return;
             }
-        	returnJson.add ("settings", getSettings(keys.settings, settings));
+        	returnJson.add ("settings", getSettings(keys.settings.toString(), settings));
+        	returnJson.add("recoveryAvail", new JsonPrimitive(keys.recoveryAvail));
             if (keys.passwordNeeded == true) {
             	returnJson.addProperty ("auth", "Password Needed");
                 response.getWriter().write(returnJson.toString());
+                LogAction.setSuccess();
                 logger.debug("Auth sent with Password Needed for cid = " + cid + " userid = " + userid);
                 return;
             }
 
             String token = acc.getToken(cookie.JSESSIONID);
-            if (token == null) token = acc.newToken(cookie.JSESSIONID, userid, cid);
+            if (token == null) {
+                token = acc.newToken(cookie.JSESSIONID, userid, cid);
+            }
             if (token == null) {
             	returnJson.addProperty ("auth", "Unable to get token");
                 response.getWriter().write(returnJson.toString());
-                logger.debug("Auth unable to get token from database for cid = " + cid + " userid = " + userid);
+                LogAction.setFail();
+                logger.error("Auth unable to get token from database for cid = " + cid + " userid = " + userid);
                 return;
             }
             EncrLib encr = new EncrLib();
             try {
                 String oxpassword = json.get("ox_password") == null ? "" : json.get("ox_password").getAsString();
-                if (oxpassword.equals("null")) oxpassword = "";
+                if (oxpassword.equals("null")) {
+                    oxpassword = "";
+                }
                 String encrpass = Core.getStringFromJson(json, "encr_password");
-                if (encrpass == null) encrpass = "";
+                if (encrpass == null) {
+                    encrpass = "";
+                }
                 String returndata = "Bad Password";
-                if (keys.getEncodedPrivate() == null) {
-                    returndata = "NoPrivate";
-                } else {
+
                     if (encrpass.length() > 1) { // If provided with encryption password, check
-                        if (keys.getDecodedPrivate(encrpass) != null) {
+                        if (keys.verifyPassword(encrpass)) {
                             returndata = encr.encryptAES(json.toString(), token);
                             CheckBad.removeBad(Integer.toString(userid) + "-" + Integer.toString(cid)); // remove bad session listing count
                             logger.debug("Auth with proper password for cid = " + cid + " userid = " + userid);
-                            // returndata = encr.encryptAES(json.toString(), token, cookie.JSESSIONID, Config.AESKeyLen);
+                            LogAction.setSuccess();
                         } else {
                         	CheckBad.addBad(Integer.toString(userid) + "-" + Integer.toString(cid));  // Increment fail password counter
+                        	LogAction.setSuccess();
                         	logger.debug("Auth, OG user, settings only for cid = " + cid + " userid = " + userid);
                         }
                     } else { // If no encr password, try the oxpassword
                         if (oxpassword.length() > 1) {
                         	try {
-	                            if (keys.getDecodedPrivate(oxpassword, false) != null) {
+                                LogAction.setSuccess();
+	                            if (keys.verifyPassword(oxpassword)) {
 	                                json.remove("encr_password");
 	                                json.addProperty("encr_password", oxpassword); // The encryption password is our ox password
 	                                returndata = encr.encryptAES(json.toString(), token);
@@ -717,7 +865,7 @@ public class Auth {
                         	}
                         }
                     }
-                }
+
                 if (json.has("extrapass")) { // File does extra login to encrypt passwords prior to creating links
                 	String extrapass = json.get("extrapass").getAsString();
                 	if (!extrapass.equals("")) {
@@ -727,8 +875,10 @@ public class Auth {
                 }
                 returnJson.addProperty("auth", returndata);
                 response.getWriter().write(returnJson.toString());
+                LogAction.setSuccess();
                 logger.info("Authorization at " + start);
             } catch (Exception e) {
+                LogAction.setFail();
                 logger.error("Error during login", e);
                 returnJson.addProperty("auth", "BAD FORMAT");
                 response.getWriter().write(returnJson.toString());
@@ -743,9 +893,11 @@ public class Auth {
                 }
                 returnJson.addProperty("auth", "No Key");
                 returnJson.add ("settings", getSettings(null, settings));
+                LogAction.setSuccess();
                 response.getWriter().write(returnJson.toString());
             } else {
                 response.getWriter().write("{\"auth\":\"BAD FORMAT\", \"lang\": " + Languages.getJsonAvail(Languages.default_code) + "} ");
+                LogAction.setSuccess(false);
                 logger.error("Error during login", e);
             }
         }
@@ -754,40 +906,41 @@ public class Auth {
 
     /**
      * Create a json of settings, include the individuals settings, plus global
-     * 
+     *
      * @param settings
      * @return
      */
     private JsonObject getSettings(String settings, Settings setting) {
         StringBuilder allSettings = new StringBuilder();
-        if (settings == null)
+        if (settings == null) {
             settings = "";
-        
+        }
+
         // If settings string loaded with key, convert to JSON
-        if (!settings.startsWith("{"))
+        if (!settings.startsWith("{")) {
             allSettings.append("{");
+        }
         if (settings != null) {
             allSettings.append(settings);
         }
-        if (!settings.endsWith("}"))
+        if (!settings.endsWith("}")) {
             allSettings.append("}");
-        
+        }
+
         // Convert to Json, then check if global defaults present and add. Individual settings override
         Gson gson = new GsonBuilder().create();
         JsonObject json = gson.fromJson(allSettings.toString(), JsonObject.class);
         if (!json.has("Private")) { // If not defined at user level database, use global settings
         	if (setting.privatepass == false) { // If config cascade value set as false, then use
         		json.add("ChangePrivate", new JsonPrimitive("false"));
-        	} else json.add("ChangePrivate", new JsonPrimitive(Config.ok_private ? "true" : "false"));
-        }
-        if (!json.has("DeleteRecovery")) { // If not defined at user level database, use global settings
-        	if (setting.deleteRecovery == false) { // If config cascade value set as false, then use
-        		json.add("DeleteRecovery", new JsonPrimitive("false"));
-        	} else json.add("DeleteRecovery", new JsonPrimitive(Config.delete_recovery? "true" : "false"));
+        	} else {
+                json.add("ChangePrivate", new JsonPrimitive(Config.ok_private ? "true" : "false"));
+            }
         }
+
         if (!json.has("extrapass")) { // If not defined at user level database, use global settings
         	json.add("extrapass", new JsonPrimitive(setting.extrapass ? "true" : "false"));
-            
+
         }
 
         if (!json.has("deleteOnRevoke")) {
@@ -797,9 +950,18 @@ public class Auth {
         if (!json.has("oxguard")) {
             json.add("oxguard", new JsonPrimitive(check(setting, Settings.permissions.MAIL)));
         }
+
+        // If either the global setting is set for no delete keys, or the config cascade, then use
+        json.add("noDeletePrivate", new JsonPrimitive(setting.noDeletePrivate));
+        json.add("noDeleteRecovery", new JsonPrimitive(setting.noDeleteRecovery));
+
+        json.add("noRecovery", new JsonPrimitive(setting.norecovery));
+
         if (Config.showStatus == false) {
         	json.addProperty("status", false);
         }
+        json.addProperty("min_password_length", Config.min_password_length);
+        json.addProperty("password_length", Config.newPassLength);
 
         return (json);
     }
@@ -809,7 +971,7 @@ public class Auth {
     }
 
     public static Settings getPermissions(com.openexchange.guard.server.OxCookie cookie, HttpServletRequest request) {
-    	Settings settings = new Settings(); 
+    	Settings settings = new Settings();
         settings.perm = Settings.permissions.NONE;
         Api ap = new Api(cookie, request);
         String dat = ap.getCapabilities().toLowerCase();
@@ -817,18 +979,45 @@ public class Auth {
         	logger.error("Unable to get permissions from OX backend");
         	return (settings);
         }
-        if (dat.contains("guard:mail") || dat.contains("guard-mail"))
+        if (dat.contains("guard")) {
+            settings.perm = Settings.permissions.GUARD;
+        }
+        if (dat.contains("guard:mail") || dat.contains("guard-mail")) {
             settings.perm = Settings.permissions.MAIL;
+        }
         if (dat.contains("guard:drive") || dat.contains("guard-drive")) {
             if (settings.perm == Settings.permissions.MAIL) {
                 settings.perm = Settings.permissions.BOTH;
-            } else
+            } else {
                 settings.perm = Settings.permissions.FILE;
+            }
+        }
+        if (dat.contains("guard-noextra")) {
+            settings.extrapass = false;
+        }
+        if (dat.contains("guard-noprivate")) {
+            settings.privatepass = false;
+        }
+        if (dat.contains("guard-nodeleterecovery")) {
+            settings.noDeleteRecovery = true;
+        } else {
+            settings.noDeleteRecovery = Config.no_delete_recovery;
+        }
+
+        if (dat.contains("guard-nodeleteonrevoke")) {
+            settings.deleteKeyOnRevoke = false;
+        }
+        if (dat.contains("guard-norecovery")) {
+            settings.norecovery = true;
+        } else {
+            settings.norecovery= Config.no_recovery;
+        }
+
+        if (dat.contains("guard-nodeleteprivate")) {
+            settings.noDeletePrivate = true;
+        } else {
+            settings.noDeletePrivate = Config.no_delete_keys;
         }
-        if (dat.contains("guard-noextra")) settings.extrapass = false;
-        if (dat.contains("guard-noprivate")) settings.privatepass = false;
-        if (dat.contains("guard-nodeleterecovery")) settings.deleteRecovery = false;
-        if (dat.contains("guard-nodeleteonrevoke")) settings.deleteKeyOnRevoke = false;
         return (settings);
     }
 
@@ -837,11 +1026,20 @@ public class Auth {
         return (check(settings, req));
     }
 
-    private static boolean check(Settings current, Settings.permissions req) {
-        if (req == current.perm)
+    public static boolean checkMinimum (Settings current) {
+        if ((current.perm == Settings.permissions.GUARD) || (current.perm == Settings.permissions.FILE)|| (current.perm == Settings.permissions.BOTH)) {
+            return(true);
+        }
+        return(false);
+    }
+
+    public static boolean check(Settings current, Settings.permissions req) {
+        if (req == current.perm) {
             return (true);
-        if (current.perm == Settings.permissions.BOTH)
+        }
+        if (current.perm == Settings.permissions.BOTH) {
             return (true);
+        }
         return (false);
     }
 }
