@@ -49,21 +49,36 @@
 
 package com.openexchange.guard.server;
 
+import java.io.ByteArrayInputStream;
 import java.io.IOException;
+import java.io.InputStream;
 import java.io.OutputStream;
 import java.security.NoSuchAlgorithmException;
 import java.security.spec.InvalidKeySpecException;
+import java.sql.Date;
 import java.sql.SQLException;
-
+import java.text.SimpleDateFormat;
+import java.util.Enumeration;
+import java.util.Properties;
+import java.util.TimeZone;
+import javax.mail.Address;
+import javax.mail.Header;
+import javax.mail.Message.RecipientType;
+import javax.mail.MessagingException;
+import javax.mail.Multipart;
+import javax.mail.Part;
+import javax.mail.Session;
+import javax.mail.internet.InternetAddress;
+import javax.mail.internet.MimeMessage;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
-
 import org.bouncycastle.openpgp.PGPException;
 import org.bouncycastle.util.encoders.Base64;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
+import com.google.gson.JsonArray;
 import com.google.gson.JsonObject;
+import com.google.gson.JsonPrimitive;
 import com.openexchange.guard.config.Config;
 import com.openexchange.guard.database.DbCommand;
 import com.openexchange.guard.database.DbQuery;
@@ -73,8 +88,12 @@ import com.openexchange.guard.database.Keymanager;
 import com.openexchange.guard.encr.Crypto;
 import com.openexchange.guard.encr.EncrLib;
 import com.openexchange.guard.encr.EncryptedObj;
+import com.openexchange.guard.exceptions.BadPasswordException;
 import com.openexchange.guard.exceptions.GuardMissingParameter;
 import com.openexchange.guard.mailcreator.Mail;
+import com.openexchange.guard.pgp.PGPResult;
+import com.openexchange.guard.pgp.PGPUtils;
+import com.openexchange.guard.pgp.PgpHandler;
 import com.openexchange.guard.translate.GetText;
 import com.openexchange.guard.util.Core;
 import com.openexchange.guard.validator.Normalizer;
@@ -85,6 +104,12 @@ public class Guest {
 	private static Logger logger = LoggerFactory.getLogger(Guest.class);
 
 
+	/**
+	 * Changing Guest password
+	 * @param request
+	 * @param response
+	 * @param cookie
+	 */
     public void ChangePass(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) {
         Auth auth = new Auth();
         cookie.JSESSIONID = cookie.OxReaderID; // Clear the ox cookie for just reader.
@@ -99,68 +124,166 @@ public class Guest {
         }
     }
 
+    /**
+     * For initial decoding of PGP Guest email
+     * @param request
+     * @param response
+     * @param cookie
+     * @throws GuardMissingParameter
+     */
     public void GetEmail(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws GuardMissingParameter {
-
+    	decodeGuestPgp (request, response, cookie, "mail");
+    }
+    
+    /**
+     * Decodes Guest PGP email.  Opens attachment if action open, returns email json if action mail
+     * @param request
+     * @param response
+     * @param cookie
+     * @param action
+     * @throws GuardMissingParameter
+     */
+    private void decodeGuestPgp(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie, String action) throws GuardMissingParameter {
     	JsonObject jdata = Core.getJSON(request);
-        String auth = Core.getStringParameter(request, "auth", true);
-        String itemid = Core.getStringFromJson(jdata, "item", true);
-        String extraPass = Core.getStringFromJson(jdata, "extrapass");
-        
-        int cid = Core.getIntFromJson(jdata, "cid", true);
-        cookie.JSESSIONID = cookie.OxReaderID; // In case person uses both reader and OX main, we need to clear out the previous cookie
-        UserData data = new UserData(auth, cookie);
-                
-        if (data.encr_password == null) {
-            response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
-            return;
-        }
-        try {
-            EncryptedObj emaildata = com.openexchange.guard.storage.Storage.readEncrObj(data.userid, data.cid, itemid);
-            if (emaildata.ItemID.equals("not found")) {
-                com.openexchange.guard.util.Core.sendFail(response, "not found");
+    	String itemid = "";
+    	if (jdata != null) {
+    		itemid = Core.getStringFromJson(jdata, "item", true);
+    	} else {
+    		itemid = Core.getStringParameter(request, "email", true);
+    	}
+    	if (itemid.startsWith("pgp")) {
+    		String auth = Core.getStringParameter(request, "auth", true);
+            cookie.JSESSIONID = cookie.OxReaderID; // In case person uses both reader and OX main, we need to clear out the previous cookie
+            UserData data = new UserData(auth, cookie);
+            if (data.encr_password == null) {
+                response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
                 return;
             }
-            Crypto crypto = new Crypto();
-            String email = crypto.decodeEncrObj(data.userid, cid, emaildata, data.encr_password);
-
-            if (extraPass != null) { // If we provided extrapass, we need to encrypt to return for attachments
-                if (!extraPass.equals("")) {
-                    if (extraPass.length() > 2) {
-                        String extrap = "";
-                        String epassword = "";
-                        EncrLib encr = new EncrLib();
-                        Access acc = new Access();
-                        String token = acc.getToken(cookie.JSESSIONID);
-                        email = crypto.decodeExtraPass(email, extraPass, emaildata.ItemID, emaildata.keyLength);
-                        if (email == "") {
-                            email = "badextra";
-                            com.openexchange.guard.util.Core.sendFail(response, email);
-                            return;
-                        }
-                        extrap = encr.encryptAES(extraPass, token);
-                        email = "{" + "\"extrapass\":\"" + extrap + "\", \"epass\":\"" + epassword + "\"," + email.substring(1); // add
-                                                                                                                                 // extrapass
-                                                                                                                                 // info
-                                                                                                                                 // to
-                                                                                                                                 // email
-                                                                                                                                 // json
-                        
-                    }
+            try {
+                InputStream in = com.openexchange.guard.storage.Storage.readObj(data.userid, data.cid, itemid);
+                if (in != null) {
+                	 PGPResult result = null;
+                     try {
+                    	 result = PGPUtils.decryptFile(in, data.userid, data.cid, data.encr_password);
+                    	 
+                     } catch (BadPasswordException ex) {
+                    	 in.close();
+                     	Core.sendFail(response, "Bad password");
+                     	return;
+                     }
+                     in.close();
+                     if (result.error != null) {
+                     	Core.sendFail(response, result.error);
+                     } else {
+                    	 switch (action) {
+                     		case "mail": 
+                     			Core.sendOK(response, createReturnJson(result, data.encr_password, cookie));
+                     		break;
+                     		case "open":
+                     			String filename = Core.getStringParameter(request, "attname", true);
+                     			boolean download = (Core.getStringParameter(request, "download") == null ? false : (Core.getStringParameter(request, "download").toLowerCase().equals("true"))); 
+                     			PgpHandler.getAttach (response, result, filename, download);
+                     		break;
+                     	}
+                     }
+                } else {
+                	Core.sendFail(response, "not found");
+                	logger.info("Guest item not found: " + itemid);
                 }
-
+            } catch (Exception ex) {
+            	logger.error("Problemd decoding Guest PGP ", ex);
+            	
             }
-            email = Normalizer.normalize(email);
-            logger.debug("Guest email decoded OK");
-            response.addHeader("Access-Control-Allow-Origin", "*");
-            response.setContentType("application/json; charset=UTF-8");
-            response.setStatus(HttpServletResponse.SC_OK);
-            response.getWriter().write(email);
-        } catch (Exception ex) {
-            logger.error("Error decoding guest email for itemID " + itemid, ex);
-            response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
-        }
-
+    	} else {  // Proprietary Guard
+    		GetOldEmail (jdata, request, response, cookie);
+    	}
     }
+    
+    /**
+     * Create resturn Json for display in Guest browser
+     * @param result
+     * @param json
+     * @param password
+     * @param cookie
+     * @return
+     * @throws MessagingException
+     * @throws IOException
+     */
+    public static String createReturnJson(PGPResult result, String password, com.openexchange.guard.server.OxCookie cookie) throws MessagingException, IOException {
+        Session s = Session.getDefaultInstance(new Properties());
+        JsonObject data = new JsonObject();
+        InputStream in = new ByteArrayInputStream(result.decoded);
+        MimeMessage message = new MimeMessage(s, in);
+        Multipart mp = (Multipart) message.getContent();
+        JsonArray attachments = new JsonArray();
+        PgpHandler.ParseMultipart (mp, attachments);
+        JsonObject Results = new JsonObject();
+        Results.add("signature", new JsonPrimitive(result.signature));
+        Results.add("verified", new JsonPrimitive(result.verified));
+        if (password != null) {
+        	Access acc = new Access();
+            String token = acc.getToken(cookie.JSESSIONID);
+        	EncrLib encr = new EncrLib();
+        	Results.addProperty("epassword", encr.encryptAES(password, token));
+        }
+        if (result.error != null)
+            Results.add("error", new JsonPrimitive(result.error));
+        data.add("attachments", attachments);
+        data.add("results", Results);
+        Address[] fromarray = message.getFrom();
+        JsonArray fromJson = new JsonArray();
+        if (fromarray != null) {
+            for (int i = 0; i < fromarray.length; i++) {
+                JsonArray recip = new JsonArray();
+                InternetAddress fromAddr = (InternetAddress) fromarray[i];
+                recip.add(new JsonPrimitive (fromAddr.getPersonal() == null ? "" : fromAddr.getPersonal()));
+                recip.add(new JsonPrimitive(fromAddr.getAddress() == null ? "" : fromAddr.getAddress()));
+                fromJson.add(recip);
+            }
+        }
+        data.add("from", fromJson);
+        Address[] toarray = message.getRecipients(RecipientType.TO);
+        JsonArray toJson = new JsonArray();
+        if (toarray != null) {
+            for (int i = 0; i < toarray.length; i++) {
+                JsonArray recip = new JsonArray();
+                InternetAddress to = (InternetAddress) toarray[i];
+                recip.add(new JsonPrimitive (to.getPersonal() == null ? "" : to.getPersonal()));
+                recip.add(new JsonPrimitive(to.getAddress() == null ? "" : to.getAddress()));
+                toJson.add(recip);
+            }
+        }
+        data.add("to", toJson);
+        Address[] ccarray = message.getRecipients(RecipientType.CC);
+        JsonArray ccJson = new JsonArray();
+        if (ccarray != null) {
+            for (int i = 0; i < ccarray.length; i++) {
+                JsonArray recip = new JsonArray();
+                InternetAddress cc = (InternetAddress) ccarray[i];
+                recip.add(new JsonPrimitive (cc.getPersonal() == null ? "" : cc.getPersonal()));
+                recip.add(new JsonPrimitive(cc.getAddress() == null ? "" : cc.getAddress()));
+                ccJson.add(recip);
+            }
+        }
+        data.add("cc", ccJson);
+        Enumeration<Header> headers = message.getAllHeaders();
+        JsonObject headersJson = new JsonObject();
+        while (headers.hasMoreElements()) {
+            Header h = headers.nextElement();
+            headersJson.addProperty(h.getName(), h.getValue());
+        }
+        data.add("headers", headersJson);
+        data.addProperty("subject", message.getSubject());
+        final String ISO_FORMAT = "yyyy-MM-dd'T'HH:mm:ss.SSS'+00:00'";
+        final SimpleDateFormat sdf = new SimpleDateFormat(ISO_FORMAT);
+        final TimeZone utc = TimeZone.getTimeZone("UTC");
+        sdf.setTimeZone(utc);
+        data.addProperty("sent", sdf.format(message.getSentDate()));
+        in.close();
+        return (data.toString());
+    }
+    
+
 
     /**
      * Function to return encrypted attachment (non email)
@@ -172,86 +295,25 @@ public class Guest {
      */
     public void getEncrAttach(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws Exception {
         String emailid = Core.getStringParameter(request, "email", true);
-        int cid = Core.getIntParameter(request, "cid", true);
-        String auth = Core.getStringParameter(request, "auth", true);
-        String filename = Core.getStringParameter(request, "attname", true);
-        String extrapass = Core.getStringParameter(request, "extrapass");
-        String encrextrapass = Core.getStringParameter(request, "encrextrapass");
-        String lang = Core.getStringParameter(request, "language");
-        cookie.JSESSIONID = cookie.OxReaderID; // In case person uses both reader and OX main, we need to clear out the previous cookie
-        UserData data;
-        data = new UserData(auth, cookie);
-        if (data.userid == -1) {
-            response.getWriter().write(GetText.gettext("Unable to decode attachment, bad authentication", lang));
-            response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
-            return;
+        if (emailid.startsWith("pgp")) {
+        	cookie.JSESSIONID = cookie.OxReaderID;
+        	decodeGuestPgp(request, response, cookie, "open");
+        	return;
+        } else {
+        	getEncrAttachOld(request, response, cookie);
         }
-        Crypto crypto = new Crypto();
-        try {
-            EncryptedObj attdata = com.openexchange.guard.storage.Storage.readEncrObj(data.userid, data.cid, emailid + "-" + filename);
-            String attach = crypto.decodeEncrObj(data.userid, cid, attdata, data.encr_password);
-            if (attach != null) {// Data was stored base64, so we need to decode when returned
-                if (extrapass == null) {// If plaintext extrapass is null, then look at encryptedpass
-                    if (encrextrapass != null) {
-                        if (!encrextrapass.equals("")) {
-                            EncrLib encr = new EncrLib();
-                            Access acc = new Access();
-                            String token = acc.getToken(cookie.JSESSIONID);
-                            extrapass = encr.decryptAES(encrextrapass, token);
-                            attach = crypto.decodeExtraPass(attach, extrapass, attdata.ItemID, attdata.keyLength);
-                        }
-                    }
-                } else {
-                    if (extrapass != "") {
-                        EncrLib encr = new EncrLib();
-                        attach = encr.decryptAES(attach, extrapass, attdata.ItemID, attdata.keyLength);
-                    }
-                }
-                if (attach == "exp") { // If our email has expired, then just send that
-                    com.openexchange.guard.util.Core.sendOK(response, "Attachment Expired");
-                    return;
-                }
-                // util.Core.sendOK(response, attdata.content_type, Base64.decode(attach));
-                if (attach.equals("")) {
-                    com.openexchange.guard.util.Core.sendFail(response, GetText.gettext("Unable to decode", lang));
-                } else {
-                    sendToSaveAttach(
-                        response,
-                        attdata.content_type,
-                        Base64.decode(attach),
-                        filename.replace(".grd2", "").replace(".grd", ""));
-                }
-                return;
-            }
-        } catch (InvalidKeySpecException | NoSuchAlgorithmException | SQLException | PGPException e) {
-        	logger.error("Error returning encrypted attachments for email " + emailid, e);
-        }
-        com.openexchange.guard.util.Core.sendFail(response, GetText.gettext("Unable to decode", lang));
-
     }
+    
+
 
     /**
-     * Send attachment contents to browser with flags to save to disk
-     * 
+     * Get password reset question
+     * @param request
      * @param response
-     * @param type
-     * @param answer
-     * @param filename
+     * @param cookie
+     * @throws IOException
+     * @throws GuardMissingParameter
      */
-    private void sendToSaveAttach(HttpServletResponse response, String type, byte[] answer, String filename) {
-        response.setContentType(type);
-        response.setHeader("Content-Disposition", "attachment; filename=\"" + filename + "\"");
-        response.addHeader("Access-Control-Allow-Origin", "*");
-        response.setStatus(HttpServletResponse.SC_OK);
-        try {
-            OutputStream out = response.getOutputStream();
-            out.write(answer);
-            out.close();
-        } catch (IOException e) {
-        	logger.error("Error writing output stream for file " + filename, e);
-        }
-    }
-
     public void getQuestion(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws IOException, GuardMissingParameter {
         String email = Core.getStringParameter(request, "email", true);
         String question = "nf";
@@ -271,6 +333,14 @@ public class Guest {
         com.openexchange.guard.util.Core.sendOK(response, question);
     }
 
+    /**
+     * Perform password reset after question answered
+     * @param request
+     * @param response
+     * @param cookie
+     * @throws IOException
+     * @throws GuardMissingParameter
+     */
     public void resetPassword(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws IOException, GuardMissingParameter {
         String email = Core.getStringParameter(request, "email", true);
         String answer = Core.getStringParameter(request, "answer", true);
@@ -340,4 +410,164 @@ public class Guest {
             Core.sendFail(response, "Failed to change");
         }
     }
+    
+    //////////////////////////////////////////////////////////////////////////////////////////////////
+    ///////////////  DEPRECIATED.  FOR OLD GUARD SUPPORT
+    
+    
+    /**
+     * Depreciated Guard proprietary format
+     * @param jdata
+     * @param request
+     * @param response
+     * @param cookie
+     * @throws GuardMissingParameter
+     */
+    public void GetOldEmail(JsonObject jdata, HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws GuardMissingParameter {
+
+    	
+        String auth = Core.getStringParameter(request, "auth", true);
+        String itemid = Core.getStringFromJson(jdata, "item", true);
+        String extraPass = Core.getStringFromJson(jdata, "extrapass");
+        
+        int cid = Core.getIntFromJson(jdata, "cid", true);
+        cookie.JSESSIONID = cookie.OxReaderID; // In case person uses both reader and OX main, we need to clear out the previous cookie
+        UserData data = new UserData(auth, cookie);
+                
+        if (data.encr_password == null) {
+            response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
+            return;
+        }
+        try {
+            EncryptedObj emaildata = com.openexchange.guard.storage.Storage.readEncrObjOld(data.userid, data.cid, itemid);
+            if (emaildata.ItemID.equals("not found")) {
+                com.openexchange.guard.util.Core.sendFail(response, "not found");
+                return;
+            }
+            Crypto crypto = new Crypto();
+            String email = crypto.decodeEncrObj(data.userid, cid, emaildata, data.encr_password);
+
+            if (extraPass != null) { // If we provided extrapass, we need to encrypt to return for attachments
+                if (!extraPass.equals("")) {
+                    if (extraPass.length() > 2) {
+                        String extrap = "";
+                        String epassword = "";
+                        EncrLib encr = new EncrLib();
+                        Access acc = new Access();
+                        String token = acc.getToken(cookie.JSESSIONID);
+                        email = crypto.decodeExtraPass(email, extraPass, emaildata.ItemID, emaildata.keyLength);
+                        if (email.equals("")) {
+                            email = "badextra";
+                            com.openexchange.guard.util.Core.sendFail(response, email);
+                            return;
+                        }
+                        extrap = encr.encryptAES(extraPass, token);
+                        email = "{" + "\"extrapass\":\"" + extrap + "\", \"epass\":\"" + epassword + "\"," + email.substring(1); // add
+                                                                                                                                 // extrapass
+                                                                                                                                 // info
+                                                                                                                                 // to
+                                                                                                                                 // email
+                                                                                                                                 // json
+                        
+                    }
+                }
+
+            }
+            email = Normalizer.normalize(email);
+            logger.debug("Guest email decoded OK");
+            response.addHeader("Access-Control-Allow-Origin", "*");
+            response.setContentType("application/json; charset=UTF-8");
+            response.setStatus(HttpServletResponse.SC_OK);
+            response.getWriter().write(email);
+        } catch (Exception ex) {
+            logger.error("Error decoding guest email for itemID " + itemid, ex);
+            response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
+        }
+
+    }
+    
+    
+    public void getEncrAttachOld(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws Exception {
+    	String emailid = Core.getStringParameter(request, "email", true);
+        String filename = Core.getStringParameter(request, "attname", true);
+        int cid = Core.getIntParameter(request, "cid", true);
+        String auth = Core.getStringParameter(request, "auth", true);
+        String extrapass = Core.getStringParameter(request, "extrapass");
+        String encrextrapass = Core.getStringParameter(request, "encrextrapass");
+        String lang = Core.getStringParameter(request, "language");
+        cookie.JSESSIONID = cookie.OxReaderID; // In case person uses both reader and OX main, we need to clear out the previous cookie
+        UserData data;
+        data = new UserData(auth, cookie);
+        if (data.userid == -1) {
+            response.getWriter().write(GetText.gettext("Unable to decode attachment, bad authentication", lang));
+            response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
+            return;
+        }
+        Crypto crypto = new Crypto();
+        try {
+            EncryptedObj attdata = com.openexchange.guard.storage.Storage.readEncrObjOld(data.userid, data.cid, emailid + "-" + filename);
+            String attach = crypto.decodeEncrObj(data.userid, cid, attdata, data.encr_password);
+            if (attach != null) {// Data was stored base64, so we need to decode when returned
+                if (extrapass == null) {// If plaintext extrapass is null, then look at encryptedpass
+                    if (encrextrapass != null) {
+                        if (!encrextrapass.equals("")) {
+                            EncrLib encr = new EncrLib();
+                            Access acc = new Access();
+                            String token = acc.getToken(cookie.JSESSIONID);
+                            extrapass = encr.decryptAES(encrextrapass, token);
+                            attach = crypto.decodeExtraPass(attach, extrapass, attdata.ItemID, attdata.keyLength);
+                        }
+                    }
+                } else {
+                    if (!extrapass.equals("")) {
+                        EncrLib encr = new EncrLib();
+                        attach = encr.decryptAES(attach, extrapass, attdata.ItemID, attdata.keyLength);
+                    }
+                }
+                if (attach == "exp") { // If our email has expired, then just send that
+                    com.openexchange.guard.util.Core.sendOK(response, "Attachment Expired");
+                    return;
+                }
+                // util.Core.sendOK(response, attdata.content_type, Base64.decode(attach));
+                if (attach.equals("")) {
+                    com.openexchange.guard.util.Core.sendFail(response, GetText.gettext("Unable to decode", lang));
+                } else {
+                    sendToSaveAttach(
+                        response,
+                        attdata.content_type,
+                        Base64.decode(attach),
+                        filename.replace(".grd2", "").replace(".grd", ""));
+                }
+                return;
+            }
+        } catch (InvalidKeySpecException | NoSuchAlgorithmException | SQLException | PGPException e) {
+        	logger.error("Error returning encrypted attachments for email " + emailid, e);
+        }
+        com.openexchange.guard.util.Core.sendFail(response, GetText.gettext("Unable to decode", lang));
+
+    }
+    
+
+    /**
+     * Send attachment contents to browser with flags to save to disk
+     * 
+     * @param response
+     * @param type
+     * @param answer
+     * @param filename
+     */
+    private void sendToSaveAttach(HttpServletResponse response, String type, byte[] answer, String filename) {
+        response.setContentType(type);
+        response.setHeader("Content-Disposition", "attachment; filename=\"" + filename + "\"");
+        response.addHeader("Access-Control-Allow-Origin", "*");
+        response.setStatus(HttpServletResponse.SC_OK);
+        try {
+            OutputStream out = response.getOutputStream();
+            out.write(answer);
+            out.close();
+        } catch (IOException e) {
+        	logger.error("Error writing output stream for file " + filename, e);
+        }
+    }
+    
 }
