@@ -1,13 +1,11 @@
 package com.openexchange.guard.server;
 
 import java.security.Security;
-
 import org.bouncycastle.jce.provider.BouncyCastleProvider;
-
 import com.google.gson.JsonObject;
 import com.openexchange.guard.config.Config;
-import com.openexchange.guard.database.RecipKey;
 import com.openexchange.guard.ox.Upgrader;
+import com.openexchange.guard.server.connectionPooling.HttpConnectionPoolService;
 import com.openexchange.guard.setup.MasterPasswordCreator;
 import com.openexchange.guard.setup.OxDatabaseCreateor;
 import com.openexchange.guard.setup.Repair;
@@ -15,10 +13,13 @@ import com.openexchange.guard.setup.Repair;
 public class HandleArgs {
 
 	public static boolean handle (String [] args) {
-		
-		if (args.length == 0) return (false);
+
+		if (args.length == 0) {
+            return (false);
+        }
 		Security.addProvider(new BouncyCastleProvider());
-		OxDbConn.init();
+		//OxDbConn.init();
+		HttpConnectionPoolService.initialize();
 		switch (args[0]) {
 		case "init":
 			System.out.println("init called, beginning setup");
@@ -26,10 +27,17 @@ public class HandleArgs {
         	if (!user.toLowerCase().contains("root")) {
         		System.out.println("Shows running as " + user);
         		System.out.println("Must run as root.  Sure you want to continue? \"yes\" to continue.");
-        		String response = System.console().readLine();
-        		if (!response.equals("yes")) return(true);
+        		try {
+        			String response = System.console().readLine();
+        			if (!response.equals("yes")) {
+                        return(true);
+                    }
+        		} catch (NullPointerException ex) {
+        			System.out.println("Null response, assuming from debug and continuing");
+        		}
+
         	}
-        	
+
 			if (Config.mysqlUsername.length() < 2) {
 				System.out.println("Set up configuration file first");
 				return(true);
@@ -44,8 +52,8 @@ public class HandleArgs {
 	            }
 			}
 			cf.setPermissions();
-			
-            
+
+
             return(true);
 		case "reset":
 			if (args.length < 2) {
@@ -140,6 +148,15 @@ public class HandleArgs {
 				System.out.println(result);
 			}
 			return(true);
+		case "upgradePGP":
+			com.openexchange.guard.setup.Upgrader up = new com.openexchange.guard.setup.Upgrader();
+			try {
+				up.doPGPUpdateAll();
+			} catch (Exception e) {
+				// TODO Auto-generated catch block
+				e.printStackTrace();
+			}
+			return (true);
 		default:
 			System.out.println("Usage: ./guard arg");
 			System.out.println("Possible arguments are:");
@@ -153,6 +170,7 @@ public class HandleArgs {
 			System.out.println("       eg. guard removepin john@somewhere.com");
 			System.out.println("upgrade: Upgrades a guest account to OX account");
 			System.out.println("       eg. guard upgrade jogn@somewhere.com");
+			System.out.println("upgradePGP: Upgrades from Guard pre 2.0 to PGP");
 			break;
 		}
 		return(true);
