@@ -50,37 +50,40 @@
 package com.openexchange.guard.server;
 
 import java.io.IOException;
-
 import javax.servlet.ServletException;
 import javax.servlet.http.HttpServlet;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
-
-import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
+import org.slf4j.MDC;
+import ch.qos.logback.classic.Logger;
+import com.openexchange.guard.config.Config;
+import com.openexchange.guard.encr.OffsiteKeys;
+import com.openexchange.guard.logging.LogAction;
+import com.openexchange.guard.mail.UploadGuest;
+import com.openexchange.guard.ox.InterOx;
+import com.openexchange.guard.pgp.PGPAttachments;
+import com.openexchange.guard.pgp.PGPPrivateKeyHandler;
+import com.openexchange.guard.pgp.PGPPublicHandler;
+import com.openexchange.guard.pgp.PgpHandler;
 import com.openexchange.guard.upseller.Upsell;
 import com.openexchange.guard.util.CheckOx;
 import com.openexchange.guard.util.Core;
-import com.openexchange.guard.config.Config;
-import com.openexchange.guard.encr.OffsiteKeys;
-import com.openexchange.guard.exceptions.GuardMissingParameter;
-import com.openexchange.guard.mail.UploadGuest;
-import com.openexchange.guard.ox.InterOx;
 
 public class MainServlet extends HttpServlet {
-	
-	private static Logger logger = LoggerFactory.getLogger(MainServlet.class);
-	
+
+	private static Logger logger = (Logger) LoggerFactory.getLogger(MainServlet.class);
+
     private final String actionpattern = "action=[a-z0-9]+";
 
     @Override
     protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
+    	LogAction.getPID();
         long start = System.nanoTime();
         String url = request.getRequestURI();
         logger.debug("url:" + url);
         try {
-            if (url.contains("/file/") || url.contains("/mail/")) {// File and attachments may have filename attached for browser data. Remove if present
+            if (url.contains("/file/") || url.contains("/mail/") || url.contains("/pgpmail/") || url.contains("/pgp/") || url.contains("/guest/")) {// File and attachments may have filename attached for browser data. Remove if present
                 int i = url.lastIndexOf("/");
                 url = url.substring(0, i);
             }
@@ -91,7 +94,8 @@ public class MainServlet extends HttpServlet {
         }
         String command = request.getQueryString();
         String action = getAction(command);
-        logger.info("Action: " + action + " From IP: " + Core.getIP(request));
+
+        LogAction.setURL(Core.getIP(request), url, action);
         com.openexchange.guard.server.OxCookie cookie = new OxCookie(request.getCookies());
 
         logger.debug("get action " + action);
@@ -101,17 +105,25 @@ public class MainServlet extends HttpServlet {
             FileHandler fh2 = new FileHandler();
             if (action.equals("get")) {
             	try {
-            		fh2.getFile(request, response, cookie, true);
+            		fh2.getFile(request, response, cookie);
             	} catch (Exception e) {
                 	logger.error("Problem getting decrypting file");
-                	handleException (e, request, response);
+
+                	//We cannot use the handleException method here
+                	//because it uses response.getWriter() and will throw
+                	//an exception because we might have already called
+                	//getOutputStream() on the response object
+                	LogAction.setSuccess(false);
+                	logger.error("Fail with url: " + request.getRequestURI(), e);
+                	response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, e.getMessage());
                 }
-            		
+
             }
             if (action.equals("remove")) {
             	try {
             		fh2.removeEncryption(request, response, cookie);
-            	} catch (Exception e) {
+            	}
+            	catch (Exception e) {
             		logger.error("Problem removing encryption");
             		handleException (e, request, response);
             	}
@@ -134,7 +146,7 @@ public class MainServlet extends HttpServlet {
         case "/mail":
             MailHandler mh = new MailHandler(cookie, request);
             switch (action) {
-           
+
             case ("getattach"):
             	try {
             		mh.getEncrAttach(request, response, cookie);
@@ -144,7 +156,7 @@ public class MainServlet extends HttpServlet {
                 }
                 break;
             case ("getkey"):
-                
+
                 try {
                     mh.getRecipKey(request, response);
                 } catch (Exception e) {
@@ -182,15 +194,142 @@ public class MainServlet extends HttpServlet {
         case "/pgpmail":
             PgpHandler pg = new PgpHandler();
             switch (action) {
-            case ("getmail"):
-                pg.getPGPemail(request, response, cookie);
-                break;
-
             case ("getkey"):
-                pg.getKey(request, response);
+            	try {
+            		pg.getKey(request, response);
+            	} catch (Exception e) {
+            		logger.error("Problem getting pgp public key");
+                	handleException (e, request, response);
+                }
+                break;
+	        case ("getkeybyid"):
+	        	try {
+	        		pg.getKeybyId(request, response);
+	        	} catch (Exception e) {
+	        		logger.error("Problem getting pgp public key");
+	            	handleException (e, request, response);
+	            }
+	            break;
+	        case ("verify"):
+	        	try {
+	        		pg.verifySignature(request, response, cookie);
+	        	} catch (Exception e) {
+	        		logger.error("Problem getting pgp public key");
+	            	handleException (e, request, response);
+	            }
+	            break;
+	        case ("getattach"):
+	        	try {
+	        		pg.getAttach(request, response, cookie);
+	        	} catch (Exception e) {
+	        		logger.error("Problem getting pgp attachment");
+	        		handleException (e, request, response);
+	        	}
+	            break;
+	        case ("getattachcid"):
+                try {
+                    pg.getAttachCID(request, response, cookie);
+                } catch (Exception e) {
+                    logger.error("Problem getting pgp content-id attachment");
+                    handleException (e, request, response);
+                }
+            break;
+	        case ("getinlineattach"):
+	        	try {
+	        		pg.getInlineAttach(request, response, cookie);
+	        	} catch (Exception e) {
+	        		logger.error("Problem getting pgp attachment");
+	        		handleException (e, request, response);
+	        	}
+	        break;
+	        case ("savepublicattach"):
+	        	PGPPublicHandler ph = new PGPPublicHandler();
+	        	try {
+	        		ph.saveKeyAttachment(request, response, cookie);
+	        	} catch (Exception e) {
+	        		logger.error("Problem saving PGP Public key Attachment");
+	        		handleException (e, request, response);
+	        	}
+	        break;
+	        case ("importvcard"):
+	            PgpHandler pgp = new PgpHandler();
+                try {
+                    pgp.importVCard(request, response, cookie);
+                } catch (Exception e) {
+                    logger.error("Problem importing vcard");
+                    handleException (e, request, response);
+                }
                 break;
             }
+        case "/pgp":
+        	switch (action) {
+	        case ("getpublist"):
+	        	try {
+	        		PGPPublicHandler ph = new PGPPublicHandler();
+	        		ph.getPubKeyList(request, response);
+	        	} catch (Exception e) {
+	        		logger.error("Problem getting pgp public key");
+	            	handleException (e, request, response);
+	            }
+	        break;
+	        case ("getuserkeys"):
+	            try {
+	                PgpHandler ph = new PgpHandler();
+	                ph.getUserKeys(request, response, cookie);
+
+	            } catch (Exception e) {
+                    logger.error("Problem getting pgp keys for user");
+                    handleException (e, request, response);
+                }
             break;
+	        case ("getsignatures"):
+	            try {
+                    PgpHandler ph = new PgpHandler();
+                    ph.checkSignatures(request, response, cookie);
+                } catch (Exception e) {
+                    logger.error("Problem checking signatures public key");
+                    handleException (e, request, response);
+                }
+            break;
+	        case ("getpubdetail"):
+	        	try {
+	        		PGPPublicHandler ph = new PGPPublicHandler();
+	        		ph.getPubKeyDetail(request, response);
+	        	} catch (Exception e) {
+	        		logger.error("Problem getting pgp public key");
+	            	handleException (e, request, response);
+	            }
+	        break;
+	        case ("getkeyimage"):
+                try {
+                    PgpHandler ph = new PgpHandler();
+                    ph.getKeyImage(request, response, cookie);
+                } catch (Exception e) {
+                    logger.error("Problem getting pgp image");
+                    handleException (e, request, response);
+                }
+            break;
+	        case ("getprivkeybyid"):
+	        	try {
+	        		PgpHandler pgk = new PgpHandler();
+	        		pgk.getPrivateKeybyId(request, response, cookie);
+	        	} catch (Exception e) {
+	        		logger.error("Problem getting pgp public key");
+	            	handleException (e, request, response);
+	            }
+	            break;
+	        case ("makecurrent"):
+	            try {
+	                PGPPrivateKeyHandler pkh = new PGPPrivateKeyHandler();
+	                pkh.makeCurrent(request, response, cookie);
+	            } catch (Exception e) {
+                    logger.error("Problem making key current");
+                    handleException (e, request, response);
+                }
+                break;
+        	}
+        break;
+
         case "/guest":
             Guest gst = new Guest();
             switch (action) {
@@ -203,7 +342,7 @@ public class MainServlet extends HttpServlet {
                 }
                 break;
             case ("getquestion"):
-            	try {	
+            	try {
             		gst.getQuestion(request, response, cookie);
             	 } catch (Exception e) {
                  	logger.error("Error getting guest question for reset ", e);
@@ -285,7 +424,7 @@ public class MainServlet extends HttpServlet {
                 	handleException (e, request, response);
                 }
                 break;
-	        case ("reset"): 
+	        case ("reset"):
 	        	try {
 	        		auth.resetPass(request, response, cookie);
 	        	} catch (Exception e) {
@@ -323,10 +462,12 @@ public class MainServlet extends HttpServlet {
             break;
         }
         logger.debug("GET processed, time " + Core.gettime(start));
+        resetMDC();
     }
 
+    @Override
     protected void doDelete(HttpServletRequest request, HttpServletResponse response) throws IOException {
-
+    	LogAction.getPID();
         String url = request.getRequestURI();
 
         com.openexchange.guard.server.OxCookie cookie = new OxCookie(request.getCookies());
@@ -338,9 +479,7 @@ public class MainServlet extends HttpServlet {
             logger.error("Problem with url " + url);
             return;
         }
-        logger.debug("url:" + url);
-        logger.debug("delete action: " + action);
-        logger.debug("command: " + command);
+        LogAction.setURL(Core.getIP(request), url, action);
         switch (url) {
         case "/file":
             FileHandler fh = new FileHandler();
@@ -350,11 +489,26 @@ public class MainServlet extends HttpServlet {
                 break;
             }
             break;
+        case "/pgp":
+        	switch (action) {
+        	case "share":
+        		PGPPublicHandler ph = new PGPPublicHandler();
+        		try {
+        			ph.deleteKey(request, response);
+        		} catch (Exception e) {
+        			logger.error("Problem with delete public key " , e);
+                	handleException (e, request, response);
+        		}
+        	}
+        	break;
         }
+        resetMDC();
+
     }
 
+    @Override
     protected void doPut(HttpServletRequest request, HttpServletResponse response) throws IOException {
-
+    	LogAction.getPID();
         String url = request.getRequestURI();
 
         com.openexchange.guard.server.OxCookie cookie = new OxCookie(request.getCookies());
@@ -366,9 +520,7 @@ public class MainServlet extends HttpServlet {
             logger.error("Problem with url " + url);
             return;
         }
-        logger.debug("url:" + url);
-        logger.debug("put action: " + action);
-        logger.debug("command: " + command);
+        LogAction.setURL(Core.getIP(request), url, action);
         logger.info("Action: " + action + " From IP: " + Core.getIP(request));
         switch (url) {
         case "/mail":
@@ -395,11 +547,45 @@ public class MainServlet extends HttpServlet {
                 break;
             }
             break;
+        case "/pgp":
+        	PGPPublicHandler ph = new PGPPublicHandler();
+        	switch (action) {
+	        	case ("share"):
+
+	    			try {
+	    				ph.shareKey(request, response);
+	    			} catch (Exception e) {
+	    			logger.error("Problem with share public key " , e);
+	            	handleException (e, request, response);
+
+	    			}
+	    		break;
+	        	case ("inline"):
+	    			try {
+	    				ph.changeInline(request, response);
+	    			} catch (Exception e) {
+	    			logger.error("Problem with changing inline public key " , e);
+	            	handleException (e, request, response);
+
+	    			}
+	    		break;
+	        	case ("savedecodedattachment"):
+	        	    try {
+	        	        PGPAttachments att = new PGPAttachments();
+	        	        att.saveDecodedAttach(request, response, cookie);
+	        	    } catch (Exception e) {
+	                    logger.error("Problem saving decoded attachment " , e);
+	                    handleException (e, request, response);
+	                    }
+        	}
+	    break;
         }
+        resetMDC();
     }
 
+    @Override
     protected void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException {
-
+    	LogAction.getPID();
         long start = System.nanoTime();
         String url = request.getRequestURI();
 
@@ -412,9 +598,7 @@ public class MainServlet extends HttpServlet {
             logger.error("Problem with url " + url);
             return;
         }
-        logger.debug("url:" + url);
-        logger.debug("post action: " + action);
-        logger.debug("command: " + command);
+        LogAction.setURL(Core.getIP(request), url, action);
         logger.info("Action: " + action + " From IP: " + Core.getIP(request));
         switch (url) {
         case "/mail":
@@ -449,7 +633,102 @@ public class MainServlet extends HttpServlet {
                 break;
             }
             break;
-            
+        case "/pgpmail":
+            PgpHandler pg = new PgpHandler();
+            switch (action) {
+            case ("getmail"):
+                pg.getPGPemail(request, response, cookie);
+                break;
+            case ("getinline"):
+            	try {
+            		pg.getPGPInline(request, response, cookie);
+            	} catch (Exception e) {
+            		logger.error("Error getting PGP inline ", e);
+                	handleException (e, request, response);
+            	}
+            	break;
+            }
+            break;
+        case "/retrieve":
+            KeyRetriever kr = new KeyRetriever();
+            switch (action) {
+                case ("login"):
+                    try {
+                        kr.login(request, response, cookie);
+                    } catch (Exception e) {
+                        logger.error("Error logging in to retrieve key ", e);
+                        handleException (e, request, response);
+                    }
+                break;
+                case ("getkeys"):
+                    try {
+                        kr.getkeys(request, response, cookie);
+                    } catch (Exception e) {
+                        logger.error("Error retrieving key ", e);
+                        handleException (e, request, response);
+                    }
+                break;
+            }
+            break;
+        case "/pgp":
+        	switch (action) {
+        	case ("uploadkeys"):
+        		try {
+        			PGPPublicHandler pgppub = new PGPPublicHandler ();
+        			pgppub.incomingPGPForm(request, response);
+
+        		} catch (Exception e) {
+            		logger.error("Error uploading PGP pulbic key ", e);
+                	handleException (e, request, response);
+            	}
+        	break;
+        	case ("uploadprivate"):
+        	    try {
+        	        PGPPrivateKeyHandler pgppriv = new PGPPrivateKeyHandler();
+        	        pgppriv.incomingPGPForm(request, response, cookie);
+        	    } catch (Exception e) {
+                    logger.error("Error uploading PGP private key ", e);
+                    handleException (e, request, response);
+                }
+            break;
+        	case ("deleteprivkey"):
+        	    try {
+        	        PGPPrivateKeyHandler pgppriv = new PGPPrivateKeyHandler();
+        	        pgppriv.DeletePrivateKey(request, response, cookie);
+        	    } catch (Exception e) {
+                    logger.error("Error deleting PGP private key ", e);
+                    handleException (e, request, response);
+                }
+            break;
+            case ("deletelocalpublic"):
+                try {
+                    PGPPrivateKeyHandler pgppriv = new PGPPrivateKeyHandler();
+                    pgppriv.deleteLocalPublic(request, response, cookie);
+                } catch (Exception e) {
+                    logger.error("Error deleting PGP public key ", e);
+                    handleException (e, request, response);
+                }
+            break;
+            case ("revokeprivkey"):
+                try {
+                    PGPPrivateKeyHandler pgppriv = new PGPPrivateKeyHandler();
+                    pgppriv.revokePrivateKey(request, response, cookie);
+                } catch (Exception e) {
+                    logger.error("Error revoking PGP private key ", e);
+                    handleException (e, request, response);
+                }
+            break;
+            case ("adduserid"):
+                try {
+                    PGPPrivateKeyHandler pgppriv = new PGPPrivateKeyHandler();
+                    pgppriv.addUID(request, response, cookie);
+                } catch (Exception e) {
+                    logger.error("Error adding userID to PGP Public Key ", e);
+                    handleException (e, request, response);
+                }
+            break;
+
+        	}
         case "/file":
             FileHandler fh = new FileHandler();
             switch (action) {
@@ -461,7 +740,7 @@ public class MainServlet extends HttpServlet {
                 	handleException (e, request, response);
                 }
                 break;
-        	case ("addextra"): 
+        	case ("addextra"):
         		try {
         			fh.addExtraPassword(request, response, cookie);
         		} catch (Exception e) {
@@ -476,7 +755,7 @@ public class MainServlet extends HttpServlet {
             switch (action) {
             case ("new"):
             	try {
-            		fh3.downloadFormFromClient(request, response, cookie, false);
+            		fh3.encryptUpload(request, response, cookie, false);
             	} catch (Exception e) {
             		logger.error("Error encrypting new file", e);
             		handleException (e, request, response);
@@ -484,7 +763,7 @@ public class MainServlet extends HttpServlet {
                 break;
             case ("update"):
             	try {
-            		fh3.downloadFormFromClient(request, response, cookie, true);
+            		fh3.encryptUpload(request, response, cookie, true);
             	} catch (Exception e) {
             		logger.error("Error updating file", e);
             		handleException (e, request, response);
@@ -495,7 +774,7 @@ public class MainServlet extends HttpServlet {
         case "/mailattach":
             FileHandler fh2 = new FileHandler();
             try {
-            	fh2.downloadFormFromClient(request, response, cookie, false);
+            	fh2.encryptUpload(request, response, cookie, false);
             } catch (Exception e) {
         		logger.error("Error mail attach", e);
         		handleException (e, request, response);
@@ -573,6 +852,14 @@ public class MainServlet extends HttpServlet {
                     handleException (e, request, response);
                 }
             	break;
+            case ("deleterecovery"):
+                try {
+                    auth.deleteRecovery(request, response, cookie);
+                } catch (Exception e) {
+                    logger.error("Error deleting recovery", e);
+                    handleException (e, request, response);
+                }
+                break;
             }
             break;
         case "/settings":
@@ -588,7 +875,7 @@ public class MainServlet extends HttpServlet {
         	    break;
         	}
            break;
-        	
+
         case "/interox":
             InterOx iox = new InterOx();
             switch (action) {
@@ -610,22 +897,31 @@ public class MainServlet extends HttpServlet {
             break;
         }
         logger.debug("POST action processed, time " + Core.gettime(start));
+        resetMDC();
     }
 
-    private void handleException (Exception ex, HttpServletRequest request, HttpServletResponse response) {
+    private static void resetMDC() {
+        MDC.clear();
+    }
+
+    public static void handleException (Exception ex, HttpServletRequest request, HttpServletResponse response) {
     	String url = request.getRequestURI();
     	Core.sendError(response, ex.getMessage());
+    	LogAction.setSuccess(false);
     	logger.error("Fail with url: " + url, ex);
     }
     /**
      * Parse action from command string
-     * 
+     *
      * @param command
      * @return
      */
     private String getAction(String command) {
         String action = "";
         try {
+            if (command == null) {
+                return "";
+            }
             command = command.toLowerCase();
             if (command.contains("action=")) {
                 action = command.substring(command.indexOf("action="));
@@ -639,5 +935,5 @@ public class MainServlet extends HttpServlet {
         }
         return (action);
     }
-    
+
 }
