@@ -91,8 +91,9 @@ public class OxCookie {
             if (cookies.length > 0) {// Make sure the cookies are present
                 for (int i = 0; i < cookies.length; i++) {
                     String name = cookies[i].getName();
-                    if (name.startsWith("JSESSIONID"))
+                    if (name.startsWith("JSESSIONID")) {
                         JSESSIONID = checkCR(cookies[i].getValue());
+                    }
                     if (name.startsWith("open-xchange-secret")) {
                     	cookie ck = new cookie();
                     	ck.name = name;
@@ -116,8 +117,9 @@ public class OxCookie {
                     	ck.value = checkCR(cookies[i].getValue());
                     	publicsession.add(ck);
                     }
-                    if (name.startsWith("OxReaderID"))
+                    if (name.startsWith("OxReaderID")) {
                         OxReaderID = checkCR(cookies[i].getValue());
+                    }
                 }
             }
         } catch (Exception ex) {
