@@ -88,12 +88,40 @@ public class Sender {
     static private String password = Config.SMTPPassword;
 
     public boolean send (JsonObject email, ArrayList<Attachment> attachments) {
-    	String messageId = "<OxGuard-" + EncrLib.getUUID() + ">";
+    	String messageId = Config.getMessageID();
     	return (send (email, attachments, "", Config.default_lang, messageId));
     }
     
-    public boolean send(JsonObject email, ArrayList<Attachment> attachments, String sender, String lang, String messageId) {
-        Properties prop = new Properties();
+    /**
+     * Sends Mail message using SMTP Server
+     * @param msg
+     * @return
+     * @throws UnsupportedEncodingException 
+     */
+    public boolean sendMessage (Message msg, String sender, String lang) throws UnsupportedEncodingException {
+        Session session = getSession();
+    	Transport transport;
+		try {
+		    if (!sender.equals("")) {
+		        msg.addHeader("Reply-To", msg.getFrom()[0].toString());
+		        InternetAddress fromaddr = (InternetAddress) msg.getFrom()[0];
+                fromaddr.setPersonal(GetText.gettext("Sent on behalf of %s", lang, ((fromaddr.getPersonal() == null) ? "" : (fromaddr.getPersonal().replace("\"", "") + " - ")) + fromaddr.getAddress().replace("<", "").replace(">", "")));
+                fromaddr.setAddress(sender);
+                msg.setFrom(fromaddr);
+            }
+			transport = session.getTransport("smtp");
+			transport.connect(Config.SMTPServer, Config.SMTPPort, Config.SMTPUsername, Config.SMTPPassword);
+			transport.sendMessage(msg, msg.getAllRecipients());
+			transport.close();
+		} catch (MessagingException e) {
+		    logger.error("Error sending email", e);
+			return (false);
+		}
+        return (true);
+    }
+    
+    private Session getSession () {
+    	Properties prop = new Properties();
         boolean authenticate = true;
         if (username.length() < 2) {
         	authenticate = false;
@@ -103,19 +131,97 @@ public class Sender {
         prop.put("mail.smtp.host", Config.SMTPServer);
         prop.put("mail.smtp.port", Integer.toString(Config.SMTPPort));
         if (Config.useStartTls) prop.put("mail.smtp.starttls.enable", "true");
+        logger.debug("Using SMTP server " + Config.SMTPServer + ":" + Config.SMTPPort + ", TLS=" + (Config.useStartTls ? "True" : "False"));
+        Session session = null;
+        if (authenticate) {
+            session = Session.getInstance(prop, new Authenticator() {
+
+                protected PasswordAuthentication getPasswordAuthentication() {
+                    return new PasswordAuthentication(username, password);
+                }
+            });
+        } else {
+        	session = Session.getInstance(prop);
+        }
+        return (session);
+    }
+    
+    public boolean send(JsonObject email, ArrayList<Attachment> attachments, String sender, String lang, String messageId) {
 
         try {
-            email = email.get("data").getAsJsonObject();
-            String to = email.get("to").getAsString();
 
-            Address[] addrTo = new Address[1];
-            addrTo[0] = new InternetAddress(to);  // right now, only one recipient per email
-            //     String[] recips = to.split(">,");
-        //    for (int i = 0; i < recips.length; i++) {
-         //       addrTo[i] = new InternetAddress(recips[i]);
-         //   }
-            String from =  email.get("from").getAsString();
-            InternetAddress fromaddr = new InternetAddress(from);
+            Session session = getSession();
+            MimeMessage message = new MimeMessage(session);
+            email = email.get("data").getAsJsonObject();
+            // Parse Json for email headers
+            // To
+            if (email.get("to").isJsonArray()) {
+                JsonArray toAddrArray = email.get("to").getAsJsonArray();
+                Address[] addrTo = new Address[toAddrArray.size()];
+                for (int i = 0; i < toAddrArray.size(); i++) {
+                    JsonArray addr = toAddrArray.get(i).getAsJsonArray();
+                    InternetAddress toaddr = new InternetAddress(addr.get(1).getAsString(), addr.get(0).getAsString(), "UTF-8");
+                    addrTo[i] = toaddr;
+                }
+                message.setRecipients(Message.RecipientType.TO, addrTo);
+            } else {
+                String to = email.get("to").getAsString();
+                if (to != null && to.contains("@")) {
+                    Address[] addrTo = new Address[1];
+                    addrTo[0] = new InternetAddress(to);  // right now, only one recipient per email if just string
+                    message.setRecipients(Message.RecipientType.TO, addrTo);
+                }
+            }
+            // CC
+            if (email.get("cc").isJsonArray()) {
+                JsonArray ccAddrArray = email.get("cc").getAsJsonArray();
+                Address[] addrCc = new Address[ccAddrArray.size()];
+                for (int i = 0; i < ccAddrArray.size(); i++) {
+                    JsonArray addr = ccAddrArray.get(i).getAsJsonArray();
+                    InternetAddress ccaddr = new InternetAddress(addr.get(1).getAsString(), addr.get(0).getAsString(), "UTF-8");
+                    addrCc[i] = ccaddr;
+                }
+                message.setRecipients(Message.RecipientType.CC, addrCc);
+            } else {
+                String cc = email.get("cc").getAsString();
+                if (cc != null && cc.contains("@")) {
+                    Address[] addrCc = new Address[1];
+                    addrCc[0] = new InternetAddress(cc);  // right now, only one recipient per email if just string
+                    message.setRecipients(Message.RecipientType.CC, addrCc);
+                }
+            }
+            // Bcc
+            if (email.get("bcc").isJsonArray()) {
+                JsonArray bccAddrArray = email.get("bcc").getAsJsonArray();
+                Address[] addrBcc = new Address[bccAddrArray.size()];
+                for (int i = 0; i < bccAddrArray.size(); i++) {
+                    JsonArray addr = bccAddrArray.get(i).getAsJsonArray();
+                    InternetAddress bccaddr = new InternetAddress(addr.get(1).getAsString(), addr.get(0).getAsString(), "UTF-8");
+                    addrBcc[i] = bccaddr;
+                }
+                message.setRecipients(Message.RecipientType.BCC, addrBcc);
+            } else {
+                String bcc = email.get("bcc").getAsString();
+                if (bcc != null && bcc.contains("@")) {
+                Address[] addrBcc = new Address[1];
+                addrBcc[0] = new InternetAddress(bcc);  // right now, only one recipient per email if just string
+                message.setRecipients(Message.RecipientType.BCC, addrBcc);
+                }
+
+            }
+            // From
+            InternetAddress fromaddr = null;
+            InternetAddress replyaddr = null;
+            if (email.get("from").isJsonArray()) {
+                JsonArray fromArray = email.get("from").getAsJsonArray();
+                fromaddr = new InternetAddress(fromArray.get(0).getAsJsonArray().get(1).getAsString(), fromArray.get(0).getAsJsonArray().get(0).getAsString(), "UTF-8");
+                replyaddr = new InternetAddress(fromaddr.getAddress(), fromaddr.getPersonal(), "UTF-8");
+            } else {
+                String from =  email.get("from").getAsString();
+                fromaddr = new InternetAddress(from);
+                replyaddr = new InternetAddress(from);
+            }
+
             if (!sender.equals("")) {
             	fromaddr.setPersonal(GetText.gettext("Sent on behalf of %s", lang, ((fromaddr.getPersonal() == null) ? "" : (fromaddr.getPersonal().replace("\"", "") + " - ")) + fromaddr.getAddress().replace("<", "").replace(">", "")));
             	fromaddr.setAddress(sender);
@@ -135,30 +241,17 @@ public class Sender {
             	}
             }
             
-            Session session = null;
-            if (authenticate) {
-	            session = Session.getInstance(prop, new Authenticator() {
-	
-	                protected PasswordAuthentication getPasswordAuthentication() {
-	                    return new PasswordAuthentication(username, password);
-	                }
-	            });
-            } else {
-            	session = Session.getInstance(prop);
-            }
 
-            Message message = new MimeMessage(session);
             if (email.get("headers") != null) {
                 JsonObject headers = email.get("headers").getAsJsonObject();
                 for (Map.Entry<String, JsonElement> h : headers.entrySet()) {
                 	message.setHeader(h.getKey(), checkCR(h.getValue().getAsString()));
                 }
             }
-            InternetAddress replyaddr = new InternetAddress(from);
+
             message.setHeader("Reply-To", replyaddr.toString());
-            message.setRecipients(Message.RecipientType.TO, addrTo);
-            ;
-            message.setSubject(subject);
+            
+            message.setSubject(subject, "UTF-8");
             MimeBodyPart multipart = new MimeBodyPart();  // Main message wrapper
             // Create alternative wrapped message
             Multipart alternate = new MimeMultipart("alternative");
@@ -233,6 +326,7 @@ public class Sender {
             transport.connect(Config.SMTPServer, Config.SMTPPort, Config.SMTPUsername, Config.SMTPPassword);
             transport.sendMessage(message, message.getAllRecipients());
             transport.close();
+            logger.debug("Success SMTP send");
             // Transport.send(message);;
 
         } catch (Exception ex) {
