@@ -4,12 +4,14 @@ import com.openexchange.guard.config.Config;
 
 public class Settings {
 	public enum permissions {
-        NONE, MAIL, FILE, BOTH
+        NONE, GUARD, MAIL, FILE, BOTH
     };
     
     public permissions perm = permissions.NONE;
     public boolean extrapass = Config.ok_extra_password;  // Set to default config level
     public boolean privatepass = true;
-    public boolean deleteRecovery = true;
+    public boolean noDeleteRecovery = false;
     public boolean deleteKeyOnRevoke = Config.deleteOnRetract;
+    public boolean noDeletePrivate = false;
+    public boolean norecovery = false;
 }
