@@ -52,14 +52,18 @@ package com.openexchange.guard.server;
 
 import java.lang.management.ManagementFactory;
 import java.security.Security;
+import java.util.List;
 import java.util.jar.Attributes;
 import java.util.jar.JarFile;
 import java.util.jar.Manifest;
-
+import javax.crypto.Cipher;
 import javax.management.MBeanServer;
 import javax.management.ObjectName;
-
 import org.bouncycastle.jce.provider.BouncyCastleProvider;
+import org.eclipse.jetty.security.ConstraintMapping;
+import org.eclipse.jetty.security.ConstraintSecurityHandler;
+import org.eclipse.jetty.security.HashLoginService;
+import org.eclipse.jetty.security.authentication.BasicAuthenticator;
 import org.eclipse.jetty.server.Connector;
 import org.eclipse.jetty.server.Handler;
 import org.eclipse.jetty.server.HttpConfiguration;
@@ -77,12 +81,13 @@ import org.eclipse.jetty.server.handler.ResourceHandler;
 import org.eclipse.jetty.servlet.ServletContextHandler;
 import org.eclipse.jetty.servlet.ServletHolder;
 import org.eclipse.jetty.util.resource.Resource;
+import org.eclipse.jetty.util.security.Constraint;
+import org.eclipse.jetty.util.security.Credential;
 import org.eclipse.jetty.util.ssl.SslContextFactory;
+import org.joda.time.DateTime;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
 import ch.qos.logback.classic.LoggerContext;
-
 import com.amazonaws.AmazonClientException;
 import com.amazonaws.AmazonServiceException;
 import com.openexchange.guard.cacheitems.Cache;
@@ -91,27 +96,35 @@ import com.openexchange.guard.cron.Jobs;
 import com.openexchange.guard.database.OGInit;
 import com.openexchange.guard.dbpool.Manager;
 import com.openexchange.guard.encr.PopulateRSA;
+import com.openexchange.guard.logging.LogMarkers;
 import com.openexchange.guard.ox.Languages;
+import com.openexchange.guard.server.authentication.BasicAuthSecurityHandlerFactory;
+import com.openexchange.guard.server.authentication.GuardAuthRole;
+import com.openexchange.guard.server.authentication.GuardAuthRoles;
+import com.openexchange.guard.server.connectionPooling.HttpConnectionPoolService;
 import com.openexchange.guard.setup.Upgrader;
 import com.openexchange.guard.storage.s3.S3ClientFactory;
 
 public class Startup {
 
 	private static Logger logger = LoggerFactory.getLogger(Startup.class);
-	
+
     public static void main(String[] args) throws Exception {
         if (com.openexchange.guard.setup.Check.confirm(args)) {
             try {
                 final Server server = new Server();
                 Security.addProvider(new BouncyCastleProvider());
-
                 setLogLevel();
-                
+                if (Cipher.getMaxAllowedKeyLength("AES") < 256) {
+                	logger.warn("The JAVA security package installed only allows AES key length " + Cipher.getMaxAllowedKeyLength("AES") +
+                			".  Key lengths < 256 will often cause PGP to fail");
+                }
                 Manager.init(); // Initialize the connection pool manager
-                OxDbConn.init(); // Initialize ox backend http pool
-                GuardServerStats stats = new GuardServerStats();
+                //OxDbConn.init(); // Initialize ox backend http pool
+                HttpConnectionPoolService.initialize();
+                GuardServerStats stats = new GuardServerStats(HttpConnectionPoolService.getStats());
                 MBeanServer mbs = ManagementFactory.getPlatformMBeanServer();
-                ObjectName name = new ObjectName("com.open-xchange.guard:name=GuardServerStats");		
+                ObjectName name = new ObjectName("com.open-xchange.guard:name=GuardServerStats");
                 mbs.registerMBean(stats, name);
                 if (Config.getStorageType().equals("s3")) {
                 	try {
@@ -132,24 +145,34 @@ public class Startup {
                         }
                         logger.info("Using file store as storage for cached objects");
                     } catch (Exception e) {
-                        logger.error("Problem with filestore", e);
+                        logger.error(LogMarkers.NOTIFY_MAJOR, "Problem with filestore", e);
                         Runtime.getRuntime().halt(1);
                     }
                 } else {
                     logger.error("Unknown value \"" + Config.getStorageType() + "\" for configuration com.openexchange.guard.storage.type. Exiting.");
                 	Runtime.getRuntime().halt(1);
                 }
-                                
 
+
+                int count = 0;
                 while (!com.openexchange.guard.util.CheckOx.oxAlive()) {
-                    logger.error("OX Not responding.  Verify Rest API installed and backend running.  Will wait 15 seconds and try again");
+                    count++;
+                    if (args.length > 0) {  // If was called from command line, then exit and print
+                        System.out.println("Unable to connect to the OX backend");
+                        Runtime.getRuntime().halt(1);
+                    }
+                    if (count == 4) {  // If no response after 4 tries, then notify admin
+                        logger.error(LogMarkers.NOTIFY_MAJOR, "OX Not responding.  Verify Rest API installed and backend running.  Will wait 15 seconds and try again");
+                    } else {
+                        logger.error("OX Not responding.  Verify Rest API installed and backend running.  Will wait 15 seconds and try again");
+                    }
                     Thread.sleep(15000);
                 }
-                
+
                 OGInit.initGuardDatabase(); // Initialize OxGuard databases
-                
+
                 Config.loadMysqlConfigData();
-                
+
                 Upgrader upgrade = new Upgrader();
                 if (upgrade.checkUpgrade()) {
                 	if (!upgrade.doUpgrade()) {
@@ -157,7 +180,7 @@ public class Startup {
                         Runtime.getRuntime().halt(1);
                 	}
                 }
-                
+
                 final Thread rsathread = new Thread(new PopulateRSA());
                 server.setStopTimeout(10000L);
                 Runtime.getRuntime().addShutdownHook(new Thread() {
@@ -166,7 +189,8 @@ public class Startup {
                     public void run() {
                         logger.info("Shutting down....Up to 10 second delay to finish");
                         try {
-                            OxDbConn.close();
+                            //OxDbConn.close();
+                            HttpConnectionPoolService.closeAll();
                             server.stop();
                             server.destroy();
                             try {
@@ -185,12 +209,14 @@ public class Startup {
                 });
                 // Handle arguments, and if done, exit
                 Languages lang = new Languages(); // Initialize translation engine
-                
-                if (HandleArgs.handle(args)) Runtime.getRuntime().halt(0);
-                
+
+                if (HandleArgs.handle(args)) {
+                    Runtime.getRuntime().halt(0);
+                }
+
                 Cache cache = new Cache(); // Initialize the cache
                 Jobs cronJobs = new Jobs(); // Initialize the cron jobs
-                
+
 
                 if (Config.rsa_cache) { // If rsa caching enabled, start
                     rsathread.setPriority(Thread.MIN_PRIORITY);// Low priority background process
@@ -222,10 +248,40 @@ public class Startup {
                 } else {
                     server.setConnectors(new Connector[] { http });
                 }
+
+                //Setting up a security handler factory for creating BASIC AUTH handlers
+                BasicAuthSecurityHandlerFactory securityHandlerFactory =
+                    new BasicAuthSecurityHandlerFactory(GuardAuthRoles.getAvailableRoles());
+
+                // Main API context
                 ServletContextHandler context0 = new ServletContextHandler(ServletContextHandler.SESSIONS);
                 context0.setContextPath("/oxguard");
                 context0.addServlet(new ServletHolder(new MainServlet()), "/*");
 
+                // PGP Context
+                ServletContextHandler pgpcontext = new ServletContextHandler(ServletContextHandler.SESSIONS);
+                pgpcontext.setContextPath("/pgp");
+                pgpcontext.addServlet(new ServletHolder(new PGPServlet()),  "/*");
+
+                // Administration context
+                ServletContextHandler admincontext = new ServletContextHandler(ServletContextHandler.SESSIONS);
+                admincontext.setContextPath("/guardadmin");
+                admincontext.addServlet(new ServletHolder(new AdminServlet()),  "/*");
+                admincontext.setSecurityHandler(
+                    securityHandlerFactory.CreateSecurityHandler(Config.OXBackendUser,
+                                                                Credential.getCredential(Config.OXBackendPassword),
+                                                                new GuardAuthRole[] {GuardAuthRole.OXBackend}));
+
+                //Support context
+                ServletContextHandler supportcontext = new ServletContextHandler(ServletContextHandler.SESSIONS);
+                supportcontext.setContextPath("/guardsupport");
+                supportcontext.addServlet(new ServletHolder(new SupportServlet()), "/*");
+                supportcontext.setSecurityHandler(
+                    securityHandlerFactory.CreateSecurityHandler(Config.SupportApiUser,
+                                                                 Credential.getCredential(Config.SupportApiPassword),
+                                                                 new GuardAuthRole[]{GuardAuthRole.SupportAPI}));
+
+
                 ResourceHandler rh0 = new ResourceHandler();
                 logger.debug("Code Path: " + Config.codePath);
                 rh0.setBaseResource(Resource.newResource(Config.codePath+ "reader"));
@@ -234,16 +290,30 @@ public class Startup {
                 context1.setHandler(rh0);
 
                 ContextHandlerCollection contexts = new ContextHandlerCollection();
-                contexts.setHandlers(new Handler[] { context0, context1 });
+                contexts.setHandlers(new Handler[] { context0,
+                                                     context1,
+                                                     pgpcontext,
+                                                     admincontext,
+                                                     supportcontext});
 
                 HandlerCollection handlers = new HandlerCollection();
                 RequestLogHandler requestLogHandler = new RequestLogHandler();
                 handlers.setHandlers(new Handler[] { contexts, new DefaultHandler(), requestLogHandler });
                 server.setHandler(handlers);
-                
-                logger.info("Guard server started");
+
+                logger.info("Guard server started at " + DateTime.now().toLocalDateTime());
                 logVersion();
-                server.start();
+                try {
+                	server.start();
+                } catch (org.eclipse.jetty.util.MultiException e2) {
+                	List<Throwable> ex = e2.getThrowables();
+                	for (int i = 0; i < ex.size(); i++) {
+                		Throwable t = ex.get(i);
+                		logger.error(t.getMessage());
+                	}
+                	logger.error(LogMarkers.NOTIFY_MAJOR, "Unable to start server");
+                	Runtime.getRuntime().halt(1);
+                }
                 server.join();
             } catch (java.net.BindException e) {
                 logger.error("Port already in use", e);
@@ -251,7 +321,29 @@ public class Startup {
             }
         }
     }
-    
+
+    private static ConstraintSecurityHandler setupAdminContraint() {
+        Constraint constraint = new Constraint(Constraint.__BASIC_AUTH, "user");
+        constraint.setRoles(new String[] {"backend", "admin"});
+        constraint.setAuthenticate(true);;
+
+        ConstraintMapping cmap = new ConstraintMapping();
+        cmap.setConstraint(constraint);
+        cmap.setPathSpec("/*");
+
+        ConstraintSecurityHandler csh = new ConstraintSecurityHandler();
+        csh.setAuthenticator(new BasicAuthenticator ());
+        csh.setRealmName("Guard");
+        csh.setConstraintMappings(new ConstraintMapping[] {cmap});
+        HashLoginService hls = new HashLoginService();
+        hls.setName("Guard");
+        hls.putUser(Config.OXBackendUser,Credential.getCredential(Config.OXBackendPassword), new String[] {"backend"});
+
+        csh.setLoginService(hls);
+        return (csh);
+
+    }
+
     private static void setLogLevel() {
         ch.qos.logback.classic.Logger root = (ch.qos.logback.classic.Logger) LoggerFactory.getLogger(org.slf4j.Logger.ROOT_LOGGER_NAME);
         root.setLevel(Config.logLevel);
@@ -262,12 +354,15 @@ public class Startup {
         	}
         }
     }
-    
+
     private static void logVersion() {
     	try {
     	String loc = FileHandler.class.getProtectionDomain().getCodeSource().getLocation().toString();
     	loc = loc.replace("file:", "");
-    	if (!loc.contains(".jar")) return;  // If local debug
+    	if (!loc.contains(".jar"))
+         {
+            return;  // If local debug
+        }
     	JarFile jf = new JarFile(loc);
     	Manifest manifest = jf.getManifest();
     	Attributes entries = manifest.getMainAttributes();
@@ -278,8 +373,8 @@ public class Startup {
     	} catch (Exception ex) {
     		logger.error("Problem getting version ", ex);
     	}
-    	
-    	
+
+
     }
 
 }
