@@ -103,7 +103,7 @@ public class Check {
             }
             Statement stmt = conn.createStatement();
             ResultSet rs;
-            rs = stmt.executeQuery("SELECT * FROM db_pool");
+            rs = stmt.executeQuery("SELECT db_pool_id FROM db_pool");
             if (rs.next()) {
                 logger.debug("OK Connection to configdb");
             } else {
@@ -120,7 +120,7 @@ public class Check {
             try {
                 Connection ogcon = DriverManager.getConnection(OGmysqlConnectionString);
                 Statement ogst = ogcon.createStatement();
-                ResultSet ogrs = ogst.executeQuery("SELECT * FROM global_settings WHERE global_settings.key = 'version'");
+                ResultSet ogrs = ogst.executeQuery("SELECT value FROM global_settings WHERE global_settings.key = 'version'");
                 if (ogrs.next()) {
                     int installedversion = ogrs.getInt("value");
                     if (installedversion < version) {
@@ -156,14 +156,23 @@ public class Check {
                 rs.close();
                 conn.close();
             } catch (Exception ex) {
-                System.out.println("Unable to connect OxGuard");
+                if (ex.getMessage().contains("Unknown database")) {
+                    System.out.println("Database not yet initialized");
+                } else {
+                    System.out.println("Unable to connect OxGuard");
+                    System.out.println(ex.getMessage());
+                }
+                
                 if (HandleArgs.handle(args)) Runtime.getRuntime().halt(0);
-                logger.error("Unable to connect to OxGuard", ex);
             }
-            logger.error("Unable to connect to oxguard database.  If first run, then use argument init?");
-            System.out.println("Unable to connect to oxguard database.  If first run, then use argument init?");
+            logger.error("Unable to connect to oxguard database.  Check permissions. If first run, then use argument init");
+            System.out.println("Unable to connect to oxguard database.  Check permissions.  If first run, then use argument init");
             return (false);
         } catch (Exception e) {
+            System.out.println(e.getMessage());
+            if (e.getMessage().contains("Access denied")) {
+                System.out.println("Database access denied");
+            }
             logger.error("Unable to start", e);
             System.out.println("Unable to start.  Check mysql and Guard log files");
             return (false);
