@@ -53,36 +53,32 @@ import java.sql.Connection;
 import java.sql.DriverManager;
 import java.sql.SQLException;
 import java.sql.Statement;
-
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
 import com.openexchange.guard.config.Config;
 import com.openexchange.guard.database.DbStrings;
-import com.openexchange.guard.database.OGInit;
 import com.openexchange.guard.database.Keymanager;
-import com.openexchange.guard.dbpool.Manager;
+import com.openexchange.guard.database.OGInit;
 import com.openexchange.guard.encr.EncrLib;
 import com.openexchange.guard.encr.GuardKeys;
-import com.openexchange.guard.server.OxDbConn;
 import com.openexchange.guard.util.CheckOx;
 
 /**
  * First time setup of OxGuard database and the tables within Will create one encrypted item in keytable for master
- * 
+ *
  * @author greg
  */
 public class OxDatabaseCreateor {
 
 	private static Logger logger = LoggerFactory.getLogger(OxDatabaseCreateor.class);
-	
-    private String mysqlIP = com.openexchange.guard.config.Config.getMysqlIP();// IP address of configdb
 
-    private String mysqlUsername = com.openexchange.guard.config.Config.getMysqlUsername();
+    private final String mysqlIP = com.openexchange.guard.config.Config.getMysqlIP();// IP address of configdb
 
-    private String mysqlPassword = com.openexchange.guard.config.Config.getMysqlPassword();
+    private final String mysqlUsername = com.openexchange.guard.config.Config.getMysqlUsername();
 
-    private String OGmysqlIP = com.openexchange.guard.config.Config.getMysqlIPOxGuard();
+    private final String mysqlPassword = com.openexchange.guard.config.Config.getMysqlPassword();
+
+    private final String OGmysqlIP = com.openexchange.guard.config.Config.getMysqlIPOxGuard();
 
     public boolean create(int version) {
 
@@ -92,14 +88,19 @@ public class OxDatabaseCreateor {
         		logger.error("Unable to establish REST API.  Please check settings and make sure the OX backend has open-xchange-rest installed");
         		return (false);
         	}
-        	
+
             Class.forName("com.mysql.jdbc.Driver").newInstance();
 
             String OGmysqlConnectionString = "jdbc:mysql://" + OGmysqlIP + "/?user=" + mysqlUsername + "&password=" + mysqlPassword;
-            String OGShardRead = Config.ogMySqlReadIP == null ? null : ("jdbc:mysql://" + Config.ogMySqlReadIP + "/");
-            String OgShardWrite = "jdbc:mysql://" + OGmysqlIP + "/";
+            String OGRead = Config.ogMySqlReadIP == null ? null : ("jdbc:mysql://" + Config.ogMySqlReadIP + "/");
+            String OgWrite = "jdbc:mysql://" + OGmysqlIP + "/";
+            if (Config.ogShardIP == null) Config.ogShardIP = OGmysqlIP;
+            String OgShardWrite = "jdbc:mysql://" + Config.ogShardIP + "/";
+            String OgShardRead = Config.ogShardReadIP == null ? null : ("jdbc:mysql://" + Config.ogShardReadIP + "/");
+            String OGShardmysqlConnectionString = "jdbc:mysql://" + Config.ogShardIP + "/?user=" + mysqlUsername + "&password=" + mysqlPassword;
             Connection intconn = DriverManager.getConnection(OGmysqlConnectionString);
             Connection configdb = DriverManager.getConnection(Config.getConfigDbConnectionString());
+            Connection shardconn = DriverManager.getConnection(OGShardmysqlConnectionString);
 
             String createDatabase = "CREATE DATABASE oxguard DEFAULT CHARACTER SET utf8 DEFAULT COLLATE utf8_general_ci";
 
@@ -115,12 +116,12 @@ public class OxDatabaseCreateor {
             }
 
             // Register database
-            RegisterDb.register(configdb, OgShardWrite, OGShardRead, "com.mysql.jdbc.Driver", mysqlUsername, mysqlPassword, "oxguard");
+            RegisterDb.register(configdb, OgWrite, OGRead, "com.mysql.jdbc.Driver", mysqlUsername, mysqlPassword, "oxguard");
 
             createDatabase = "CREATE DATABASE oxguard_1";
 
             try {
-                mysqlExecute(createDatabase, intconn);
+                mysqlExecute(createDatabase, shardconn);
             } catch (Exception ex) {
             	if (ex.getMessage().contains("database exists")) {
             		logger.info("oxguard database exists, using");
@@ -129,7 +130,7 @@ public class OxDatabaseCreateor {
 	                return (false);
             	}
             }
-            RegisterDb.register(configdb, OgShardWrite, OGShardRead, "com.mysql.jdbc.Driver", mysqlUsername, mysqlPassword, "oxguard_1");
+            RegisterDb.register(configdb, OgShardWrite, OgShardRead, "com.mysql.jdbc.Driver", mysqlUsername, mysqlPassword, "oxguard_1");
 
             intconn.close();
             OGInit.initGuardDatabase();
@@ -138,7 +139,7 @@ public class OxDatabaseCreateor {
             Connection conn = DriverManager.getConnection(OGmysqlConnectionString);
 
             // Master KeyTable
-            String createKeyTable = DbStrings.createKeyTable;
+            String createKeyTable = DbStrings.createGlobalKeyTable;
 
             try {
                 mysqlExecute(createKeyTable, conn);
@@ -150,14 +151,14 @@ public class OxDatabaseCreateor {
                     return (false);
             	}
             }
-            try {		
+            try {
                 Keymanager keyman = new Keymanager();
-                GuardKeys keys = keyman.createMasterKeysNC(-1, 0, "dont@remove.com", "master", Config.mpass, "");// Create master key for password recovery
+                GuardKeys keys = keyman.createMasterKeysNC(-1, 0, "dont@remove.com", "master", Config.mpass, "", false);// Create master key for password recovery
                 if (keys == null) {
                 	logger.error("Unable to create master keys");
                     return (false);
                 }
-                GuardKeys keys2 = keyman.createMasterKeysNC(-2, 0, "dont@remove.com", "master", Config.rpass, "");// Create master key for client encryption
+                GuardKeys keys2 = keyman.createMasterKeysNC(-2, 0, "dont@remove.com", "master", Config.rpass, "", false);// Create master key for client encryption
                 if (keys2 == null) {
                 	logger.error("Unable to create master keys for client encryption");
                     return (false);
@@ -168,7 +169,7 @@ public class OxDatabaseCreateor {
 
             }
 
-            String createOgEmail = "CREATE TABLE `og_email` (`email`  varchar(200) NULL ,`id`  int NULL ,`cid`  int NULL ,`db`  int NULL ," + " PRIMARY KEY (`email`), INDEX `uid` (`id`, `cid`));";
+            String createOgEmail = DbStrings.createOGEmail;
             try {
                 mysqlExecute(createOgEmail, conn);
             } catch (Exception ex) {
@@ -180,7 +181,7 @@ public class OxDatabaseCreateor {
             	}
             }
 
-            String sharding = "CREATE TABLE `sharding` (`current_shard`  int NULL ,`count`  int NULL);";
+            String sharding = DbStrings.createShardingTable;
             try {
                 mysqlExecute(sharding, conn);
                 sharding = "INSERT INTO sharding VALUES (1, 0)";
@@ -195,7 +196,7 @@ public class OxDatabaseCreateor {
 
             }
 
-            String createSession = "CREATE TABLE `oxguard_session` (`sessionid` varchar(40) COLLATE utf8_unicode_ci NOT NULL DEFAULT ''," + " `token` varchar(60) COLLATE utf8_unicode_ci DEFAULT NULL, `date` datetime DEFAULT NULL, `userid` int(11) NOT NULL," + " `cid` int(11) DEFAULT NULL, PRIMARY KEY (`sessionid`)) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;";
+            String createSession = DbStrings.createSessions;
 
             try {
                 mysqlExecute(createSession, conn);
@@ -210,7 +211,7 @@ public class OxDatabaseCreateor {
 
             }
 
-            String cacheTable = "CREATE TABLE `cache` (  `email` varchar(50) NOT NULL, `RSAPublic` varchar(1024) DEFAULT NULL, `date` datetime DEFAULT NULL," + " PRIMARY KEY (`email`)) ENGINE=InnoDB DEFAULT CHARSET=latin1;";
+            String cacheTable = DbStrings.createRSACache;
 
             try {
                 mysqlExecute(cacheTable, conn);
@@ -225,8 +226,7 @@ public class OxDatabaseCreateor {
 
             }
 
-            String fileCacheTable = "CREATE TABLE `FileCache` (`itemID` varchar(200) NOT NULL, `userid` int(11) NOT NULL," + "`location` varchar(255) DEFAULT NULL, `lastDate` date DEFAULT NULL, KEY `item` (`itemID`)) ENGINE=InnoDB DEFAULT CHARSET=latin1;";
-
+            String fileCacheTable = DbStrings.createFileCacheTable;
             try {
                 mysqlExecute(fileCacheTable, conn);
 
@@ -240,7 +240,7 @@ public class OxDatabaseCreateor {
 
             }
 
-            String publicPGPTable = "CREATE TABLE `PublicPGP` (  `email` varchar(50) NOT NULL DEFAULT '',  `PublicKeyRing` varchar(2048) DEFAULT NULL,  `LastUp` date DEFAULT NULL,  PRIMARY KEY (`email`)) ENGINE=InnoDB DEFAULT CHARSET=latin1;";
+            String publicPGPTable = DbStrings.createPGPPublic;
             try {
                 mysqlExecute(publicPGPTable, conn);
             } catch (Exception ex) {
@@ -252,7 +252,7 @@ public class OxDatabaseCreateor {
             	}
 
             }
-            String badTable = "CREATE TABLE `Bad` (`id`  varchar(100) NOT NULL , `count`  int NULL ,`last`  datetime NULL ,	PRIMARY KEY (`id`));";
+            String badTable = DbStrings.createBadTable;
             try {
                 mysqlExecute(badTable, conn);
             } catch (Exception ex) {
@@ -265,7 +265,7 @@ public class OxDatabaseCreateor {
 
             }
 
-            String createIOX = "CREATE TABLE `iox` (`domain` varchar(50) NOT NULL DEFAULT '',  `url` varchar(500) DEFAULT NULL, `token` varchar(50) DEFAULT NULL," + " `lastcheck` date DEFAULT NULL,  PRIMARY KEY (`domain`)) ENGINE=InnoDB DEFAULT CHARSET=latin1;";
+            String createIOX = DbStrings.createIOXTable;
             try {
                 mysqlExecute(createIOX, conn);
             } catch (Exception ex) {
@@ -278,7 +278,7 @@ public class OxDatabaseCreateor {
 
 
             }
-            String rsacache = "CREATE TABLE `key_cache` (  `id` int(11) NOT NULL AUTO_INCREMENT,  `privkey` varchar(5000) DEFAULT NULL," + "PRIMARY KEY (`id`)) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=latin1;";
+            String rsacache = DbStrings.createKeyCache;
             try {
                 mysqlExecute(rsacache, conn);
             } catch (Exception ex) {
@@ -289,8 +289,44 @@ public class OxDatabaseCreateor {
                     return (false);
             	}
             }
-            
-            String settingsTable = "CREATE TABLE `global_settings` (  `key` varchar(20) NOT NULL DEFAULT '', `value` varchar(250) DEFAULT NULL," + " PRIMARY KEY (`key`)) ENGINE=InnoDB DEFAULT CHARSET=latin1;";
+
+            String remoteKeys = DbStrings.createRemoteKeys;
+            try {
+                mysqlExecute(remoteKeys, conn);
+            } catch (Exception ex) {
+                if (ex.getMessage().contains("already exists")) {
+                    logger.info("remote_keys exists, using");
+                } else {
+                    logger.error("Unable to create remote_keys keytable", ex);
+                    return (false);
+                }
+            }
+
+            String remoteKeysCache = DbStrings.createRemoteKeyCache;
+            try {
+                mysqlExecute(remoteKeysCache, conn);
+            } catch (Exception ex) {
+                if (ex.getMessage().contains("already exists")) {
+                    logger.info("remote_key_cache exists, using");
+                } else {
+                    logger.error("Unable to create remote_key_cache keytable", ex);
+                    return (false);
+                }
+            }
+
+            String deletedKeys = DbStrings.createDeletedKeys;
+            try {
+                mysqlExecute(deletedKeys, conn);
+            } catch (Exception ex) {
+                if (ex.getMessage().contains("already exists")) {
+                    logger.info("deletedkeys table exists, using");
+                } else {
+                    logger.error("Unable to create deletedkeys table", ex);
+                    return (false);
+                }
+            }
+
+            String settingsTable = DbStrings.createGlobalSettings;
             try {
                 mysqlExecute(settingsTable, conn);
                 String setVersion = "INSERT INTO global_settings VALUES ('version','" + version + "');";
@@ -306,8 +342,20 @@ public class OxDatabaseCreateor {
             	}
 
             }
-            
-            
+
+            String PGPLookup = DbStrings.createPGPKeysLookupTable;
+            try {
+                mysqlExecute(PGPLookup, conn);
+            } catch (Exception ex) {
+            	if (ex.getMessage().contains("already exists")) {
+            		logger.info("PGPKey Table exists, using");
+            	} else {
+            		logger.error("Unable to create PGPKeyTable keytable", ex);
+                    return (false);
+            	}
+            }
+
+
    //         try {
    //             Manager.createTables(conn);
    //         } catch (Exception ex) {
