@@ -51,10 +51,10 @@ package com.openexchange.guard.upseller;
 
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
-
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
+import com.openexchange.guard.config.OxConfiguration;
+import com.openexchange.guard.logging.LogAction;
 import com.openexchange.guard.ox.Languages;
 import com.openexchange.guard.util.Core;
 
@@ -66,9 +66,16 @@ public class Upsell {
 		try {
 		int id = Core.getIntParameter(request, "id", true);
 		int cid = Core.getIntParameter(request, "cid", true);
+		LogAction.setUser(id, cid);
 		String lang = Core.getStringParameter(request, "lang");
 		// UPSELL LOGIC HERE
-		String templ = Languages.getTranslation("upsell.html", lang, cid);
+		int templid = 0;
+		try {
+		    templid = OxConfiguration.getTemplateId(cid, id);
+		} catch (Exception e) {
+		    logger.error("Problem getting template ID for upsell");
+		}
+		String templ = Languages.getTranslation("upsell.html", lang, templid);
 		if (templ != null) {
 			templ = templ.replace("%id", Integer.toString(id)).replace("%cid", Integer.toString(cid));
 		} else templ = "";
