@@ -55,11 +55,12 @@ import java.io.InputStreamReader;
 import java.io.OutputStream;
 import java.io.StringWriter;
 import java.io.UnsupportedEncodingException;
+import java.math.BigInteger;
 import java.security.PrivateKey;
-import java.security.PublicKey;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
-
+import javax.mail.internet.AddressException;
+import javax.mail.internet.InternetAddress;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import javax.xml.transform.Transformer;
@@ -67,12 +68,9 @@ import javax.xml.transform.TransformerException;
 import javax.xml.transform.TransformerFactory;
 import javax.xml.transform.dom.DOMSource;
 import javax.xml.transform.stream.StreamResult;
-
-import org.apache.http.HttpResponse;
 import org.apache.http.client.methods.CloseableHttpResponse;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
 import com.google.gson.Gson;
 import com.google.gson.JsonObject;
 import com.google.gson.JsonParser;
@@ -81,21 +79,23 @@ import com.openexchange.guard.database.Access;
 import com.openexchange.guard.encr.EncrLib;
 import com.openexchange.guard.encr.GuardKeys;
 import com.openexchange.guard.exceptions.GuardMissingParameter;
+import com.openexchange.guard.logging.LogAction;
 
 
 public class Core {
-	
+
 	private static Logger logger = LoggerFactory.getLogger(Core.class);
 	private static PrivateKey passwordKey = null;
 	private static String pubkey = null;
 
     /**
      * Send answer to response stream
-     * 
+     *
      * @param response
      * @param answer String to put in HTML
      */
     public static void sendOK(HttpServletResponse response, String answer) {
+        LogAction.setSuccess();
         response.setContentType("text/html; charset=UTF-8");
         response.addHeader("Access-Control-Allow-Origin", "*");
         response.setStatus(HttpServletResponse.SC_OK);
@@ -106,9 +106,26 @@ public class Core {
         }
     }
 
+    /**
+     * Send JSON response
+     * @param response
+     * @param json
+     */
+    public static void sendJson(HttpServletResponse response, JsonObject json) {
+        LogAction.setSuccess();
+        response.setContentType("application/json; charset=UTF-8");
+        response.addHeader("Access-Control-Allow-Origin", "*");
+        response.setStatus(HttpServletResponse.SC_OK);
+        try {
+            response.getWriter().write(json.toString());
+        } catch (IOException e) {
+            logger.error("Error sending OK message", e);
+        }
+    }
+
     /**
      * Send answer to response stream
-     * 
+     *
      * @param response
      * @param answer String to put in HTML
      */
@@ -120,6 +137,7 @@ public class Core {
         try {
             OutputStream out = response.getOutputStream();
             out.write(answer);
+            out.flush();
             out.close();
         } catch (IOException e) {
         	logger.error("Error sending OK message", e);
@@ -127,6 +145,7 @@ public class Core {
     }
 
     public static void sendFail(HttpServletResponse response, String answer) {
+        LogAction.setFail();
         response.setContentType("text/html");
         response.addHeader("Access-Control-Allow-Origin", "*");
         response.setStatus(HttpServletResponse.SC_NOT_ACCEPTABLE);
@@ -137,7 +156,16 @@ public class Core {
         }
 
     }
-    
+
+    public static void sendAnswer(HttpServletResponse response, int returnCode, String answer)
+        throws IOException {
+
+        response.setContentType("text/html");
+        response.addHeader("Access-Control-Allow-Origin", "*");
+        response.setStatus(returnCode);
+        response.getWriter().println(answer);
+    }
+
     public static void sendError(HttpServletResponse response, String answer) {
         response.setContentType("text/html");
         response.addHeader("Access-Control-Allow-Origin", "*");
@@ -150,7 +178,12 @@ public class Core {
 
     }
 
-    public static JsonObject getJSON(HttpServletRequest request) {
+    /**
+     * Get the string contents of a post request
+     * @param request
+     * @return
+     */
+    public static String getPostBody (HttpServletRequest request) {
         StringBuffer sb = new StringBuffer();
         String line = null;
         try {
@@ -161,12 +194,26 @@ public class Core {
             reader.close();
         } catch (Exception e) {
             logger.error("Error reading from request", e);
-        } 
+        }
+        if (sb.length() == 0) {
+            return null;
+        }
+        return (sb.toString());
+    }
+
+    /**
+     * Get the json object from post request
+     * @param request
+     * @return
+     */
+    public static JsonObject getJSON(HttpServletRequest request) {
         JsonParser jsonParser = new JsonParser();
-        JsonObject json = (JsonObject) jsonParser.parse(sb.toString());
+        String postBody = getPostBody(request);
+        if (postBody == null) return(null);
+        JsonObject json = (JsonObject) jsonParser.parse(postBody);
         return (checkEpass(json));
     }
-    
+
     public static JsonObject getJSON (String data) {
     	JsonParser jsonParser = new JsonParser();
         JsonObject json = (JsonObject) jsonParser.parse(data);
@@ -175,27 +222,34 @@ public class Core {
 
     /**
      * Checks for error in reply string
-     * 
+     *
      * @param response
      * @return true if contains error
      */
     public static boolean checkFail(String response) {
-        if (response == null) return(true);
+        if (response == null) {
+            return(true);
+        }
         if (response.contains("\"data\":null")) {
         	logger.debug("No error");
-        } else logger.debug(response);
+        } else {
+            logger.debug(response);
+        }
         if (response.contains("error")) {
             logger.info(response);
             return (true);
         }
         return (false);
     }
-    
+
     public static boolean checkFail(JsonObject resp) {
+    	if (resp == null) {
+            return(false);
+        }
     	Gson gson = new Gson();
     	return (checkFail(gson.toJson(resp)));
     }
-    
+
     public static String getError (String response) {
     	try {
     		int i = response.indexOf("{");
@@ -205,8 +259,9 @@ public class Core {
     		JsonObject resp = (JsonObject)parser.parse(response);
     		if (resp.has("error_desc")) {
     			return (resp.get("error_desc").getAsString());
-    		}
-    		else return ("");
+    		} else {
+                return ("");
+            }
     	} catch (Exception ex) {
     		return ("");
     	}
@@ -227,21 +282,22 @@ public class Core {
     }
 
     public static String dbformat(String data) {
-        if (data == null)
+        if (data == null) {
             return (data);
-        data = data.replace("'", "\'");
-        data = data.replace("`", "\'");
+        }
+        data = data.replace("'", "''");
+        data = data.replace("`", "``");
         data = data.replace("\\", "\\\\");
         return (data);
     }
-    
+
     public static String gettime (long start) {
     	return (((System.nanoTime() - start) / 1000000) + "ms");
     }
-    
+
     /**
      * Format response into a JSON object
-     * 
+     *
      * @param response
      * @return
      * @throws UnsupportedEncodingException
@@ -256,7 +312,7 @@ public class Core {
         response.close();
         return (result);
     }
-    
+
     public static String getXMLString(org.w3c.dom.Document doc) throws TransformerException {
         StringWriter writer = new StringWriter();
         StreamResult result = new StreamResult(writer);
@@ -270,20 +326,29 @@ public class Core {
         }
         return (XML);
     }
-    
+
+    /**
+     * Parse JSOn and decode all encrypted passwords
+     * @param result
+     * @return
+     */
     private static JsonObject checkEpass (JsonObject result) {
-    	if (result == null) return(null);
+    	if (result == null) {
+            return(null);
+        }
     	result = decr(result, "e_password", "password");
     	result = decr(result, "e_encr_password", "encr_password");
     	result = decr(result, "e_ox_password", "ox_password");
     	result = decr(result, "e_extrapass", "extrapass");
         return (result);
     }
-    
+
     private static JsonObject decr(JsonObject result, String key, String newkey) {
     	if (result.has(key)) {
     		String encr = result.get(key).getAsString();
-    		if (passwordKey == null) loadPrivate();
+    		if (passwordKey == null) {
+                loadPrivate();
+            }
         	if (passwordKey != null) {
         		EncrLib encrlib = new EncrLib();
         		String decoded = encrlib.decryptRSA(encr, passwordKey);
@@ -294,7 +359,18 @@ public class Core {
     	}
         return(result);
     }
-    
+
+    /**
+     * Decode RSA encrypted passwords sent from UI
+     * @param epass
+     * @return
+     */
+    public static String decodeRSApassword (String epass) {
+        EncrLib encrlib = new EncrLib();
+        if (passwordKey == null) loadPrivate();
+        return(encrlib.decryptRSA(epass, passwordKey));
+    }
+
     private static void loadPrivate () {
     	try {
 	    	Access acc = new Access();
@@ -304,16 +380,18 @@ public class Core {
 	    	}
     	} catch (Exception ex) {
     		logger.error("Problem loading password RSA key", ex);
-    		
+
     	}
-    	
+
     }
-    
+
     public static PrivateKey getPasswordKey () {
-    	if (passwordKey == null) loadPrivate();
+    	if (passwordKey == null) {
+            loadPrivate();
+        }
     	return (passwordKey);
     }
-    
+
     private static void loadPublic() {
     	Access acc = new Access();
     	GuardKeys passwordKeys;
@@ -327,16 +405,28 @@ public class Core {
 		}
 
     }
-    
+
     public static String getPublicKey() {
-    	if (pubkey == null) loadPublic();
+    	if (pubkey == null) {
+            loadPublic();
+        }
     	return (pubkey);
     }
-    
+
     public static String getStringParameter (HttpServletRequest request, String name) throws GuardMissingParameter {
     	return (getStringParameter(request, name, false));
     }
-    
+
+    public static boolean getBooleanParameter (HttpServletRequest request, String name) throws GuardMissingParameter {
+    	String var = getStringParameter(request, name, false);
+        if (var!= null) {  // If sending inline
+        	if (var.toLowerCase().contains("true")) {
+        		return(true);
+        	}
+        }
+        return (false);
+    }
+
     /**
      * Get String from HttpServletRequest
      * @param request
@@ -352,11 +442,11 @@ public class Core {
     	}
     	return (item);
     }
-    
+
     public static int getIntParameter (HttpServletRequest request, String name) throws GuardMissingParameter {
     	return (getIntParameter (request, name, false));
     }
-    
+
     /**
      * Get int from HttpServletRequest
      * @param request
@@ -367,10 +457,12 @@ public class Core {
      */
     public static int getIntParameter (HttpServletRequest request, String name, boolean mandatory) throws GuardMissingParameter {
     	String item = request.getParameter(name);
-    	if (item == null) {
+    	if (item == null || item.equals("null")) {
     		if (mandatory) {
     			throw new com.openexchange.guard.exceptions.GuardMissingParameter("Missing parameter " + name);
-    		} else return(0);
+    		} else {
+                return(0);
+            }
     	}
     	try {
     		int val = Integer.parseInt(item);
@@ -380,11 +472,11 @@ public class Core {
     		throw new com.openexchange.guard.exceptions.GuardMissingParameter("Bad integer value for parameter " + name);
     	}
     }
-    
+
     public static String getStringFromJson (JsonObject json, String name) throws GuardMissingParameter {
     	return (getStringFromJson (json, name, false));
     }
-    
+
     /**
      * Get String from JsonObject
      * @param json
@@ -397,8 +489,9 @@ public class Core {
     	if (!json.has(name)) {
     		if (mandatory) {
     			throw new com.openexchange.guard.exceptions.GuardMissingParameter("Missing JSON parameter " + name);
-    		}
-    		else return(null);
+    		} else {
+                return(null);
+            }
     	}
     	try {
     		String dat = json.get(name).getAsString();
@@ -408,11 +501,32 @@ public class Core {
     		throw new com.openexchange.guard.exceptions.GuardMissingParameter("Problem getting string from json for parameter " + name);
     	}
     }
-    
+
     public static int getIntFromJson (JsonObject json, String name) throws GuardMissingParameter {
     	return (getIntFromJson (json, name, false));
     }
     
+    public static boolean getBooleanFromJson (JsonObject json, String name) throws GuardMissingParameter {
+        return (getBooleanFromJson (json, name, false));
+    }
+    
+    public static boolean getBooleanFromJson (JsonObject json, String name, boolean mandatory) throws GuardMissingParameter {
+        if (!json.has(name)) {
+            if (mandatory) {
+                throw new com.openexchange.guard.exceptions.GuardMissingParameter("Missing JSON parameter " + name);
+            } else {
+                return(false);
+            }
+        }
+        try {
+            boolean dat = json.get(name).getAsBoolean();
+            return (dat);
+        } catch (Exception ex) {
+            logger.error("Problem getting string parameter " + name + "from json" , ex);
+            throw new com.openexchange.guard.exceptions.GuardMissingParameter("Problem getting string from json for parameter " + name);
+        }
+    }
+
     /**
      * Get int from JsonObject
      * @param json
@@ -425,8 +539,9 @@ public class Core {
     	if (!json.has(name)) {
     		if (mandatory) {
     			throw new com.openexchange.guard.exceptions.GuardMissingParameter("Missing JSON parameter " + name);
-    		}
-    		else return(0);
+    		} else {
+                return(0);
+            }
     	}
     	try {
     		int dat = json.get(name).getAsInt();
@@ -436,7 +551,7 @@ public class Core {
     		throw new com.openexchange.guard.exceptions.GuardMissingParameter("Problem getting int from json for parameter " + name);
     	}
     }
-    
+
     public static String getErrorDescr(String error) {
     	try {
     		if (error.contains("{")) {
@@ -451,12 +566,77 @@ public class Core {
     	}
     	return ("");
     }
-    
+
     public static String getIP (HttpServletRequest request) {
     	if (request.getHeader("X-Forwarded-For") != null) {
     		return (request.getHeader("X-Forwarded-For"));
     	}
-    	if (request.getRemoteAddr().equals("127.0.0.1")) return(null);
+    	if (request.getRemoteAddr().equals("127.0.0.1")) {
+            return(null);
+        }
     	return (request.getRemoteAddr());
     }
+
+    public static boolean unified (String emailid) {
+    	if (emailid.contains("/")) {
+    		return (true);
+    	}
+    	return (false);
+    }
+
+    public static int getUnifiedEmailId (String emailid) {
+    	int i = emailid.lastIndexOf("/");
+		return (Integer.parseInt(emailid.substring(i+1)));
+    }
+
+    public static String getUnifiedFolder (String emailid) {
+    	int i = emailid.lastIndexOf("/");
+		return(emailid.substring(0, i));
+    }
+
+    /**
+     * Return the hex representation of PGP Key ID
+     * @param id
+     * @return
+     */
+    public static String PGPKeyId (long id) {
+    	return (Long.toHexString(id).substring(8));
+    }
+
+    public static String PGPKeyLongId (long id) {
+    	return (Long.toHexString(id));
+    }
+
+    public static Long getLongPGP (String id) {
+        return (new BigInteger(id, 16).longValue());
+    }
+
+
+    public static String getHex (String id) {
+    	try {
+    		return (Long.toHexString(Long.parseLong(id)).substring(8));
+    	} catch (Exception ex) {
+    		logger.error("Error parsing long string " + id, ex);
+    		return (null);
+    	}
+    }
+
+    public static String getLongHex (String id) {
+    	try {
+    		return (Long.toHexString(Long.parseLong(id)));
+    	} catch (Exception ex) {
+    		logger.error("Error parsing long string " + id, ex);
+    		return (null);
+    	}
+    }
+    
+    public static InternetAddress parseEmailAdress (String address) throws AddressException, UnsupportedEncodingException {
+        InternetAddress[] recips = InternetAddress.parse(address, false);
+        if (recips.length > 0) {
+            return (new InternetAddress(recips[0].getAddress(), recips[0].getPersonal(), "UTF-8"));
+        }
+        return (null);
+        
+    }
+
 }
