@@ -145,7 +145,7 @@ public class EmailValidator {
      * @return
      */
     public boolean testSMTP() {
-        if (location == "")
+        if (location.equals(""))
             validate();
         TelnetClient tc = new TelnetClient();
         try {
