@@ -57,18 +57,16 @@ import org.apache.http.protocol.HttpContext;
 import org.apache.http.util.EntityUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
 import com.google.gson.Gson;
 import com.google.gson.JsonArray;
 import com.google.gson.JsonObject;
 import com.google.gson.JsonParser;
 import com.openexchange.guard.config.Config;
-import com.openexchange.guard.database.OGInit;
-import com.openexchange.guard.server.OxDbConn;
+import com.openexchange.guard.server.connectionPooling.HttpConnectionPoolService;
 import com.openexchange.guard.util.Core;
 
 public class Normalizer {
-	
+
 	private static Logger logger = LoggerFactory.getLogger(Normalizer.class);
 
 	public static String normalize (String data)
@@ -83,7 +81,31 @@ public class Normalizer {
 			return(data);
 		}
 	}
-	
+
+	/**
+	 * Send string to backend for normalization.  Primarily for PGP Inline
+	 * @param data
+	 * @return
+	 * @throws Exception
+	 */
+	public static String normalizeString (String data) {
+	    try {
+    	    JsonObject content = new JsonObject();
+            content.addProperty("content", data);
+            JsonObject newcontent = putRequest (content);
+            String emailContent = newcontent.get("content").getAsString();
+            return (emailContent);
+	    } catch (Exception e) {
+	        logger.error("Problem normalizing string ", e);
+	        return (data);
+	    }
+	}
+
+	/**
+	 * Send email JSON to OX backend for normalization (strip scripts, etc)
+	 * @param json
+	 * @return
+	 */
 	public static JsonObject normalize (JsonObject json) {
 		if (json.has("attachments")) {
 			JsonArray attachments = json.get("attachments").getAsJsonArray();
@@ -105,7 +127,9 @@ public class Normalizer {
 								if (type.contains("plain")) {  // If plaintext, save index in case no HTML
 									plainTextIndex = i;
 								} else {
-									if (type.contains("html") || (type.contains("alternative"))) hasHTML = true;
+									if (type.contains("html") || (type.contains("alternative"))) {
+                                        hasHTML = true;
+                                    }
 								}
 							}
 							catch (Exception e) {
@@ -120,7 +144,10 @@ public class Normalizer {
 					newattach.add(attach);
 				}
 			}
-			if (!hasHTML) {  // If no HTMl available, convert the plaintext to HTML for display
+		/** Currently disable HTML conversion.  Done in UI
+		 *
+		  	if (!hasHTML) {  // If no HTMl available, convert the plaintext to HTML for display
+
 				JsonObject attach = attachments.get(plainTextIndex).getAsJsonObject();
 				String emailContent = attach.get("content").getAsString();
 				emailContent = plainTextConvert (emailContent);
@@ -129,19 +156,21 @@ public class Normalizer {
 				attach.remove("content");
 				attach.addProperty("content", emailContent);
 			}
+			**/
+
 		}
 		return (json);
 	}
-	
+
 	private static String plainTextConvert (String content)
 	{
 		content = content.replace("<body>", "").replace("</body>", "").replace("<", "&lt;").replace(">", "&gt;").replace("\n", "<br/>").replace("\"", "&quot;");
 		return (content);
 	}
-	
+
 	 /**
      * Actual send of data to OX Backend
-     * 
+     *
      * @param data String of JSON data to send
      * @param database The ID of the database
      * @param cid ContextId if pertinent. 0 if not
@@ -150,9 +179,9 @@ public class Normalizer {
      * @throws Exception
      */
     private static JsonObject putRequest(JsonObject content) throws Exception {
-    	Gson gson = new Gson();
     	String data = content.toString();
-        CloseableHttpClient httpClient = OxDbConn.httpClient;
+        //CloseableHttpClient httpClient = OxDbConn.httpClient;
+    	CloseableHttpClient httpClient  = HttpConnectionPoolService.getClient();
         HttpContext context = HttpClientContext.create();
         String url = "http" + (Config.backend_ssl ? "s" : "") + "://" + Config.getOxBackend() + ":" + Config.ox_backend_port + "/preliminary/htmlproc/v1/sanitize";
         HttpPut putRequest = new HttpPut(url);
@@ -160,6 +189,7 @@ public class Normalizer {
         entity.setContentType("text/javascript;charset=UTF-8");
         putRequest.addHeader("accept", "application/json");
         putRequest.addHeader("Authorization", "Basic " + Config.getBasicAuthenticationEncoding());
+        putRequest.setHeader("User-Agent", Config.apiUserAgent);
         putRequest.setEntity(entity);
         CloseableHttpResponse response = httpClient.execute(putRequest, context);
         JsonObject result = null;
